{% extends "!layout.html" %}

{%- block sidebar1 %}{{ sidebar() }}{%- endblock %}
{%- block sidebar2 %}{%- endblock %}

{% block sidebarlogo %}
<div class="logo">
  <a href="http://www.pyside.org">
    <img alt="PySide" src="{{ pathto('_static/pysidelogo.png', 1) }}"/>
  </a>
</div>
{% endblock %}

{%- block header %}
<div id="container">
  <div class="header">
    <div class="header_container">
      <div class="related">
        <ul>
          {%- block rootrellink %}
          <li><a href="{{ pathto( 'index' ) }}">{{ shorttitle|e }}</a></li>
          {%- endblock %}
          {%- for parent in parents %}
          <li>{{ reldelim1 }} <a href="{{ parent.link|e }}" {% if loop.last %}{{ accesskey("U") }}{% endif %}>{{ parent.title }}</a></li>
          {%- endfor %}
          {%- block relbaritems %} {% endblock %}
        </ul>
      </div>
    </div>
  </div>
{%- endblock -%}

{%- block document %}
  <div class="documentwrapper">
    <div class="bodywrapper">
      <div class="body" role="main">
        {% block body %} {% endblock %}
      </div>
    </div>
  </div>
{% endblock %}

{%- block footer %}
    <div class="footer">
    <a href="https://www.qt.io/"><img src="{{ pathto('_static/logo_qt.png', 1) }}" alt="Qt" border="0" /></a>
    <a href="http://www.python.org"><img src="{{ pathto('_static/logo_python.jpg', 1) }}" alt="Python" border="0" /></a>
    <p>{{ copyright }}</p>
    </div>
</div>
{%- endblock %}

{%- block relbar1 %}{%- endblock %}
{%- block relbar2 %}{%- endblock %}
