# Resource object code (Python 3)
# Created by: object code
# Created by: The Resource Compiler for Qt version 6.10.0
# WARNING! All changes made in this file will be lost!

from PySide6 import QtCore

qt_resource_data = b"\
\x00\x00\x04\xe8\
/\
/ Copyright (C) \
2021 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aColumn {\x0a\
    id: root\x0a   \
 spacing: 8\x0a\x0a   \
 Label {\x0a       \
 text: \x22Total nu\
mber of screens:\
 \x22 + screenInfo.\
count\x0a        fo\
nt.bold: true\x0a  \
  }\x0a\x0a    Flow {\x0a\
        spacing:\
 12\x0a        widt\
h: parent.width\x0a\
\x0a        Repeate\
r {\x0a            \
id: screenInfo\x0a \
           model\
: (Qt.applicatio\
n as Application\
).screens\x0a      \
      Label {\x0a  \
              re\
quired property \
string name\x0a    \
            requ\
ired property in\
t virtualX\x0a     \
           requi\
red property int\
 virtualY\x0a      \
          requir\
ed property var \
modelData // avo\
id shadowing Lab\
el.width and hei\
ght\x0a\x0a           \
     lineHeight:\
 1.5\x0a           \
     text: name \
+ \x22\x5cn\x22 + virtual\
X + \x22, \x22 + virtu\
alY + \x22 \x22 + mode\
lData.width + \x22x\
\x22 + modelData.he\
ight\x0a           \
 }\x0a        }\x0a   \
 }\x0a\x0a    Componen\
t.onCompleted: {\
\x0a        var scr\
eens = (Qt.appli\
cation as Applic\
ation).screens;\x0a\
        for (var\
 i = 0; i < scre\
ens.length; ++i)\
\x0a            con\
sole.log(\x22screen\
 \x22 + screens[i].\
name + \x22 has geo\
metry \x22 +\x0a      \
                \
  screens[i].vir\
tualX + \x22, \x22 + s\
creens[i].virtua\
lY + \x22 \x22 +\x0a     \
                \
   screens[i].wi\
dth + \x22x\x22 + scre\
ens[i].height)\x0a \
   }\x0a}\x0a\
\x00\x00\x05\x01\
(\
\xb5/\xfd`\xff\x16\xbd'\x00\x06p\x912`\x8d:\
0\x0c\xc3\xc0\x161\x1c\x00\x98G\xc1\xb5\xb6\x93$\xda\
IN\x11E,I\x22\xd5#\xdck\xe3\x5c\x9d/Y\
\xc2BlU\xb2\xcdF\x1e\x12Xb\x833\xad\x1c\x83\
\x00~\x00\x85\x00\x15\xc9\xe1\xec\x8b\xb7\xd4\x8d&Lf\
+\xd1\xec\xe70\xcf\xcb\xcc\x85\x18\xaetL\xb7\xf4\x94\
1\xa7\xd8\xbb\xd3\xad`\xef\xeeN \xfb\x8f7@\x13\
\xdf\x99\xa6*\x9d\xfet\x8b\xe2x\x9du\xbe\xb3O\x1d\
\xbbz\x9a\xb8fNZ\x96\x8d\xa5\xc1\xc3\xb0\x15\x13+\
]?\xca\x97\xe5;\x96\x10WlE\xb9C\xfd\x93!\
\x84\x85\xeb\xe4\xd7\x11\x16\x8c\x06\x0a\xcfD\xc1\x8f\xd8\xd2\
t\xd8\xe5\xd6~g9~6\xa3\xc1h\xb6bQ\xf6\
\xa3\xa8i\x09sk/\x97\x1c\x8f\xa3\x059c\x01.\
\x17\x11\x80M\x80\xcc\xf1ZZ\xd3\xa3\xa7m\xb5\x8da\
s\xda*,KP\xd7\x1eQ_\xf9I\x1e\xab\xffg\
\x0c\x9b'\x82M\x1fD\xb6\x91\x16N\x7f\x5c\x97\xe3\xbf\
\x07\xff\x82\xacd\x90\x8b\xa8k\xd9\xcd^\xf6BJ~\
\xb5\xaa\xb3E\xd4]?\x99J\xd9!\xaaR\x8fo\xba\
ye\xd2\xad\xd6\xb6b\xf9\xfd$\x8a\xffY:l+\
\xea\x1c\x8e\x9a\x15Xj`/\xbe_K\xecjJ\xfa\
\xdb*>l\xd2\x06_\xf9\xd30\x98\x02\x83\x8c&\xa0\
\x15h0\x9a\x8d\x08\xc8\xd6\x05\x10\x9aY\x19\x96\xa2\xe9\
\xfeqE\xa5\xba\xb8p\xbam\x03[\x99M\xd9\x81m\
[{I>9f\x16!l\x8e\xab\xa1\x0b\xa1\x91\xda\
N\x9a=\xd3\x8a\xab{\x1b\xdf7\xa9VF\x14\xd1l\
\xfa\x1e\xff\xec\xf4\xfe7\xb3\x90\x87\x83\xfc\xd1W\xcf\x96\
\xb0h\xaf\xdf]\xfe\xd9B\xf3\xcf\x5c\x9b\xcc\xf4yV\
\xca\xcca\x13\x12\x12\xb6\xf4-\x9b3\xe9F\x1c\xdfN\
\x87\xb3\x19\xa8\x0b\x0fg\xd0\xf0pP\x93H\xe6|\xb5\
'V'\xb1Z;\xfeV(\x5c]\xbe\x91\x92\x10\xcb\
\xad\xa8\x7fe\xda(\xf4S;VA\xba\x11\xe5\x9f\x10\
P:\x07\xce\x85Z\x99Rp\xcd\x9ce\xe9k\xc9I\
\x86\x90)\x8f<b\xfd\x91\xee\x9ffBrR)p\
F\x1f\xe40\xa74\xb4\xc9\x14\x1cP\x98\x9d\x084{\
\xd9\x0a\xe5r8\x1d\xa5\xe2;\x89\x1dX\x03\x0d\x85\xa1\
8\x90\x88&`\xd3)K8\x1d\x07s\x1f\xcf\x84\xa0\
\xfc\x93\xdf\xf4\xb39\x0e!\xd4w\xb6\xaa\x22'C\xf1\
\x03\x81 \xa8\x11\xa6\x19\x1a\x11\x19IR\x94\x0c\x07A\
\x08\x820\x14uV\x9a\x07Rx$IC\x18\xc3 \
\x14\x8d\x10)`\x06\x11D\x08\x99HH\x82\x0bd\x92\
4\xd5\x01\xfd\x86\xe3\x00;U\xd9[%\xb0+\xa0H\
Y\x22\xa8\xc45\x02LtZ\xa2\x1f\xf5yiy\xf2\
1\xe7\xffg\xce\x8eR\xb6\x88\xc1\xb6\x88m\xaf\xd4)\
\xd7\xa7\x00\xb5\x15Y,,MT\xd3\x84\xcc\xb2\xd8\xca\
\xa0\xd5&S\x9b&\x97\x05\xed\x06\x92\xfc]\xe6-i\
q\x1ckF\x85\xbc(\x08\xa3\x9c\xbd\x00\xb3~1Z\
\xf8\xd0\xa6d\xb1\x8a\x8f\xd7{\x9b\x97/=\xf0F\xf8\
\xb4L\xefw\xce\xcd\xd0\xdb\xe9\xc9\xc7\xa8\x08\xdd\x97\xb9\
\xfc\x035B\xc9\xfa\xcdKA\x83\xd7)\x8e\xdf\xb0\xd4\
\x04F\xe2\xe7\xca\xcf<v\xa5D\xa0\xbd\xc9x\x87E\
R\xb3@\x8cTv\x10\x04+\xc8\xd8\xc35\x07r\x18\
VCyf\xa6\x08q\x03\x10\x07<\xefF\xbd\xff\xf9\
\xff\x84\x9d\xfb\xa7\x88\xff\x0c\x0d\xed\xed\xb4\xd8\xa2\xe22\
\x10\xac\xd3PA\xe2\xf8\x17\x13;4\x9e\xfd1\x92\x11\
\xb5\xf6\x228t\xd6\x8c&\x87\xaf\xd0\xc3\xc3\xba\xa6\xa1\
\xb8\xb9N\xb9/\xf7\x05~\x22\xa5d,\xc1b\xd1#\
%U\x05\xfe\xc3\x14T\xbc0\xc9\xe7\x80PZ\xea_\
\xdd\x9eO@1\xf8\x10.\xf7\x8c\x1fe\xc0\xacHQ\
<\xb6\x026\xee\xcb\x81] 4\x0e%\xbe,\x0f\x87\
Jk\x91XN9\xb1\x14$\xa0\xad\xd8I\xc1Uy\
S\xb4\xae\xd4F\xf7X3\x92\x9e!E\xca$\xe7x\
\xaazGx\x0eG(\x8e\x0cmij\x0aL\xd3\xfd\
\xc0\x0a\xe4\xf3BKz\xa02G\xf6\x10\xca\x04D\x1b\
\xc9\x86\xe0\xa7^\xb5Dz@\xcd\x9c\xfa\xd3-\x97\x10\
a5\xb1\xba\x97D\x86\xb1\x0f\xea\x0c\x95\xf4tU\x1f\
\x93\xf7\xacR\x9c\xb6\x00\x83\xf4\xefK\xa5\xf8]\x97\x80\
'\xa2\x04\x17\xeeu\xf43\x1f\x81\xe0C\xee\xf4\xf7\x85\
7\xda17\x00\x1a\x14]\xdb\xb6\xad\x1c\x0d8\x12q\
\x9c:\x0c1\x22\xf3\xd1\x09Vz\x916\x01h\xd9\x88\
\x8b\xbf\xb5\xa3\xb84\xe2\xcd\xe7b\xf1\xac\xad\x9cR\x96\
\xe3\x11\x97\x92\x93\xa3\xc1\x87\xf2:\xf2y\x0b\xb0\xca\xbd\
\xdd \xc7t?D\xaf\x16-\x05a\xd2\xbd9 \xb3\
\x9fRI\xaf\xbc\x99\x1cC\x0eK?\x9e\xdc\x86\xc3$\
\xfe)\xab-\x01v\xbb\x1e\x8e\xdd\xfbm\x12\xc8\x1b\xf5\
\xce\x1fym\xd8\x10\x0e\xe5\xd5w\xd1\x05m\x14`\xd5\
\x0bAo)\xb0c,\x81'H\x81.xsAP\
=&\xe5\xa0Z\xe0\x828\xa6\xdc\xde\xa0\xeb\x8e\xb1F\
\xcb\xc0D6_,\x04\xa8y\xe0\xdf\x95\x9f\xfb\xe1\x97\
\xa3I\xa6\x1f#k)\xf4\xaf\x03m\xd7\xc8AP\x12\
]\x154_\xbd\x1cU\x0dcv\xb8\xbf\x1a\xc0\xa8\x02\
\
\x00\x00\x03\xd5\
/\
/ Copyright (C) \
2021 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port shared\x0a\x0a//!\
 [splash-propert\
ies]\x0aWindow {\x0a  \
  id: splash\x0a   \
 color: \x22transpa\
rent\x22\x0a    title:\
 \x22Splash Window\x22\
\x0a    modality: Q\
t.ApplicationMod\
al\x0a    flags: Qt\
.SplashScreen\x0a  \
  property int t\
imeoutInterval: \
2000\x0a    signal \
timeout\x0a//! [spl\
ash-properties]\x0a\
//! [screen-prop\
erties]\x0a    x: (\
Screen.width - s\
plashImage.width\
) / 2\x0a    y: (Sc\
reen.height - sp\
lashImage.height\
) / 2\x0a//! [scree\
n-properties]\x0a  \
  width: splashI\
mage.width\x0a    h\
eight: splashIma\
ge.height\x0a\x0a    I\
mage {\x0a        i\
d: splashImage\x0a \
       source: I\
mages.qtLogo\x0a   \
     TapHandler \
{\x0a            on\
Tapped: splash.e\
xit()\x0a        }\x0a\
    }\x0a\x0a    funct\
ion exit() {\x0a   \
     splash.visi\
ble = false\x0a    \
    splash.timeo\
ut()\x0a    }\x0a\x0a    \
//! [timer]\x0a    \
Timer {\x0a        \
interval: splash\
.timeoutInterval\
; running: splas\
h.visible; repea\
t: false\x0a       \
 onTriggered: sp\
lash.exit()\x0a    \
}\x0a    //! [timer\
]\x0a}\x0a\
\x00\x00\x03\x03\
(\
\xb5/\xfd`d\x09\xcd\x17\x00fbg\x1f\x00\xdb6\
\x8e\x19\x94\xa4Zd_\xd9sQ1d\xb4\x22\x92\x81\
P\xe2a+\xcaH\xcc\xa5\xff\xed?\x94`\x00\x5c\x00\
^\x00\xddtf\xe6\x0c\xc6^?&\xcb@j\xefW\
;\xd9V\xd2\xc3\xcew\xf8\xaf\xfd<\xcf\xb2\x06\xc3H\
'\x06\x85\x81\xf2\x91\xb0g\xc34\xba\xbc\xed\x19\xbec\
@\xb5\x87\xbb\xdb\x8e\xf2\xaf=1\x86\x06kK\xcf\x18\
D\x14\xce'\xdb\xe6g\xf8U\xe3\xe7\xd9\x09\x7f\x87[\
\x05bD\xe2\x90`\xc1$\xc0\xafJ\xf9}}v\xc2\
\x1c\x17\xb6\xaej\x98\xc5\xd8g\xffmV\xc5\xd0\xfe\xa2\
\x04g\x8d\x99\xf7x\xff\xda\x87\xf6i\x5co\xf8}9\
\xeb6\xaf\x1b=\xbf\x9c\x0d\x8c\x92U\xf2\xbd\x05\x0al\
 U\xfd\xfe\xc7\x82\xe2\xd7\xf6k\xb9Z\x7fYCT\
\xd6@0\x81/\xef\xd2\xe96\xfdZ^\x7f\xfb\xb3\xf7\
\xe3\xaeU\x87\xc3\x82;\xd5Oo\xbd\xb5\x97\x03\xb5=\
\x9f\x9a\x93\x17V\xa6\xd1o\x83\xb4\xa7\x8dq\x07\xe1\x18\
k\xb7o\xf82\xa2\xc2\x1a\xbd\xec6\xc7q\x8c\x11\x0f\
D\x87\xf7\xb9\x9b\xd4\x9e\xb6\xaf\x16\xa5uZIc\x02\
\xb7\x17\xeb\xc6\x04\xd6\xb4\x12\x7f4\xee\xde\xd5O\xfb\x14\
3\xa6\xda}%Hb\x00\xc7\xbd:\x97\x83\xbbTu\
\x19\xbf\x04w`\xf5Bj\xa7\xc7j+Y\xec\x80\x98\
}\xaf\xf54M\xd5\xf8\xb2/{\xde\x01\xb1\xe1\xed\xad\
\xbd#.\x88\xaa;h/\xa1\xc8g\x7f\x05\xc6\xf0\xaf\
\xb3\xb8\xe9)R\x8f\xac\xe8\xb1\xfe\xf1\xb6tVO\x0e\
\xd3(=e\xa9@%\xab\x0b\x16\x1b#\xc1\x03\x0e\x04\
\xd8\xf38\xf0\xeddA\x220\xdb\xe5\x15\xfb\x1b\xe5\xf5\
\x93i\xbb\xfcs\x81\xd5:\x18\x80\x93\xa0\xa1\xa532\
C3\x92$)\xa5\x03 \x84\x10\xc4\xdcq\x07\xe9\x80\
\xaa\xec\xf5}\xa7\x0a~\xe6v j\x95\xc9\x9a\x9f\x93\
\xb1\x1fxx\x8dL\x93\x8d$fa.\xaf\x96kF\
\x7fRr.\x00r\x00\xedF2\x85\xee\x9e\x1a\x02H\
\xfc\x150\x0f\x05'x\xbb\x5c\x19\xc8p\xef\xa2\xad\xb2\
'W\xec]a^\x5cb\x0f\x22\xcb\xd4\x0d+\x15\x8c\
\xe4lK\x86\x11\xea$T\x9a'\xf2\xae\xe7BL\xee\
\xc5k\xac!\xbf8\x8d5D\xb0\x0e\xed\xd7\x91\x83\xbb\
\x880\xa8mr\xbb\xc2\xd9\x0cO+\xd34\x80\xc9\xbc\
o\x14Y\x0b\x883O\x10\xfc\x88]p>\x8cr\x8d\
T\xf0k\xf8\x0bj\xef?\xd0r\xb57\x1d\x07(\xe7\
\xee{h\x13\x8b\xc1\xb6\x90\x92\xf9\x8a\xe5\x95)\xd9\x22\
2\xc6\xc9e\xe7i\xb0R\x1f\x18W\x95lPV#\
}Mgl\xb3\xb8\xa7\xd8EF\x22\xaa\xe40P8\
\xf1\xaf\x0f\x82\x0dt\xe6\xa4\x01G@\x07\xa5;^\x0c\
v\x14\xe4\x90k*\x5c\xfe\x9b\x14o\xd6@9\xd6\xa2\
\x7f\xb0\x85\x1f\xe7yV2\xe4+\xf4\xcc1S<\x98\
\x01\x9e<Q]\xa5R\xc8\xc4\x84\x06\x22!\x8f\x0f$\
\xcc\xec\x19pP\xe8\xed\xc2\x83\xa3\xc1H.\xd7\x80\x85\
\x91K\xe8\xdc\x9cW\xf2V\x8a\x15\x02\xbeH,\xe0<\
\x16\x8c\xbbSP\xfb_\xd8)}\x00u\xcc(5\xe0\
U\x01\
"

qt_resource_name = b"\
\x00\x06\
\x07\xe0Kg\
\x00w\
\x00i\x00n\x00d\x00o\x00w\
\x00\x0e\
\x0ei\x17\x5c\
\x00A\
\x00l\x00l\x00S\x00c\x00r\x00e\x00e\x00n\x00s\x00.\x00q\x00m\x00l\
\x00\x0a\
\x0bi\x98\xbc\
\x00w\
\x00i\x00n\x00d\x00o\x00w\x00.\x00q\x00m\x00l\
\x00\x0a\
\x08\x90\x1a|\
\x00S\
\x00p\x00l\x00a\x00s\x00h\x00.\x00q\x00m\x00l\
\x00\x11\
\x02YG\x1c\
\x00C\
\x00u\x00r\x00r\x00e\x00n\x00t\x00S\x00c\x00r\x00e\x00e\x00n\x00.\x00q\x00m\x00l\
\
"

qt_resource_struct = b"\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x01\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x04\x00\x00\x00\x02\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00h\x00\x04\x00\x00\x00\x01\x00\x00\x0d\xca\
\x00\x00\x01\x975l\xc7\xec\
\x00\x00\x00N\x00\x00\x00\x00\x00\x01\x00\x00\x09\xf1\
\x00\x00\x01\x998\xf9\x98B\
\x00\x00\x004\x00\x04\x00\x00\x00\x01\x00\x00\x04\xec\
\x00\x00\x01\x998\xf9\x98B\
\x00\x00\x00\x12\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\
\x00\x00\x01\x975l\xc7\xec\
"

def qInitResources():
    QtCore.qRegisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

def qCleanupResources():
    QtCore.qUnregisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

qInitResources()
