// Copyright (C) 2025 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#ifndef INVISIBLENAMESPACE_H
#define INVISIBLENAMESPACE_H

#include "libminimalmacros.h"

namespace InvisibleNamespace::VisibleNamespace {
    struct ValueType {
        int i = 0;
    };
}

#endif
