.text-center {
  text-align: center !important;
}

.text-center img {
    padding-top: 10px;
    height: 70px !important;
}

.cover-img img {
  object-fit: cover;
  height: 50%;
}

/* Tables */
.section .docutils.container td {
    float:left;
}

table.docutils {
    margin-right: auto;
    margin-bottom: 10px;
    border: none;
    width: initial;
    font-size: 0.8em;
}

table.docutils.colwidths-given td {
    float: none;
}

table.docutils th,
table.docutils td {
    padding-left:0;
    border: none;
}

table.docutils td ul {
    margin:0
}

table.docutils td ul > li {
    margin: 0 0 0.5em;
}

.hide {
    display: none;
}

.fixed .container {
  max-width:1280px;
  margin:0 auto;
  padding:0 3.9%; /* 0? */
  position:relative;
  overflow:visible
}

/* We cannot put a :download:`....` command inside
 * a sphinx-design button, so we add some properties from the button
 * to the download class to mimic it */
code.download {
  text-align: center;
  color: var(--color-brand-primary);
  display: block;
  border-color: transparent;
  background-color: transparent;
  border: 1px solid var(--color-brand-primary) !important;
  border-radius: 0.25rem;
  font-size: 1rem;
  font-weight: 400;
  vertical-align: middle;
  padding: .375rem .75rem;
  user-select: none;
  line-height: 1.5;
  transition: color .15s ease-in-out, background-color .15s ease-in-out, border-color .15s ease-in-out, box-shadow .15s ease-in-out;
}

code.download:hover {
  color: white;
  background-color: var(--color-brand-primary);
  border-color: var(--color-brand-primary);
  text-decoration: none;
  padding: .375rem .75rem;
}

dl[class]:not(.option-list):not(.field-list):not(.footnote):not(.glossary):not(.simple):first-child > dt {
  font-size: +2.25rem;
  font-weight: 700;
  color: #ff00dd;
}

.theme-toggle svg{
  width: +1.25rem;
  height: +2.25rem;
}

.sd-card-title code span {
  font-size: +1rem;
  color: var(--color-brand-primary);
}

article[role="main"] section[id^="pyside6-"] h1 {
    font-size: 1.5em;
}

article[role="main"] section[id^="pyside6-"] dl dt em span.pre{
    font-size: 1.5em;
}

article[role="main"] section[id^="pyside6-"] dl dt span.pre {
    font-size: 1.5em;
}

.prominent-summary {
    font-size: 1em;
    font-weight: bold;
    color: #2a9d8f;
    background-color: #f0f0f0;
    padding: 5px;
    border-radius: 5px;
}

section#examples .sd-card {
  transition: transform 0.2s, box-shadow 0.2s;
}

section#examples .sd-card-title {
  font-size: 1.25em;
  font-weight: bold;
}

section#examples .sd-card-body{
  font-size: 1em;
}

/* Example card hover */
section#examples .cover-img img:hover {
    background-color: var(--color-background);
    opacity: 0.3;
}

section#examples .sd-card {
    z-index: 1;
    position: relative;
}

section#examples .sd-card-hover:hover {
    background-color: var(--color-background);
    transform: translateY(-5px);
    box-shadow: 0 4px 8px rgba(27, 31, 35, 0.12), 0 12px 48px rgba(27, 31, 35, 0.24);
}

section#examples .sd-card-hover:hover .sd-card-footer {
    bottom: 0;
    display: block;
    left: 0;
    position: absolute;
    color: var(--color-background-font);
    right: 0;
    top: 0;
    padding: 20px;
    opacity: 1;
    z-index: 5;
    background-color: var(--color-background);
}

section#examples .sd-card-hover:hover a {
    z-index: 7;
}

section#examples details .sd-card .sd-card-footer {
    display: none;
}

section#examples details .sd-card-body p strong {
    display: block;
    font-size: 1.5em;
}

section#examples details .sd-card-title {
    font-size: 1.1em;
}
