// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef PYSIDE_QUICK_REGISTER_TYPE_H
#define PYSIDE_QUICK_REGISTER_TYPE_H

#include <sbkpython.h>

namespace PySide
{
void initQuickSupport(PyObject *module);
}

#endif // PYSIDE_QUICK_REGISTER_TYPE_H
