// Copyright (C) 2022 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#ifndef SMART_TEST_H
#define SMART_TEST_H

namespace Test {

enum DummyEnum { Dummy1, Dummy2 };

}

#endif // SMART_TEST_H
