#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QBluetoothLocalDevice'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths  # noqa: E402
init_test_paths(False)

from PySide6.QtBluetooth import QBluetoothLocalDevice  # noqa: E402


class QBluetoothLocalDeviceTest(unittest.TestCase):
    def testInitialization(self):
        device = QBluetoothLocalDevice()  # noqa: F841


if __name__ == '__main__':
    unittest.main()
