#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QObject.sender()'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QCoreApplication, QObject, QTimer, Signal, Slot
from helper.usesqapplication import UsesQApplication

import samenamesender


class ExtQTimer(QTimer):
    def __init__(self):
        super().__init__()


class Sender(QObject):
    foo = Signal()


class SameNameSender(samenamesender.SameNameSender):
    ''' Test sender class for SameNameSenderTest (PYSIDE-3201).'''
    signal3 = Signal()

    def __init__(self, parent=None):
        super().__init__(parent)
        self.signal1.connect(self.slot2)
        self.signal2.connect(self.slot2)
        self.signal3.connect(self.slot3)
        self.slot3_invoked = False

    @Slot()
    def slot1(self):
        pass

    @Slot()
    def slot2(self):
        pass

    @Slot()
    def slot3(self):
        self.slot3_invoked = True


class Receiver(QObject):
    def __init__(self):
        super().__init__()
        self.the_sender = None

    def callback(self):
        self.the_sender = self.sender()
        if QCoreApplication.instance():
            QCoreApplication.instance().exit()


class ResultReceiver(QObject):
    def __init__(self):
        super().__init__()

    @Slot(result=int)
    def slotWithResult(self):
        return 3


class ObjectSenderTest(unittest.TestCase):
    '''Test case for QObject.sender() method.'''

    def testSenderPythonSignal(self):
        sender = Sender()
        recv = Receiver()
        sender.foo.connect(recv.callback)
        sender.foo.emit()
        self.assertEqual(sender, recv.the_sender)


class ObjectSenderCheckOnReceiverTest(unittest.TestCase):
    '''Test case for QObject.sender() method, this one tests the equality on the Receiver object.'''

    def testSenderPythonSignal(self):
        sender = Sender()
        recv = Receiver()
        sender.foo.connect(recv.callback)
        sender.foo.emit()
        self.assertEqual(sender, recv.the_sender)


class ObjectSenderWithQAppTest(UsesQApplication):
    '''Test case for QObject.sender() method with QApplication.'''

    def testSenderCppSignal(self):
        sender = QTimer()
        sender.setObjectName('foo')
        recv = Receiver()
        sender.timeout.connect(recv.callback)
        sender.start(10)
        self.app.exec()
        self.assertEqual(sender, recv.the_sender)

    def testSenderCppSignalSingleShotTimer(self):
        recv = Receiver()
        QTimer.singleShot(10, recv.callback)
        self.app.exec()
        self.assertTrue(isinstance(recv.the_sender, QObject))

    def testSenderCppSignalSingleShotTimerWithContext(self):
        recv = Receiver()
        QTimer.singleShot(10, recv, recv.callback)
        self.app.exec()
        self.assertTrue(isinstance(recv.the_sender, QObject))

    def testSenderCppSignalWithPythonExtendedClass(self):
        sender = ExtQTimer()
        recv = Receiver()
        sender.timeout.connect(recv.callback)
        sender.start(10)
        self.app.exec()
        self.assertEqual(sender, recv.the_sender)


class ObjectSenderWithQAppCheckOnReceiverTest(UsesQApplication):
    '''Test case for QObject.sender() method with QApplication.'''

    def testSenderCppSignal(self):
        sender = QTimer()
        sender.setObjectName('foo')
        recv = Receiver()
        sender.timeout.connect(recv.callback)
        sender.start(10)
        self.app.exec()
        self.assertEqual(sender, recv.the_sender)

    def testSenderCppSignalWithPythonExtendedClass(self):
        sender = ExtQTimer()
        recv = Receiver()
        sender.timeout.connect(recv.callback)
        sender.start(10)
        self.app.exec()
        self.assertEqual(sender, recv.the_sender)


class SameNameSenderTest(UsesQApplication):
    '''PYSIDE-3201: Test whether the meta object system is confused by identical
       class names.'''
    def test(self):
        sender = SameNameSender()
        sender.signal1.emit()
        sender.signal2.emit()
        sender.signal3.emit()
        self.assertTrue(sender.slot3_invoked)


class ResultSlotTest(UsesQApplication):
    '''PYSIDE-3266: Test that a slot declaring a result type does not cause crashes
       in signal connections due to args[0] == 0.'''
    def test(self):
        sender = Sender()
        recv = ResultReceiver()
        sender.foo.connect(recv.slotWithResult)
        sender.foo.emit()


if __name__ == '__main__':
    unittest.main()
