// Copyright (C) 2020 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#ifndef PYSIDETEST_GLOBAL_H
#define PYSIDETEST_GLOBAL_H

// PySide global.h file
#include "connection.h"
#include "containertest.h"
#include "testobject.h"
#include "testview.h"
#include "flagstest.h"
#include "hiddenobject.h"
#include "sharedpointertestbench.h"
#include "testqvariantenum.h"

#endif // PYSIDETEST_GLOBAL_H
