// Copyright (C) 2024 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef WEBENGINEFRAME_H
#define WEBENGINEFRAME_H

#include <QtCore/QtClassHelperMacros>

QT_FORWARD_DECLARE_CLASS(QWebEngineFrame)

QT_BEGIN_NAMESPACE

QWebEngineFrame defaultConstructedWebEngineFrame();

QT_END_NAMESPACE

#endif // WEBENGINEFRAME_H
