// Copyright (C) 2022 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#ifndef CUSTOMCONVERSION_TYPEDEFS_H
#define CUSTOMCONVERSION_TYPEDEFS_H

#include <QtCore/qlist.h>

#include <memory>

class CustomConversion;
using CustomConversionPtr = std::shared_ptr<CustomConversion>;

#endif // CUSTOMCONVERSION_TYPEDEFS_H
