# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

''' Test bug 243: http://bugs.openbossa.org/show_bug.cgi?id=243'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QApplication, QMainWindow, QLayout


class QAppPresence(unittest.TestCase):

    def testBug(self):
        app = QApplication(sys.argv)  # noqa: F841
        window = QMainWindow()
        layout = window.layout()
        self.assertTrue(isinstance(layout, QLayout))


if __name__ == '__main__':
    unittest.main()
