// Copyright (C) 2022 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef PYSIDEQMLNAMEDELEMENT_P_H
#define PYSIDEQMLNAMEDELEMENT_P_H

#include <sbkpython.h>

void initQmlNamedElement(PyObject *module);

#endif // PYSIDEQMLNAMEDELEMENT_P_H
