// Copyright (C) 2019 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#ifndef SMART_H
#define SMART_H

#include "smart_sharedptr.h"
#include "smart_integer.h"
#include "smart_obj.h"
#include "smart_registry.h"
#include "smart_test.h"
#include "stdsharedptrtestbench.h"
#include "stdoptionaltestbench.h"
#include "stduniqueptrtestbench.h"

#endif // SMART_H
