// This is a placeholder file for shiboken, because shiboken expects a header file listing the
// header files whose bindings are to be created. This file is empty because shibokenmodule
// functions are implemented directly inside the typesystem_shiboken.xml file through CPython code.
