// Copyright (C) 2021 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef PYSIDEQMLLISTPROPERTY_H
#define PYSIDEQMLLISTPROPERTY_H

#include <sbkpython.h>

namespace PySide::Qml {
void initQtQmlListProperty(PyObject *module);
}

#endif // PYSIDEQMLLISTPROPERTY_H
