# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Unit test for Location'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths  # noqa: E402
init_test_paths(False)

from PySide6.QtLocation import QGeoServiceProvider  # noqa: E402


class QLocationTestCase(unittest.TestCase):
    def test(self):
        geoServiceProvider = QGeoServiceProvider("none")
        self.assertEqual(geoServiceProvider.errorString(),
                         'The geoservices provider none is not supported.')


if __name__ == "__main__":
    unittest.main()
