# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)
from PySide6.QtCore import Qt
from PySide6.QtWidgets import QApplication, QSlider


class TestBug736 (unittest.TestCase):

    def testIt(self):
        app = QApplication([])  # noqa: F841
        slider = QSlider(Qt.Orientation.Horizontal)
        slider2 = QSlider(Qt.Orientation.Horizontal)

        slider2.setMaximum(10)
        slider.valueChanged[int].connect(slider2.setMaximum)
        slider.valueChanged[int].emit(100)
        self.assertEqual(slider2.maximum(), 100)


if __name__ == '__main__':
    unittest.main()
