# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject


class MyBaseObject(QObject):
    def __init__(self, number):
        self.myMember = 'myMember' + number
        super().__init__()


class QObjectConstructorTest(unittest.TestCase):
    def testBug(self):
        for i in range(10):
            number = str(i)
            obj = MyBaseObject(number)
            self.assertEqual(obj.myMember, 'myMember' + number)


if __name__ == '__main__':
    unittest.main()
