# Copyright (C) 2025 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from PySide6.QtWidgets import QLabel


class LabelInFolder(QLabel):
    def __init__(self):
        super().__init__()
        self.setText("Label in folder")
