// Copyright (C) 2020 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#include "flagstest.h"

namespace FlagsNamespace
{
    ClassForEnum::ClassForEnum(FlagsNamespace::Options) {}

    ClassForEnum::~ClassForEnum() = default;
}
