// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#include "../samplebinding/global.h"
#include "extendsnoimplicitconversion.h"
#include "number.h"
#include "otherderived.h"
#include "otherobjecttype.h"
#include "othermultiplederived.h"
#include "othertypesystypedef.h"
#include "smartptrtester.h"
