# Resource object code (Python 3)
# Created by: object code
# Created by: The Resource Compiler for Qt version 6.7.0
# WARNING! All changes made in this file will be lost!

from PySide6 import QtCore

qt_resource_data = b"\
\x00\x00\x02\xe2\
\x00\
\x00\x07\xc4x\x9c\xb5T]k\xd4@\x14\x9d\xed\xa6]\
\x9b\xd6\xda\x0f\xd07\x19\xe9\xcb\x16%\xa6\xb5\x15q\xdd\
\xfa\xb0U)\x14Z\xdbR\x84e\x091\x9bM\x07\xb2\
\xc9\x92\x8fE)\x05\xdf}\xf7w\xf8'\xfcO\xbe\x88\
\xde;s\xd3\xcc\xa6[*\x88\x03\xb3\x999s\xef=\
g\xceM\x9616\xcf\x183`\xd6`.\xb11s\
\x98\xc7b\x16\xc2L\x00\xa9\xc3d\xbfi\xb0)\x03\xd3\
V\xd9\x00\x82]\x16\xb0\x8e\x96j\xdc\x96\xfa\xafc\x8e\
\x9e}M\xcd\xf7\xfa\xccz\x03U\x99\xec\x0e[#t\
\x85\xcdH\x9d\x0b\x90\x82\xcfw\x07'\x07V\x9a\xf5\xad\
\xed\x1d\x1b\xcf\x97\xd45\xe5\xd9=\xd6\x90\xcaq\x0e]\
\x11!\x8e\x16-\xc2\x5c\x868\xc4\x1bX\x03V\x88?\
$\x0e\xdc/j\xfb5\xe0\x9c\x81\xe7#\xc8\xc1\xea\xb8\
\xbe\x0f+\x14\xcda>\x80\xf8\x06]\xc2\x90\x98\x01\x8a\
\x95\xe1\x88\xb7h?O\x18\xc6\x9b\x14_\xa7\xf8\x05\xd2\
lR\xfc\x02\xe9D\xec1\xec\xe7\xe4\xddp|y\xfd\
\x9c\xcdJ\x0d\x06\xe9\xc3\x1a?\x01\x99\x85g\x9bj\xdf\
\xa5\xfc\xf7\x80\xceI/\x14\xc64l\x99\xb0\x9a\x86\xad\
\x10\x86\xf5\x8f@=\xea_\xa5\xfce:G\x1d\xbb\xc0\
:Og\xbf\xa0\xc2\x0bY\x85\x19\xe4s\x8dzY#\
\xca\xaf\xebc?IE\x1c\xf1M\xdb6G\x89\xef\x09\
\xb9;\x17\xc1\xf9\x88\x0f\xc2\xd8\xcdZ\xd7`\x11\x01h\
\x8e\xdd\xe4\xb3\x88\x02>\xf6\xbdg|\xecxq\x18'\
\x08\xc7\xa2\xcf\x91\xad\xb9a^\x98\x1cF\x10:o\x13\
7\xd8s3\xb7k\xf7x\x1b3\xb6\x9b\x94\xf1\x84o\
Z\xf6F\xcb\xbc4\xcd\x8a\xccO\xe5+\xc7\x9cR\xe6\
\x96\xfd\xff\xa9\xbfi\xd4\xa2\xa4\xde\x01\xea8\xcfd\x11\
>\x80\xba\x1dE,\xa2\xdb\x95\x5c\x85\xff\x95\x08\xec\xf2\
V\xa9\xa1\xf6#\xcd\xdcLx\xaa!\x13\xdc\xfa\x81\xae\
 \xcd\x92\xdc\xcb\xf8\xc9\xd1\xfe\xf1\x99\xd3I\xe24u\
\xf6\xa3Q\x9e\x15r&\xe2\xf9K~\xfa\xe6C\xe7\xf0\
\xf0x\xcf\x06)\xd3\xd3\x0f\xf3\xac\x92\xaf\x09\x81\x0a'\
g\xce\xa9\x9b\x04~F%\xa4\x07\x18\xe0\xdch\x84\xaa\
2a\xc5@yq\x9dX\xd9y\xed>\x1c\x0c\x08|\
G\xe0\xba\xa0(n\xd5\xd6\x0f\xad+o\x0a\x15\xa4K\
\x01S\x08Ur,7*HG\xac\x89\x8b\x94\x0d\xc1\
\xb8\xc4\xcf\xf2$\xaa\x14\xb8\xd4\x1bl\xd0\xdf@\xd1\xe0\
\xfdu\x11ya\xde\xf7\xf9\xab\xa1\x9f\xb9\xa1\x03\x7f\x9d\
\xa1\xf8\xb8k\x96x*\x86\xfd\xa7\xf8c\x9d\xef\x9af\
\x9e\xe2\x07\x10\xb9C?\x1d\xb9\x9e\xcfeV\xd98\xa4\
\xb1\x91\xfa\xa6~u\xbb\xd2\x8d\xa6\xbd\xd1\xebM\xb4\x5c\
e\x8ah\xfa\x8b\xd2\xed\xe6\xa9\x9f4\xc3\xd8\x8b\xcaL\
\xac:\xf4#\x8dU\xad\x9aE-\xf8\xc7\x80\xcc\xa2\x19\
\xbd^\xd1\xa82\x1eg\x9b_\x5c*\xff\xe2\xaa\xbd\xea\
=\x11\x91U\xf9j4\xb3c\xe51\x9a<+\xeb\xd2\
gT\xf9\x94\xe4\xa8\xb8_\xb6\xe1\xa6\xb3?Y\xbd\xa1\
\xc8\
\x00\x00\x04C\
\x00\
\x00\x0c\xf4x\x9c\xddW[o\x1bE\x14\x1e_\x92\x92\
-\xb9\xb4i\x93\x16\xda2\xa9%dCe\x9c4A\
\xa8&\x11\xa8 Z\xa9R\xaa\x06UHV\xb4Z\xaf\
\xd7\xe9\xc2^\xac\xddY7\xa8\xf2\x8f\xe0\x85\xbf\xc3o\
\xe0o\xf0\xc8\x13/U9g\xe6\xcc\xee\xec\xda\xa1*\
\x0f<0\xd2d\xe7\x5c\xe6|\xe7|sf\x920\xc6\
V\x98\x1au\x98\x16sY\xcc\x02\x98\x09H\x0d\x98\xb5\
74\xd8\x82\xb1\xc1&\xe0\x9a2\x9f\x09\x981\x8b@\
\xd7D\xc3?m\xaa\xc1\x5ccSf\xcfa\xbdu\xdb\
2\x1b\xb2\x8c\x8d\xe5\xcaf{\xec\x00V_\xcd\xf9\x84\
\x10|\x02\xab\xcb\xa45=6\x16\x85~\x97Q\xa3o\
\x93\xbe\xcb\xf4\x1d\x15.\x8d\x17\x8dz\xeb\x12\xbaZ\xec\
=\xd6\x22\xed\x15 \xb8&\xb3Z\x96\xdf\xef\x9e\x9c<\
\xe9\xa6b\xd4\xdd?\xe8\xa1}MQ m\xebt(\
\x08\x12:>\xd2*5X\xd1&\xcc;\xb8\x1f\xac+\
\xb4f$_&\x19c<bK\x80\xa9\x86fB\xeb\
j\xa43\xfd\xea\xa4k\xc8X\x8d\x5c\xf7\x08\xe2n\xd3\
\xfe%\xf2\xd7r\xcb\x88\xabuP\xb8d\x19clS\
\x0c\xcc\xed&|\xef\x1a\xb9\xa2\xbcc\xc8w\x8cZ6\
aW]\xda\x1b2\x1f\x5co\xc1\x0a\xc9\xe60\xb7\xc1\
\xff\x12\x91\xdf\x90\xba&0\xad\xd6\xa8\xef\x93\xbcb\xd8\
-\xaaW\xdb-\x83\x03\x8c\xb7J\xf1\x90\xf3\xeb\xf0s\
\x8d\xb0p|J\xf2:\xf9\xdf\x02y\x83\xfc\xd7e\xde\
\x8aC\x1dc\x830\xaf\x12\xfe\x15\xc2\xbcJ\xe7\xd7 \
\x8ck\x84Q#\x8ckR\xaf\xc6\x0d\x90\xb7(fS\
b(>\xb7(\xf6\x0d\xe2e\x9bb\xa3|\x93th\
\xff \xe7M\xc9\xb7\x09g\x95\xfco\xb3\xa2W\xd0\xbe\
Cy\xa1\xfds\xa8\xa7\xce\x8a&G\xfd_\xa0\xc1\xf3\
?$\xee\xdf'\xfe\x8e\xc0\xbaB\xf2\xd7\xe0\x81\xb8\x1f\
R.\xd7\xc9\x7fKr\xa6\xf4\x87\xc4\xf5G\x84\xff\x0b\
\xecY%\x1en\x91\x1e\xe3\xecP\xbflR\x1c\xc4\xb9\
K~\xaf!\xeb/\xd4Ul\xd2\xfd@a\xc4\x8a\xfb\
\xf9{k\xea%\xa9\x1fG|\xb7\xd7\xb3\xacT$\x99\
+\xf80\x1b[\xaf,\x0e#t\xc4>\x0f\xa7\x93\xbe\
5\xeb[V\x16\xf9\xe38\x09\xd1\xce\xed\xbd\x03\xd0L\
\x9d\xe4g?:\xe3S\xcf\xbd\xcf\xa7\xb6\x1b\x07q\xd2\
\xb7\x1c!\x12\x7f\x98\x09O\xe9\x17i\xf7\xf9$N}\
\x01\xc8\x18%\xf6G\x1c3lw\x08\x97B\xf1C\xbd\
\x19\x95g\x81\xfd\x946\x81\x01\x12\xe8Bf\xfc\x13#\
\xd2\xcc\xb2*\xc5\x9e\xb3b\x18\xc5\xee\xfd/\x8b\xfd\xd5\
(\xf6\xb7\xa2\xd8\x83\x7fSl\x9c\x89J\xa1~T\xaa\
P\x89\xffEix\xc7\xf6\x8a\xca\xea\x7f\xb8\x90\xe6\xd8\
Kd\xb6\x0fx\xe2\x9d\xf9\xa9\xf0\x92\xf6\xb0\xa7a\x93\
\xf8\xa5\x1d:?\x02\xf08\x88\xa1\xd0\xf3}\x84\xb1\x11\
\xe6\x01\x9f8\xeeO\xf1x\x9cz\xa2\xed\xf6:\xaa~\
\xe0\xc7\x11\xbeK\xeef\x86\xfd\x92\xc9\xa0\xa3\xac^\xa0\
,QC\xfc\x9f<}\xfc\xec\xb9\xfd0\x89\xd3\xd4~\
\x1cM2A\x09W6@\x96\xdf\x7f\xfb\xc3\xc3\xe3\xe3\
g\xdf\xf4\xfa\x85\x03\xc1\x18\xd6]\x95\xfe\x82\xe8\xc7\x99\
\xa8\x84\xcfs\xbf z\xa9l\xf09yn\x900\xd3\
\xc7\x0b]%\xecw=\xe30\x0b\xda\xba\xb6{\xfa(\
:\xf2\x9c\xe7sV\x1d4\xc7\x14\x07n\xcf<\xdb\xc7\
\xb5\x86\xd6\xc0\x86\xa9k$1)20=\x8ac\x91\
\xe9\x17\x05)\xc5\x82\x8c\xd4\xeeX\x0a\xca\xc9\xd4t\xcb\
\xc5\x96\x9ag\xce\xb7`+\xef$\xd9\xb1\x9e\xc8\x92\xa8\
\x0243oA\x93~{\xd0\xa8\xfd\xd9\xf2#7\xc8\
F\x1e\xff2\xf4\x84\x13\xd8\xf0WR\xe0\x0f\x8f\xacB\
\x9f\xfa\xe1\xe83\xfc\xd1}q\x04w<\xc5\xc7+r\
B/\x85+\xe0q\xb9\xab\xbf\xe0i\xc8\xefL\xfe<\
\x90\x0bf\xd2\xc3\xec\x16\xb7\xd5`\x90\xa5p\x0d\x83\xd8\
\x8dz\x9d\xd3\xd3\x0b[k0\xd0'\x80Ns\xf1\xfd\
\xe8\x82K1\x18\xe4\xafj\xbb\x02\xa0o\x86\xe9\xb2\xdb\
\xd1\xe1\xf1\x88\xbd\xf3\x22}\xb5jk8\xeecl\xdd\
!\xa7\xa7\xf7 X\x04b$I\xf9\x18\xfb\x15\xec\xea\
\xcd\x91\xc0\x9d\xfc\x0d\xd5\x01q\x1e\xf2W3\x95R\x5c\
:g?2{2\x9e\xeb\x97\xe2\x01\x04\xcfroR\
W\xc4\xaa\x19\xb0\x1b\x96$&=\x8a\x95\x871\xff\xad\
\xce\xe8\xbf\x83J\xcb\xcc\xd9\xb5\xae\xea'\xc7\xdf%\xe1\
\xb4\xa2\
"

qt_resource_name = b"\
\x00\x0d\
\x06E\xc5\xd3\
\x00s\
\x00h\x00a\x00d\x00e\x00r\x00_\x00a\x00s\x00s\x00e\x00t\x00s\
\x00\x0e\
\x04\x16\xeb\xb2\
\x00c\
\x00o\x00l\x00o\x00r\x00.\x00f\x00r\x00a\x00g\x00.\x00q\x00s\x00b\
\x00\x0e\
\x00\xfb\xe9\x92\
\x00c\
\x00o\x00l\x00o\x00r\x00.\x00v\x00e\x00r\x00t\x00.\x00q\x00s\x00b\
"

qt_resource_struct = b"\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x01\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x02\x00\x00\x00\x02\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00B\x00\x00\x00\x00\x00\x01\x00\x00\x02\xe6\
\x00\x00\x01\x8a!\x0c\xa5\xeb\
\x00\x00\x00 \x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\
\x00\x00\x01\x8a!\x0c\xa5\xeb\
"

def qInitResources():
    QtCore.qRegisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

def qCleanupResources():
    QtCore.qUnregisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

qInitResources()
