// Copyright (C) 2025 Ford Motor Company
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef PYSIDEREMOTEOBJECTS_H
#define PYSIDEREMOTEOBJECTS_H

#include <sbkpython.h>

namespace PySide::RemoteObjects
{

void init(PyObject *module);

} // namespace PySide::RemoteObjects

#endif // PYSIDEREMOTEOBJECTS_H
