# Resource object code (Python 3)
# Created by: object code
# Created by: The Resource Compiler for Qt version 6.10.2
# WARNING! All changes made in this file will be lost!

from PySide6 import QtCore

qt_resource_data = b"\
\x00\x00\x0b\xe1\
\x00\
\x005Ex\x9c\xbd\x1b]W\xdb:\xf2=\xbfB7\
\xfb\x12n\x1b\xa7\xd0m\x1f\xd2\xed\xde\x03\xa1\x1f\x9c\xa5\
K\x0b\xf4v\xcf\xe9\xe9\x83b+\x89\x16\xc72\x92\x0c\
\xa4-\xff}gd\xcb\xf1\x87d\xc0\xf4\xae\xce)\xb1\
\xad\xd1\xcch4\x9a/\xa9\x93\x09\x99\x89t#\xf9r\
\xa5\xc9h\xb6C\xf6\x9e\xed\xed\x92\xf3\x15#\x9f4\xf4\
\xacS\x9al\xc8\xb1\x8e\x82\xc1dB\xce>\x1e\xfeg\
|\xccC\x96(6>\x8aX\xa2\xf9\x8239%\xc5\
\xb7S\xb6\x18\x7f\xd2c\x18\xb6f2\xe44&'\xa7\
\xe4\xe0\xecp\xfc|<\x8bi\xa6\xd8`\x90J\xba\x5c\
S\x83Y$\x80\xe0\x80\xad\xe8\x15\x17\x80\xe3@dI\
4\x18p\xe8\x90\x1a\xa8\xcf\x84d\xdb\xb7O\x19\x0f/\
\x1a\xaf\xc11\xdd\x88L\xab\xe6\xe7\x99H\xb4\x14\xb1*\
\x91\x0d\x83!\xa1\x8a\xec\xa7\xe9`\x00\x7fb\x1eR\xcd\
E\xf2\x85'\x91\xb8&?\x06\x04\x1a\x8f\xa6\xe4\xda|\
0\xaf\xd7<\xd2\xab)y\xfe\xf2\x99y]1\x14\xd0\
\x94\xbc\xd8\xcb\xdf\xaf\xb8\xe2\xf3\x98M\x89\x96\x193_\
4\xd7\xf8~\xa9\xce\xe5h\x08\xb23\xac\x10\xcb\xcap\
g`\xa0&\x93\xdf\xc8W!9\xcc\xdc\xb0\xf0\xcd|\
\x95\x8cF\x22\x897$\x95\x22eRo\xc8\x5c\x88\x98\
 \xef\x92r\xfdAD\x80\xf9\xb7\xca\xb0\xd9\x8a\x85\x17\
\x07\xe2&\x08\xf1\x81E\xe4\xe7\xcf\x82\xf9\xc00N\xfe\
a_s\xc6=\xb4\xcd\xe7E\x96\x84\xf8\x0as\x8c\xd3\
\xd1N!\x0el1\xd3$\xe2*\x8d\xe9\x86'\xcbb\
*\xe45\x89\xb9\xd2\x7frv\x1d\x84\x99\x94\x80\xed(\
\x89\xd8\x0d\xf9\xed\xf5k2\xde\xad\x0d.\xfa\x8b\x91\xff\
\xa6k\x06\xa3[\x18\xcb!\xd8\xfe\xd8b_\xc3\xb4\xe3\
`\xc9\xf4\xc8Ip'0\x22\x0f\xb48\x16\xd7L\xce\
\xa8b\xc0\xfd\x94\x0c\x875\x1e2\x89,\x0fWZ\xa7\
j:\x99D\x22\x0c.u\xc0\xc5\xe4R\x8f_N\x86\
5\xe2O\xc8\xa8\x9b;\xc3\xe0\xf0r\x1d\x8f/\xf5%\
.\xf08,\x16xo<\x84\xe1\x8e\x09?\x01\xed[\
\xe9u<l\x22\x02N\xdbH\x005L;\x83i\x99\
1;\xaf\xcaQ\x9ft\x00\xba\x91|\x96\xf1\x9b\x1b\xcd\
dB\xe3x3\x82\xc9\xed\x18\x88\xdbA\xa1H\x80P\
\x82>\x94\x8atE%\x99g<\x06\x99\x9d\xe9M\xcc\
T\x0ex\xc6\xb4\x869\xaa\xcaj\xe3\x06P\xc5\xe7\xf2\
c\x89Gi\x09\x1d\xf0\x038\xaa\x14\xcfV\xa0\xa4a\
\xa6+\x88\x140\xc1\x92\x90\xa9)\xf9:|\xa3\xc2\xe1\
S2<\xa0\xe1\xc5\xf0[\x09\xc2\x12\x0a\xbb\x07\x09j\
\xe80K\x1b\xb1\x14\xf4\xf6\x9fd\xabA\x22\xd9\x07\xc5\
\xbc\xa2\x1a\x01\x13\xb0\x12KxDD\xfbF_\x03\xe0\
h\xb9dr\xb4\xf3\x00~\xce4M\x22*\xa3\x7f\xb1\
M\xf0\x1e\xf4\x1dW\x0a\x16L}sS-7\x11\xee\
\x8c*\x99\x9c\x85\x86\xf4\xda<n\xbba\x89\x83\x04\xf4\
\xc19eT\xaa9J\x08\x95\x22\x92\x14\xd4yXa\
\xe8<\x9f(2\xf4\xa3\xa6E|AF\x0et;\x0d\
0#\x84\x12,\x15v.\xd5\xe6\xde\xd3\xb5\x1dm\xa6\
OX\xac\x98\x03\x7f\xce\xb5Q\xd1\x06\xf6\xdbA\xfd\xa9\
K\x82\x22\xc5\x8f\xea\x03K2\xbf\x00\x87k\xe8\xf6\xc8\
\xa7\x82\xa0\xcaKAs\x05v\x16\xdd\x15\xf8\x80\xe0\x1c\
,\xec\x01l\x8e-\x03\xa7\xe2:w(\x8d\xe9\xa9\x94\
\x86\xa0\xfbSR\xd8~\xdbh\x12\x82\xb2\xa9`\xc1\xe3\
xJR\x8abs\x02\xc4l\xa1?P\xb9\xe4\x09X\
\xf1B\xa5\xaa\xb6\x1d\x96\xbf\x10_n\xbc\xa7\x04<!\
[\xf0\x84E\x83\x1aB\xc3s\xa6uMn\xb6\xed\x87\
\xa0\xe2\xc6+\x15r\xca]\xd1\xb9X.cV\x10\x18\
\xb6\x17\x9e\x1a9w*\xafm\xa5\xd3s\xcc\xa1\xb1\xe4\
\xb5\xd7c:g\xb1\x83c\x5cqc\xc3\x0d@\xab[\
\xb3\x1b\xf0\xb8-\xc5\xd4l\x0d\x02k\xfb\x04\xfc\x0eN\
\x1e\x7f\x0fY\xccp.\xe0\x22\x00\x07\xb1\xa2x\x076\
\x93\xc9\x8dC\x08\x0b0\x03A\xcaoX|\xc6\xbf\xb3\
\xd6Jcc1G/lX\x0d\xde\xe0\xcbi\xe9Y\
\xab\x0dV\x9c\x7f\x07t4\xde\x8f\xf92\x01]\x85I\
\x80\xf16o\xefg\xf0\xcad[\xb2`c!(q\
\x0d\xf9\xd33$WU\xa3|_\xf2X\xa5\x0cE<\
\xcb\xd0\xa9=V\x0d\xfc;\xd06\x8f\x9e\x9d\xc0n\xb3\
\xa3\x89\xd9\xa1;\x83\xd6XD\xeb \x8e\xad\xb1\xfd\x9d\
07v\x9b\x15\x1be\x9c\x87iNX\xd0\xccD-\
\x84\x5c\x9f\x80u\xc0\x9dgl\xc2\xb9H\xf3es\x8e\
i\xd9\xa4\x16R\xa3\x92\xf9\x84\xad\x0bu\xa8\x93m5\
\xd3d}\xeb!\xc4\xc5b\xe9\xb2\x94\xb6\xdd>\x9e9\
\xf4m\xf7e\xac\xed\xe4\xfe\x02\x86\xf6\xe7\xa0\xad\xf7\xe5\
\x88\x22p\x1f9\xdd\x0e\xdco5\xcfshLa\xc3\
\xf3\xe4\xf6q\xab\x16E\xfc\xff\x81\xeaU\xb0\xe6\xc9\xa8\
\x1a]?\xad\x07\xd7;dB\x9e\x93\xdf\xc9^9\xd8\
f\x0b\xed\x18\x1c\x1b\x04x4\xee6\xa2\x1c\xb7<n\
\xc9+\xb7\xb5E\x03\xd8t\xfc\xaf!\xfe\xde\xc5\x00x\
A\xc1Eo#8\xa1x\xbe\xb5\xddx\x9e\xc1\x88\xad\
\x8f/M\xbc\xcbOm\xa5s\x5c\x18\xdff<\x12m\
\xedu}\x87-D\x98)\x87\x89\xaaF\x1b\xd3f\xb0\
\xe1t\xb05\x08\x93\x22L\x0d;\xc8\xa0\xcb\xc9`\xdf\
\x9b\x98\xa1Q%?\xeaY\xdaA\xa66@\x19\xb3A\
\x01\xae\xf1\x15Q\x22\x93!F\x18\x972\x9cNR\xba\
djR\x03\xfa\x08\x9f\x02\x88\xd1\x87\x0e\xdd\xeb$\x84\
V\xd7O\x01{{\xa2\xb6\x99\xa0\x0f\xb9\xed\xef\x8b^\
\xac\xe7\xa2\x0b}\xd1\xdf\x13=8j\xba\xe9\x96N\x05\
\xa4/\x110$^\xec\xd0\xf7\x08\xb4b\xd9\x85X,\
\xfb\x8b\xc5\xc4/\xaaC(\x06\xa0'\xfe\xb7\x12\x1c\xb7\
\x0f\xb7\xe9\xec\x89\xf8\x9d\x14Y\xda\xa1.\xb6\xbf'z\
t\xe1\x10\xb3\xfb\xb0\x17\xdd}\x91C\xd0\xb6z'y\
\xe4Eo\x01z\x12\xc0aw\x9a\x9b\x1aP_BR\
,%\xc4h\x1d\xa2\xaa\x80\xf4$rJ#.\xba\xb7\
n\x05\xa47\x91d\xc9\xce0\xd8\xf6\xce\xa4\x02\xd2\x93\
\xc8Y(E\x1cw\x08\xab\x04x\x14\x81;\x97\xbe\x01\
\xd6\x97\x18\xa32\x5c\xbd\xe5,\xf6*r\x05\xa4/\x91\
\xce%y\xdcj\xa4<\xe9\xb0 Ewo\xe417\
\xd1\x89\x1f}\x01\xd0\x97\x80\x0d\xca\xbc\x04,@_\x02\
\xd7<e\x9d\x04,@\x7f\x02:\x5cu`\x87\xde\x9e\
\xa8\xcf\xe9\xbcc\x97\xe5\xbd\xfdQ\xc7\x9db)\x01\xfa\
\x12\x80\x5cf_2\xea\xc5_\xf4?\x02}\xe7\x9e-\
\x01\xfa\x12\xc8\xcb]^\xf4y\xf7#\x90\x9f\xf3\xb4\x0b\
9t\xf7E.Y\xf7\xca\x16\xfd}\xd1gk\xd0\x0c\
\xbf`\xf2n\x1f\xf2F\x89%*\xe2\xb2i\xad\x0c\xe5\
)}Y`\x04m\x01\x14\xf9\xa7M\xb4\x82+\xfc\xe3\
\xaev\xe4i\xb6\x99T\xbb\x1a\x05\x99g\x8c\xd9'\xa6\
\xd5%2\xaef\xdb\xa2Y\xbb\x14\xd2>E\x80|\x14\
\xfeA\x8ev\x0fX<q0y\xd9=`\x8bS\x05\
7\xef^\xf0|\x9d\xda|\x8bd\x16s<\x0ek\xd6\
\xc9m\xf3\xd5\xcb\xbdU\x09\xc9t&\x13w\xb1\xc8W\
1w\xcb\x09[\xa5\x06\x9f\xa9\xd5(\x9f\x86\x9b8r\
\xea\xc8\xc0\xfd\x9c\x16\x05\xe40\x16x\x0cvga\xa4\
\xf6\xda\x08<\x02[\x8d\x9c6{`\xefxj*\xa5\
[k\x1e(\xd9\xef[\x8a\x9de\xf3_Z2\xe7\x09\
\xd7\x90\x86\xa1\x92O\xc9G\x9a47\x222\x98\xc2\xe7\
\xba4\x8e\xd6\xb0\xd7=[\x16R:\xe1\xdb\xaa\x88)\
\xa0W\x94\xc7\xe8jLM\x96L*E!\xdblq\
\xa8\x0e\xff\xde|u\x0e\xb02\x09M!\xf8(q\x9e\
5T\x01\xed\xfa\xe5\x95\xe3\x93\xc5B1\xa07~\xd1\
.j\xe3\x1a\xe4\xa7\xcbf\xd6\xc1G\xc8\x09\x98\xbcb\
\xfb*e!\xf8\x9b6\x8d\xd2F\xf2\xb5\xb1\x8f\x97z\
\x8cB\x09\xd2d9\xec\xd21\xdf\x19@\xb5F\xd8:\
5\xc7\x9d\x7f\x05\xb1\xa4\x22\x14\xcb\xa7D,\x88=*\
%zE5\x09iB\xe6\x8cd\x0a\xac\x84\x16\xe6\xa0\
3\x02\x90u\x1a30\xbb\xa6\x8a\xb6\xa0!\x8c\xe7\x09\
\xb1\xd8\x03\xd7!H!\xb9\xb5Q9\xe5<\x00\xb00\
Z\xa4\xb9\x22\x04s\x01\xa9M\xdbtWu\xb8,X\
\xe3\x8b\x17RZ\x8d0\xa0\xd2y\xae`as\xa2S\
B\xa5\x84=\x01\xcc\xdc\xef\x04\x22?\xb7x\xe8!D\
e\x94\xef\x1c\xe2Z\xd24W\xa1\x1c\xf8\x0b\xbcw)\
B\xd7\xf62s\xbaS\xe5\xf2\x99\xb7\x14\xae*\xa4\xfb\
\x8b\xde\x8a\xb3\x80\xf5,\xe9\x03N\xbe\xeaO\xb6\xe4l\
\xaaA\x9e\xc3\xf6\xbc\xb3\xec\xba)\xaa\xce\x12\xef\xc3\x8c\
j\x85g{\xe0\x81\x15\xe7\xbd\xad\x16oj#jE\
g\x00|\xb9\x05,*\xcf\xb5*\xac\xab2\xdbH(\
\xcbs\x8e\x12@\x15\xc7\xe8yr\x0f\xc3\x8bb\xfd\xc9\
\x05\xf9i\x9fg \xe2J\x10\x80\xe7\xea!x\xdc\xb6\
w\xb6r\x08\xcc\xe5\x02\xf0\xa0\xe6\x17o\xb7\x14\x971\
0\xf6\xad\x8d(\x0e\xf2\xc0L,\xf8\x12\xa1\xccm\x82\
\xfcn\x09>Z\xfb\xd1\xbc&\xe3$[\xb0\xdb\xf4\x9c\
\xb7\x15\xceO\xd9\x7f\xc1\x1c:8\xb7\x8c6B\x80\xf2\
\xbby8j\x05\x04w\x92.\x1f\xc3\xfc~B\xee\xc2\
f\x22\xce\xd6\x89\xf3\x8c\xba\xaaO9X\x9d`\xe5\x08\
\xbb\xd6\xe1;\xf3\xae\x0d\xda}\xd6\x8e\x80|\xf6\x1c[\
\xed\x94\x0c%0u\x98\xdc\xdb6N[L\xee8!\
\xb4\x92u\x02\xd4\xc2\xd5\xad\xe8[\xe7\x0a\xb6\x15\xa7\x07\
\xc5\x86\xa9\xdf\x92q\xc1\x97\x97\xd6\x02\x884\x0b'\xe3\
\x8d5\xb1my\x00\xa5\x80\xe0$\x1a\xd5u\xfd)\x9e\
\xf2\xc2\xd6\xc5b\xcc\x0d\x8b\xceLx\xeb\x8f\xf1\xf2\xf0\
\xb5D\x99_\xb8\xf2\x83cs)f[\x1fms\x9f\
\xf3\xdd\xe3\xa4\xb9=\xf8\xf6\xfez\x86\xeb\x1a\x8aX\xc8\
3\xd8\xa6k\x87\xec\xf1\xe6\xa1X\xb3\xd2\x83\xe7\xd3\xc0\
\xe8\x80\x83\xab\xd3\xfc;\x83h\x80A\xb0n\xa2\x16~\
\xc5 \xb5R\xe0\xb2\x88\xda(\xbc\x15\xb0\xcc \x86\x88\
!6t\xa2\x16\xe0\xf7\xcc\xf8\x02:\xa5\xb0\xae\x9a\x05\
-\xd8\xf2\xd2\xd2\xd7\xe1\xdb8\x03\xa9~\xe1\xc9\xe7\xa3\
\xe7x\xbd\xe9m\xa6\xb8H\xf0\x89\x9f\x9c\xe5\xf7\x9d\x14\
\x0f\x87\xdf\x02\x9e\x84q\x06\xdc\x8c\x9a\xa6\x02mZ{\
\xe9\xac\xc1\xea\xd8\x014\xd3b\xb6\x95\x96\x13\xae0w\
\x9e\x85\xc2V;\x9a\x05\x8c\xce\x0d\xda\x83;p\x0a\x17\
wqW\xa5}\x08\xf0\x04\xbd\xe8\xafb\xc0$\xbb\x0f\
\xe1\xe0\xd8\xb8\xc9{\xb3\x90{<s\x8e\xe1\xe1\x82\xdd\
\xc0\x92+sr\xeb\x15\xff\xdc\xfa\xcd\xaa\xde\x83\x97\x82\
\xf0\x15\x94\xc39\xa4\xb8\xf4\xc9\x0a\x9f;[a\xdd\xbb\
\xd3\xfa\xe0\xedGeP\xbf\xf2[(S\xcf#\xa3\xb0\
\x8a\xdbu\x7f\xac\xdaB\xaaXS\x0f\xa7\x9d#\x0c)\
\x03\x07F\x08l\xde\xe7\xe4\x22\x11\xd7\xee\x89V\xdb\x5c\
2z\xe1\xe7\xbed\xa6\xa1v\x0fc\x06u\xf0\x97q\
\xd2\xd4\xbf\x87\xb1r\xec\x0c\xfd\x1f\xc8\x8b\xdb\x8ac\x03\
\x0a\xc6\x0e\xbd\x07\x07\x09\xea\xb6\xe5\x12]\x83\x7f\xbb\xdc\
}\xcf\xa2\xf6\xda\xb1K\xcd%\x9f\xf65\xe6\xce\xac\xf0\
\x8d1\xbb\xe0\x85\x92H\x854um\xd3\xd2\xde\xd5\xaf\
>\xd8\xf6\xf0\xabRwL\xc1\x13f\xfa\xf9r\x07\xac\
\x9d\xd3>\xcc\x81\xaa7\xc8\x1f?\xaf\xfbd\xe1\xa7\x0c\
\xa2{\xa9\xcb\xf2\x9bK\xde\xa89\x90\x90\xfd\x8d\xfd}\
\x97\xed\xbdh\xe7a\x02CG\xbd\x99z\x22d\x8c\x5c\
\x1c12\xf9\x83\xec\x06x\x1f\xe5Y\xd0N\xbf\xff\xbf\
Y\xed=c\x9e\x0a\xd8\xfb\xa2\xac\xe3X\x82\xfaSW\
JX\xb9\xf8\xf4\xb8\xa4\xady\xdd\x0a\x82`\x7f&Y\
M$[\xd9c\xd9\xf9\xa8\x1bQE\x1ec\x85d\xe6\
\x99\xe7)\xf6vT\x09\x9a\x7fv\xe47\x95A\xf7K\
n|\xda^\xcc\xa4z\xcb\xac^\xb7\xeb\xdc\x1f\xc5\xff\
\x8diT\xaa\xf2[\xfbX\xd3\x07\xcf/\x95\x09(\x13\
\xbc|\xbad\x09\x93\xc6\xdaa\x99Jn\x8bR\xdbR\
\xd6\x9cb\xfdJt\x97\xa8\xee\xac\xb3\xe4zQ\xbf\xc7\
\xba\xbb\xd7\xc7\x1a<N>G\x89\xa9\xc0Q\xc9\x15L\
I\x0b\x87\xb0v\x9f:>\xb6\xad\x08\xb6'dH%\
D\x1b\x09\xf8\x8d\x08\xc4'\x16dM\x97\x09\xd7\x10U\
\x13\xc51\x09\x93Os\xaf\x8b\x0f\xe0$\xc0\x09@\x1c\
/\xff:\x19\xd6\x9fn\x07\xb7\x83\xff\x01f\x8c\xabf\
\
\x00\x00\x00`\
[\
Material]\x0aPrimar\
y=#41cd52\x0aAccent\
=#41cd52\x0aTheme=S\
ystem\x0a\x0a[Universa\
l]\x0aAccent=#41cd5\
2\x0aTheme=System\x0a\
\x00\x00\x00\x94\
/\
/ Copyright (C) \
2021 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick.Co\
ntrols\x0a\x0aToolBar \
{}\x0a\
\x00\x00\x00?\
m\
odule App\x0asingle\
ton GalleryConfi\
g 1.0 pages/Gall\
eryConfig.qml\x0a\
\x00\x00\x03\x5c\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aScrollabl\
ePage {\x0a    id: \
page\x0a\x0a    Column\
 {\x0a        spaci\
ng: 40\x0a        w\
idth: parent.wid\
th\x0a\x0a        Labe\
l {\x0a            \
width: parent.wi\
dth\x0a            \
wrapMode: Label.\
Wrap\x0a           \
 horizontalAlign\
ment: Qt.AlignHC\
enter\x0a          \
  text: qsTr(\x22Sp\
inBox allows the\
 user to choose \
an integer value\
 by clicking the\
 up or down indi\
cator buttons, \x22\
\x0a               \
 + \x22by pressing \
up or down on th\
e keyboard, or b\
y entering a tex\
t value in the i\
nput field.\x22)\x0a  \
      }\x0a\x0a       \
 SpinBox {\x0a     \
       enabled: \
!GalleryConfig.d\
isabled\x0a        \
    value: 50\x0a  \
          anchor\
s.horizontalCent\
er: parent.horiz\
ontalCenter\x0a    \
        editable\
: true\x0a         \
   Accessible.na\
me: qsTr(\x22demo s\
pinbox\x22)\x0a       \
 }\x0a    }\x0a}\x0a\
\x00\x00\x02\x8a\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aScrollabl\
ePage {\x0a    id: \
page\x0a\x0a    Column\
 {\x0a        spaci\
ng: 40\x0a        w\
idth: parent.wid\
th\x0a\x0a        Labe\
l {\x0a            \
width: parent.wi\
dth\x0a            \
wrapMode: Label.\
Wrap\x0a           \
 horizontalAlign\
ment: Qt.AlignHC\
enter\x0a          \
  text: qsTr(\x22Pa\
geIndicator is u\
sed to indicate \
the currently ac\
tive page in a c\
ontainer of page\
s.\x22)\x0a        }\x0a\x0a\
        PageIndi\
cator {\x0a        \
    count: 5\x0a   \
         current\
Index: 2\x0a       \
     anchors.hor\
izontalCenter: p\
arent.horizontal\
Center\x0a        }\
\x0a    }\x0a}\x0a\
\x00\x00\x05\xb8\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aPage {\x0a  \
  id: page\x0a    e\
nabled: !Gallery\
Config.disabled\x0a\
\x0a    SwipeView {\
\x0a        id: swi\
peView\x0a        a\
nchors.fill: par\
ent\x0a        curr\
entIndex: tabBar\
.currentIndex\x0a\x0a \
       Repeater \
{\x0a            mo\
del: 3\x0a\x0a        \
    Pane {\x0a     \
           width\
: SwipeView.view\
.width\x0a         \
       height: S\
wipeView.view.he\
ight\x0a\x0a          \
      Column {\x0a \
                \
   spacing: 40\x0a \
                \
   width: parent\
.width\x0a\x0a        \
            Labe\
l {\x0a            \
            widt\
h: parent.width\x0a\
                \
        wrapMode\
: Label.Wrap\x0a   \
                \
     horizontalA\
lignment: Qt.Ali\
gnHCenter\x0a      \
                \
  text: qsTr(\x22Ta\
bBar is a bar wi\
th icons or text\
 which allows th\
e user \x22\x0a       \
                \
       + \x22to swi\
tch between diff\
erent subtasks, \
views, or modes.\
\x22)\x0a             \
       }\x0a\x0a      \
              Im\
age {\x0a          \
              so\
urce: \x22../images\
/arrows.png\x22\x0a   \
                \
     anchors.hor\
izontalCenter: p\
arent.horizontal\
Center\x0a         \
           }\x0a   \
             }\x0a \
           }\x0a   \
     }\x0a    }\x0a\x0a  \
  footer: TabBar\
 {\x0a        id: t\
abBar\x0a        cu\
rrentIndex: swip\
eView.currentInd\
ex\x0a\x0a        TabB\
utton {\x0a        \
    text: qsTr(\x22\
First\x22)\x0a        \
}\x0a        TabBut\
ton {\x0a          \
  text: qsTr(\x22Se\
cond\x22)\x0a        }\
\x0a        TabButt\
on {\x0a           \
 text: qsTr(\x22Thi\
rd\x22)\x0a        }\x0a \
   }\x0a}\x0a\
\x00\x00\x02\xcc\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ap\
ragma ComponentB\
ehavior: Bound\x0a\x0a\
import QtQuick\x0ai\
mport QtQuick.Co\
ntrols\x0a\x0aScrollab\
lePage {\x0a    id:\
 page\x0a\x0a    Colum\
n {\x0a        spac\
ing: 40\x0a        \
width: parent.wi\
dth\x0a\x0a        Lab\
el {\x0a           \
 width: parent.w\
idth\x0a           \
 wrapMode: Label\
.Wrap\x0a          \
  horizontalAlig\
nment: Qt.AlignH\
Center\x0a         \
   text: qsTr(\x22B\
usyIndicator is \
used to indicate\
 activity while \
content is being\
 loaded,\x22\x0a      \
            + \x22 \
or when the UI i\
s blocked waitin\
g for a resource\
 to become avail\
able.\x22)\x0a        \
}\x0a\x0a        BusyI\
ndicator {\x0a     \
       anchors.h\
orizontalCenter:\
 parent.horizont\
alCenter\x0a       \
 }\x0a    }\x0a}\x0a\
\x00\x00\x02\x01\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aPage {\x0a  \
  id: page\x0a\x0a    \
default property\
 alias content: \
pane.contentItem\
\x0a\x0a    Flickable \
{\x0a        anchor\
s.fill: parent\x0a \
       contentHe\
ight: pane.impli\
citHeight\x0a      \
  flickableDirec\
tion: Flickable.\
AutoFlickIfNeede\
d\x0a\x0a        Pane \
{\x0a            id\
: pane\x0a         \
   width: parent\
.width\x0a        }\
\x0a\x0a        Scroll\
Indicator.vertic\
al: ScrollIndica\
tor { }\x0a    }\x0a}\x0a\
\
\x00\x00\x03\xca\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aScrollabl\
ePage {\x0a    id: \
page\x0a\x0a    Column\
 {\x0a        spaci\
ng: 40\x0a        w\
idth: parent.wid\
th\x0a\x0a        Labe\
l {\x0a            \
width: parent.wi\
dth\x0a            \
wrapMode: Label.\
Wrap\x0a           \
 horizontalAlign\
ment: Qt.AlignHC\
enter\x0a          \
  text: qsTr(\x22Pr\
ogressBar indica\
tes the progress\
 of an operation\
. It can be set \
in an \x22\x0a        \
        + \x22indet\
erminate mode to\
 indicate that t\
he length of the\
 operation is un\
known.\x22)\x0a       \
 }\x0a\x0a        Prog\
ressBar {\x0a      \
      id: bar\x0a  \
          value:\
 0.5\x0a           \
 anchors.horizon\
talCenter: paren\
t.horizontalCent\
er\x0a            A\
ccessible.name: \
qsTr(\x22demo progr\
essbar\x22)\x0a       \
 }\x0a\x0a        Prog\
ressBar {\x0a      \
      indetermin\
ate: true\x0a      \
      anchors.ho\
rizontalCenter: \
parent.horizonta\
lCenter\x0a        \
    Accessible.n\
ame: qsTr(\x22indet\
erminate progres\
sbar\x22)\x0a        }\
\x0a    }\x0a}\x0a\
\x00\x00\x06\x87\
/\
/ Copyright (C) \
2025 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aScrollabl\
ePage {\x0a    id: \
page\x0a\x0a    Column\
 {\x0a        spaci\
ng: 40\x0a        w\
idth: parent.wid\
th\x0a\x0a        Labe\
l {\x0a            \
width: parent.wi\
dth\x0a            \
wrapMode: Label.\
Wrap\x0a           \
 horizontalAlign\
ment: Qt.AlignHC\
enter\x0a          \
  text: qsTr(\x22Se\
archField is a s\
tyled text input\
 for searching, \
typically \x22\x0a    \
                \
   + \x22with a mag\
nifier and clear\
 icon.\x22)\x0a       \
 }\x0a\x0a        List\
Model {\x0a        \
    id: colorMod\
el\x0a            L\
istElement { col\
or: \x22blue\x22 }\x0a   \
         ListEle\
ment { color: \x22g\
reen\x22 }\x0a        \
    ListElement \
{ color: \x22red\x22 }\
\x0a            Lis\
tElement { color\
: \x22yellow\x22 }\x0a   \
         ListEle\
ment { color: \x22o\
range\x22 }\x0a       \
     ListElement\
 { color: \x22purpl\
e\x22 }\x0a        }\x0a\x0a\
        SortFilt\
erProxyModel {\x0a \
           id: c\
olorFilter\x0a     \
       model: co\
lorModel\x0a       \
     sorters: [\x0a\
                \
RoleSorter {\x0a   \
                \
 roleName: \x22colo\
r\x22\x0a             \
   }\x0a           \
 ]\x0a            f\
ilters: [\x0a      \
          Functi\
onFilter {\x0a     \
               c\
omponent CustomD\
ata: QtObject { \
property string \
color }\x0a        \
            prop\
erty var regExp:\
 new RegExp(colo\
rSearch.text, \x22i\
\x22)\x0a             \
       onRegExpC\
hanged: invalida\
te()\x0a           \
         functio\
n filter(data: C\
ustomData): bool\
 {\x0a             \
          return\
 regExp.test(dat\
a.color);\x0a      \
              }\x0a\
                \
}\x0a            ]\x0a\
        }\x0a\x0a     \
   SearchField {\
\x0a            id:\
 colorSearch\x0a   \
         suggest\
ionModel: colorF\
ilter\x0a          \
  anchors.horizo\
ntalCenter: pare\
nt.horizontalCen\
ter\x0a        }\x0a  \
  }\x0a}\x0a\
\x00\x00\x03\x08\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aScrollabl\
ePage {\x0a    id: \
page\x0a\x0a    Column\
 {\x0a        spaci\
ng: 40\x0a        w\
idth: parent.wid\
th\x0a\x0a        Labe\
l {\x0a            \
width: parent.wi\
dth\x0a            \
wrapMode: Label.\
Wrap\x0a           \
 horizontalAlign\
ment: Qt.AlignHC\
enter\x0a          \
  text: qsTr(\x22Te\
xtArea is a mult\
i-line text edit\
or.\x22)\x0a        }\x0a\
\x0a        TextAre\
a {\x0a            \
enabled: !Galler\
yConfig.disabled\
\x0a            wid\
th: page.availab\
leWidth / 3\x0a    \
        anchors.\
horizontalCenter\
: parent.horizon\
talCenter\x0a\x0a     \
       wrapMode:\
 TextArea.Wrap\x0a \
           text:\
 qsTr(\x22TextArea\x5c\
n...\x5cn...\x5cn...\x22)\
\x0a            Acc\
essible.name: qs\
Tr(\x22demo text ar\
ea\x22)\x0a        }\x0a \
   }\x0a}\x0a\
\x00\x00\x02\xec\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aScrollabl\
ePage {\x0a    id: \
page\x0a\x0a    Column\
 {\x0a        spaci\
ng: 40\x0a        w\
idth: parent.wid\
th\x0a\x0a        Labe\
l {\x0a            \
width: parent.wi\
dth\x0a            \
wrapMode: Label.\
Wrap\x0a           \
 horizontalAlign\
ment: Qt.AlignHC\
enter\x0a          \
  text: qsTr(\x22A \
tool tip is a sh\
ort piece of tex\
t that informs t\
he user of a con\
trol's function.\
\x22)\x0a        }\x0a\x0a  \
      Button {\x0a \
           text:\
 qsTr(\x22Tip\x22)\x0a   \
         anchors\
.horizontalCente\
r: parent.horizo\
ntalCenter\x0a\x0a    \
        ToolTip.\
timeout: 5000\x0a  \
          ToolTi\
p.visible: press\
ed\x0a            T\
oolTip.text: qsT\
r(\x22This is a too\
l tip.\x22)\x0a       \
 }\x0a    }\x0a}\x0a\
\x00\x00\x02=\
\x00\
\x00\x07\x7fx\x9c\xd5TM\x8f\xd30\x10\xbd\xe7W\x0c\
9\xed\x0a\x92.]\x81\xd4\xdchW\xd0JEl?\
\xc4r\xe0\xe2\xc6\x93\xc6\xc2\xb5\x83\xed\xb4\x14\xb4\xff\x9d\
\xb1\xd3n\xbfO\xcb\x81\xf5\xc5\x9a\xf1\x9b7\xe37\x1e\
\xb7Z\xd0\xd3\xd5\xda\x88y\xe9\xe0\xaaw\x0d\xed\x9b\xf6\
;\x98\x96\x08#G'\x8b\x8a\xa95\x0c\x1dO\xa3V\
\x0b&\xf7w\xdf\x92\xa1\xc8QYL\x06\x1c\x95\x13\x85\
@\x93\xc1\xc67\xc6\x22\x19\xb9\x84\xc2\x16hr\xc1$\
|\x19Cwr\x97\xdc&=\xc9j\x8bQ$\x16\x95\
6\x8e\xc8G\xb5\xc8\x7f\x1c\x99iO+g\xb4\xb4\xc7\
\xfe![\xeb\xda\xd9(\xbags\x84?\x11\xd0\x12<\
\x83\x8a\xcc`\xa0b3\x89\xe4y\xf5\x89I\x89fM\
L\x85\x98\xa7\x5c\xd8p\x10\x05T\x89\x8c\xfbr\xa7Z\
\xcb.3\x1b\x22\xbf\xc6z\xd5\xe4\xd8\xf3\xf9\xc5T^\
jc\xd3BH\xe9\xd3\x19\xbast\x80\x188\x5c\x1c\
\x05\xf9\xd5\xb0\x85\xb8>zq3p\xa6\xc6K\xb8\xca\
`\x81\xc6 \x7f\x10\xdc\x95\x19U\xeac\x0e\xd0\x8f\x87\
y\x87l\x86\xf2Lb*W\xfc&\x19\x99\xfc \xc5\
\x5c-\xa8`\xba0\xferi\xb0\xfb=r\xa09\x09\
[\xa2q\x22\xbf\x10\xf4\xf5B\x90#D\x06?\xed\xd4\
\x5c\xc5\xfd\xa0m|\x1d=W\x0a\x8f\xdb\xa8p\x02;\
\xd2\xc0\xf7q\x82\xd4\x15\xe64u\xf3\xdcq\xb7vN\
+:kj\x8d\xbf\xd7\xed\xf7\x9dNL\xd0\x1d\xe7\x1e\
\xf3\xb1\xaa\xdb\xfe\xe7A\x80\x81zz\x03[\xc0\xaa\xa9\
\xb4\xf1\xa6\xc1\x82\x84\x86h\x070\xac\xfa\xac9f\x0d\
w\xfa@\xf6\xd3\xe1\xd9f\x8d.\xb4j_\xed\xed\x0b\
\xae\x8c^\x0a\x8e\x16\xd8\x1e\x17\xe4~\x13\x0a\x0d\x14\xa4\
\x0b\xab*I\xaduB\xab\x84\x0aD\x88O\xa4\x87\xd7\
\x103\xc5C eI,M\xb3pb\x89\xc1\xe3'\
\xf2\x0d\xd8:/\x81YPl)\xe6\x81\x0dfA\x5c\
JN\xa1\xe7I-2Ca\xf4KHN\x1cn]\
\xf9G&\xd7@\x9f\x01\xf7ll3\x94\xa0}\xb1\x9a\
\xeeK\x92\xbaR(\x22\xbdT\xe8\xee>\x84U\x5c\xaf\
\xe8\xd5\xed\xf5\xb0\xa1\xf9\xb7S\xfe\xffL\x9b\x7f\xc1o\
o;\xf1\xb3\xe7\xec\xe5\x7f9\x1fC\xa3_\xc4\x97s\
{s\xee\xcby\x8c\xfe\x02O\xe4\x17\xf2\
\x00\x00\x03\xb3\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aScrollabl\
ePage {\x0a    id: \
page\x0a\x0a    Column\
 {\x0a        spaci\
ng: 40\x0a        w\
idth: parent.wid\
th\x0a\x0a        Labe\
l {\x0a            \
width: parent.wi\
dth\x0a            \
wrapMode: Label.\
Wrap\x0a           \
 horizontalAlign\
ment: Qt.AlignHC\
enter\x0a          \
  text: qsTr(\x22Sl\
ider is used to \
select a value b\
y sliding a hand\
le along a track\
.\x22)\x0a        }\x0a\x0a \
       Slider {\x0a\
            enab\
led: !GalleryCon\
fig.disabled\x0a   \
         value: \
0.5\x0a            \
anchors.horizont\
alCenter: parent\
.horizontalCente\
r\x0a            Ac\
cessible.name: q\
sTr(\x22demo slider\
\x22)\x0a        }\x0a\x0a  \
      Slider {\x0a \
           enabl\
ed: !GalleryConf\
ig.disabled\x0a    \
        orientat\
ion: Qt.Vertical\
\x0a            val\
ue: 0.5\x0a        \
    anchors.hori\
zontalCenter: pa\
rent.horizontalC\
enter\x0a          \
  Accessible.nam\
e: qsTr(\x22vertica\
l demo slider\x22)\x0a\
        }\x0a    }\x0a\
}\x0a\
\x00\x00\x02x\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aScrollabl\
ePage {\x0a    id: \
page\x0a\x0a    Column\
 {\x0a        spaci\
ng: 40\x0a        w\
idth: parent.wid\
th\x0a\x0a        Labe\
l {\x0a            \
width: parent.wi\
dth\x0a            \
wrapMode: Label.\
Wrap\x0a           \
 horizontalAlign\
ment: Qt.AlignHC\
enter\x0a          \
  text: qsTr(\x22Tu\
mbler is used to\
 select a value \
by spinning a wh\
eel.\x22)\x0a        }\
\x0a\x0a        Tumble\
r {\x0a            \
enabled: !Galler\
yConfig.disabled\
\x0a            mod\
el: 10\x0a         \
   anchors.horiz\
ontalCenter: par\
ent.horizontalCe\
nter\x0a        }\x0a \
   }\x0a}\x0a\
\x00\x00\x0b\xb4\
/\
/ Copyright (C) \
2025 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0aimport QtQ\
uick.Layouts\x0aimp\
ort Qt.labs.qmlm\
odels\x0a\x0aPage {\x0a  \
  id: page\x0a\x0a    \
GridLayout {\x0a   \
     anchors.fil\
l: parent\x0a      \
  anchors.margin\
s: 10\x0a\x0a        L\
abel {\x0a         \
   wrapMode: Lab\
el.Wrap\x0a        \
    horizontalAl\
ignment: Qt.Alig\
nHCenter\x0a       \
     text: qsTr(\
\x22TreeView provid\
es a hierarchica\
l view for displ\
aying and \x22\x0a    \
                \
 + \x22navigating t\
ree-structured d\
ata, allowing us\
ers to expand an\
d \x22\x0a            \
         + \x22coll\
apse nodes to ex\
plore parent-chi\
ld relationships\
 within a model\x22\
)\x0a\x0a            L\
ayout.fillWidth:\
 true\x0a          \
  Layout.columnS\
pan: 2\x0a        }\
\x0a\x0a        Item {\
\x0a            imp\
licitHeight: 40\x0a\
\x0a            Lay\
out.columnSpan: \
2\x0a            La\
yout.row: 1\x0a    \
    }\x0a\x0a        H\
orizontalHeaderV\
iew {\x0a          \
  clip: true\x0a   \
         enabled\
: !GalleryConfig\
.disabled\x0a      \
      syncView: \
treeView\x0a       \
     model: [qsT\
r(\x22Location\x22)]\x0a\x0a\
            Layo\
ut.column: 1\x0a   \
         Layout.\
row: 2\x0a         \
   Layout.fillWi\
dth: true\x0a      \
  }\x0a\x0a        Ver\
ticalHeaderView \
{\x0a            cl\
ip: true\x0a       \
     enabled: !G\
alleryConfig.dis\
abled\x0a          \
  syncView: tree\
View\x0a           \
 model: Array.fr\
om({length: tree\
View.rows}, (v, \
k) => k + 1)\x0a\x0a  \
          Layout\
.column: 0\x0a     \
       Layout.ro\
w: 3\x0a           \
 Layout.fillHeig\
ht: true\x0a       \
 }\x0a\x0a        Tree\
View {\x0a         \
   id: treeView\x0a\
            clip\
: true\x0a         \
   enabled: !Gal\
leryConfig.disab\
led\x0a            \
rowSpacing: 2\x0a  \
          model:\
 treeModel\x0a\x0a    \
        Layout.c\
olumn: 1\x0a       \
     Layout.row:\
 3\x0a            L\
ayout.fillWidth:\
 true\x0a          \
  Layout.fillHei\
ght: true\x0a\x0a     \
       selection\
Model: ItemSelec\
tionModel {}\x0a   \
         delegat\
e: TreeViewDeleg\
ate { }\x0a\x0a       \
     columnWidth\
Provider: (colum\
n) => column ===\
 0 ? treeView.wi\
dth : 0\x0a\x0a       \
     Component.o\
nCompleted: expa\
ndRecursively()\x0a\
        }\x0a    }\x0a\
\x0a    TreeModel {\
\x0a        id: tre\
eModel\x0a\x0a        \
TableModelColumn\
 { display: \x22loc\
ation\x22 }\x0a\x0a      \
  rows: [\x0a      \
      {\x0a        \
        location\
: qsTr(\x22America\x22\
),\x0a             \
   rows: [\x0a     \
               {\
 location: qsTr(\
\x22Brazil\x22) },\x0a   \
                \
 {\x0a             \
           locat\
ion: qsTr(\x22Canad\
a\x22),\x0a           \
             row\
s: [\x0a           \
                \
 { location: qsT\
r(\x22Calgary\x22) },\x0a\
                \
            { lo\
cation: qsTr(\x22Va\
ncouver\x22) }\x0a    \
                \
    ]\x0a          \
          }\x0a    \
            ]\x0a  \
          },\x0a   \
         { locat\
ion: qsTr(\x22Asia\x22\
) },\x0a           \
 {\x0a             \
   location: qsT\
r(\x22Europe\x22),\x0a   \
             row\
s: [\x0a           \
         {\x0a     \
                \
   location: qsT\
r(\x22Italy\x22),\x0a    \
                \
    rows: [\x0a    \
                \
        { locati\
on: qsTr(\x22Milan\x22\
) },\x0a           \
                \
 { location: qsT\
r(\x22Rome\x22) }\x0a    \
                \
    ]\x0a          \
          },\x0a   \
                \
 { location: qsT\
r(\x22Portugal\x22) }\x0a\
                \
]\x0a            }\x0a\
\x0a        ]\x0a    }\
\x0a}\x0a\
\x00\x00\x03&\
\x00\
\x00\x0c\xb5x\x9c\xd5VKo\xdb0\x0c\xbe\xfbWp\
\x01\x0a$H\xed<\xb6]\x8c\xb6\xc0\x9a`]\x81\xf4\
\xdd\xa1\xdb\xa9Pm:\x11*K\x9e,\xa3\xf3\x86\xfc\
\xf7Iv\x9a\xf8\x99\xa5+v(O\x12\xc9\x8f\x22)\
\x92\xd2`\x00\x13\x11\xa5\x92\xce\x17\x0a\xba\x93\x1e\x8c\x87\
\xe3\x8fp\xbb@\xb8RZ\x12F\x84\xa70S\xbec\
\x0d\x06ps9\xfdf\xcf\xa8\x87<F\xfb\xd4G\xae\
h@Q\xba\xb0\xe2]c`_)[\xc3B\x94\x1e\
%\x0c.\xae\xe1\xf8fj\xbf\xb7'\x8c$1Z\x16\
\x0d#!\x956~\x95P\xef\xb1\xb2u&\x82+)\
X\x5c\xe5\xcfH*\x12\x15[\xd6%\x99#\xfc\xb6@\
\x13\xf5]\x88\xf46\xdb '\x0f\x0c5\xe7\xdd\x09a\
\x0ce\xaa-\x05t\xee\xf84\xce\x04V\xa65\x11,\
\x09\xf9\x0ao(\x8e\x88G\xf9\xdc\x85\x0f\xc35\xef\x89\
\xfajaLK\x1d\x9f\x93\xed\xac\xb5pF\x1e\x90\x15\
\x0c\xb4\x01JrI\xa23\xe1\xa3\x9b\xa3\x9d;\xbd/\
),\x84\xa4\xbft\xe4\x84}bt\xceCm\xc6\xd5\
\xa1;\xd9\xee\xcbDoQ\x96\x00\x0a\x7fj\x8d\x1f\xf1\
\xad\xecv\xce4pq\x22\xa9\x0f\x91\xc4X\xeb\xc6@\
\xc0#\x0c\xb9O$\x84F\x0a\xc4\xf0\xe6FG\x04\xe0\
\x934\xde\x87N\xc9\xe0\x9a\xfa\xd0\xd1X/aD\xa1\
\x0f\x81\x90\x1a\x18G\xe8\xe9{\xf6rc\xfb\x90\x22\x91\
\xfb@\xb8\x0fL\x98\x83\x9cNoml\xb9IU\x9e\
\xec\xfc\xe6*\x19[\xa7}<,\xf1\x09\xf7t.b\
g\x93\x90<\xf8ur\xab\x02\xab\x04\xbf\x16O\x8d\xc7\
\x95\x8e\x1c\x0dk\xb2\x1c\xe4\x04\x94\xb1\xbb\xfc.\x95L\
\xd0\xaa\xe9\x1d'J\x09\xde`\xdc\x90\xaeXF=\xaa\
V\x16\x16h:\xaaQ\xf3\xaf\xb5\xda\x04\x0a\xf4\x8d\xac\
\xfcj\x12\x17+\xe2\xa0p\x1fE\x12|\xa2=|4\
'7\x87`\xc8\x13<V\xc0\xf1\xe9>/\x9d\xc3\xfc\
\xd6M\x8199\xc7\x86Q+\x9a\x06\xd0\xdd`\x0f`\
\xd8\xdbr\x94\xa1\xaa\xf1C\x18\xb5[7d\xdb\x1b\x88\
)\xc4V\xe5% \x8b\xf1\xc5\xc7\xaf\xbdo7\xdc(\
\xa9s\xeb\x9cS\x85a\x8bC\x85\x12\xfc\x92U\xce\x96\
\xbbn)\xd7f\xd5\xfa\xb8*\xd2s\xc3yY7\x9d\
\xf2\xe7Fk\x05\x14\xeblo\x04{\xe3N\xcf!r\
\xde\xdd$r5\x132\xc69\x09\xb1[\xc9q\xaf\xb9\
8[\xa8b\xdc\xdcx3~\x97\xfc\xbf\x91\xfe=\xfa\
\xbf\xfd\xdb\xdf\xb9\x7f\x8ft7\x8e_\xde\xc1\xf5\x01[\
\xa4~\xffm6\xf0\xb2\xfc\x1e\x983[\x1f\x1b/{\
\xfab\xfd\xbe\xed\xfa\xd6lQ+\xcd\x83\x9a\xe2\x94\xa4\
\x17\xc1\x1d\xe2\xa3~\xfcZr\x95w\xa4\x0b\xd5\x1e}\
\xfdp\xc9T\xf3`\xdd\x86\xaaZ\xd6\xdd5\x9e\x9e'\
\xe1\x03\xca\xda_\xec\x9f]6UTTm\xad\xaaL\
\xc5\xad\xd6\xcc\xab\xe6qC\x88\x9b\x0fY\xcb\xa0\xf1\x0b\
.l\x8d^\x7f\x01\xf3e\xb7\x83\xfc\xfe\xebM\xcb\x5c\
\xc8\xe3\xf7\x12i&\xf7T\x7f\xdc\x9c9\xaa\xcf\x09c\
\xdf\xb5\xa0\xdb\x8cY\xa5\xa2\x02\xca\x5coAH$\xbe\
\xe0,\xd5\xdfL\x11\xa1T\xa9\xfeH*,ZpM\
\xef\x81Y\xb5\x98\xd8\xb1\xb6\xaa}W^-\xad\xa5\xf5\
\x07\x08T\x18\x9d\
\x00\x00\x07\xc4\
/\
/ Copyright (C) \
2025 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0aimport QtQ\
uick.Layouts\x0a\x0aPa\
ge {\x0a    id: pag\
e\x0a    enabled: !\
GalleryConfig.di\
sabled\x0a\x0a    Colu\
mnLayout {\x0a     \
   anchors.fill:\
 parent\x0a        \
spacing: 40\x0a\x0a   \
     CheckBox {\x0a\
            id: \
orientationCheck\
Box\x0a            \
text: qsTr(\x22Vert\
ical\x22)\x0a        }\
\x0a\x0a        Label \
{\x0a            wr\
apMode: Label.Wr\
ap\x0a            h\
orizontalAlignme\
nt: Qt.AlignHCen\
ter\x0a            \
text: qsTr(\x22Spli\
tView provides a\
 container that \
arranges items h\
orizontally \x22\x0a  \
                \
   + \x22or vertica\
lly, separated b\
y draggable spli\
tters, allowing \
users \x22\x0a        \
             + \x22\
to interactively\
 resize adjacent\
 views within an\
 application.\x22)\x0a\
            Layo\
ut.fillWidth: tr\
ue\x0a        }\x0a\x0a  \
      SplitView \
{\x0a            or\
ientation: orien\
tationCheckBox.c\
hecked ? Qt.Vert\
ical : Qt.Horizo\
ntal\x0a           \
 Layout.fillHeig\
ht: true\x0a       \
     Layout.fill\
Width: true\x0a\x0a   \
         Rectang\
le {\x0a           \
     implicitWid\
th: 200\x0a        \
        implicit\
Height: 100\x0a    \
            colo\
r: \x22lightblue\x22\x0a \
               S\
plitView.maximum\
Width: 400\x0a\x0a    \
            Labe\
l {\x0a            \
        text: \x22V\
iew 1\x22\x0a         \
           ancho\
rs.centerIn: par\
ent\x0a            \
    }\x0a          \
  }\x0a\x0a           \
 Rectangle {\x0a   \
             id:\
 centerItem\x0a    \
            colo\
r: \x22lightgray\x22\x0a \
               S\
plitView.minimum\
Width: 50\x0a      \
          SplitV\
iew.minimumHeigh\
t: 50\x0a          \
      SplitView.\
fillWidth: true\x0a\
                \
SplitView.fillHe\
ight: true\x0a\x0a    \
            Labe\
l {\x0a            \
        text: \x22V\
iew 2\x22\x0a         \
           ancho\
rs.centerIn: par\
ent\x0a            \
    }\x0a          \
  }\x0a\x0a           \
 Rectangle {\x0a   \
             imp\
licitWidth: 200\x0a\
                \
implicitHeight: \
100\x0a            \
    color: \x22ligh\
tgreen\x22\x0a\x0a       \
         Label {\
\x0a               \
     text: \x22View\
 3\x22\x0a            \
        anchors.\
centerIn: parent\
\x0a               \
 }\x0a            }\
\x0a        }\x0a    }\
\x0a}\x0a\
\x00\x00\x09\x93\
/\
/ Copyright (C) \
2025 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0aimport QtQ\
uick.Layouts\x0aimp\
ort Qt.labs.qmlm\
odels\x0a\x0aPage {\x0a  \
  id: page\x0a    e\
nabled: !Gallery\
Config.disabled\x0a\
\x0a    GridLayout \
{\x0a        anchor\
s.fill: parent\x0a\x0a\
        Label {\x0a\
            wrap\
Mode: Label.Wrap\
\x0a            hor\
izontalAlignment\
: Qt.AlignHCente\
r\x0a            te\
xt: qsTr(\x22TableV\
iew provides a s\
crollable grid t\
hat displays dat\
a from \x22\x0a       \
                \
+ \x22a model in ro\
ws and columns, \
allowing users t\
o view and inter\
act \x22\x0a          \
             + \x22\
with structured \
information with\
in an applicatio\
n.\x22)\x0a           \
 Layout.fillWidt\
h: true\x0a        \
    Layout.colum\
nSpan: 2\x0a       \
 }\x0a\x0a        Hori\
zontalHeaderView\
 {\x0a            c\
lip: true\x0a      \
      syncView: \
tableView\x0a      \
      model: tab\
leModel.headerMo\
del\x0a            \
Layout.column: 1\
\x0a            Lay\
out.row: 1\x0a     \
       Layout.fi\
llWidth: true\x0a  \
      }\x0a\x0a       \
 VerticalHeaderV\
iew {\x0a          \
  clip: true\x0a   \
         syncVie\
w: tableView\x0a   \
         Layout.\
column: 0\x0a      \
      Layout.row\
: 2\x0a            \
Layout.fillHeigh\
t: true\x0a        \
}\x0a\x0a        Table\
View {\x0a         \
   id: tableView\
\x0a            col\
umnSpacing: 1\x0a  \
          rowSpa\
cing: 1\x0a        \
    clip: true\x0a\x0a\
            sele\
ctionModel: Item\
SelectionModel {\
}\x0a            mo\
del: tableModel\x0a\
\x0a            Lay\
out.column: 1\x0a  \
          Layout\
.row: 2\x0a        \
    Layout.fillW\
idth: true\x0a     \
       Layout.fi\
llHeight: true\x0a\x0a\
            dele\
gate: TableViewD\
elegate {\x0a      \
          implic\
itWidth: 100\x0a   \
             imp\
licitHeight: 50\x0a\
                \
Component.onComp\
leted: {\x0a       \
             if \
(contentItem as \
Label) {\x0a       \
                \
 contentItem.hor\
izontalAlignment\
 = Qt.AlignHCent\
er\x0a             \
           conte\
ntItem.verticalA\
lignment = Qt.Al\
ignVCenter\x0a     \
               }\
\x0a               \
 }\x0a            }\
\x0a        }\x0a    }\
\x0a\x0a    TableModel\
 {\x0a        id: t\
ableModel\x0a      \
  property var h\
eaderModel: [qsT\
r(\x22Name\x22), qsTr(\
\x22Color\x22)]\x0a      \
  TableModelColu\
mn { display: \x22n\
ame\x22 }\x0a        T\
ableModelColumn \
{ display: \x22colo\
r\x22 }\x0a        row\
s: [\x0a           \
 {\x0a             \
   \x22name\x22: qsTr(\
\x22cat\x22),\x0a        \
        \x22color\x22:\
 qsTr(\x22black\x22)\x0a \
           },\x0a  \
          {\x0a    \
            \x22nam\
e\x22: qsTr(\x22dog\x22),\
\x0a               \
 \x22color\x22: qsTr(\x22\
brown\x22)\x0a        \
    },\x0a         \
   {\x0a           \
     \x22name\x22: qsT\
r(\x22bird\x22),\x0a     \
           \x22colo\
r\x22: qsTr(\x22white\x22\
)\x0a            }\x0a\
        ]\x0a    }\x0a\
}\x0a\
\x00\x00\x02D\
\x00\
\x00\x08Ux\x9c\xd5TMo\x9b@\x10\xbd\xf3+\xa6\
>%j\xc1nZ\xa9\x12\x97\xaa&jS\xc9U\xfd\
\x11)\xbd\xaea0\xab,\xbbdw\x91C\xab\xfc\xf7\
\xce\x02v\x0c8Q\xd3(R\xcb\xc5\x9e\x997o\x87\
\xb7\xf3\x18\x8f!RE\xa5\xf9&\xb3p\x12\x9d\xc2\xd9\
\xe4\xed\x07\xb8\xcc\x10\x16\x96*y\xc1d\x053\x9b\x04\
\xdex\x0c\xab\xf9\xf9\x0f\x7f\xc6c\x94\x06\xfd\xaf\x09J\
\xcbS\x8e:\x846\xb7\xc4\xd4_X\x9f\xdar\xd41\
g\x02\xbe/a\xba:\xf7\xdf\xf9\x91`\xa5A\xcf\xe3\
y\xa1\xb4%\xf2E\xc9\xe3\xeb^\x18\xccX\xa5Jk\
\xfa\xe9HI\xab\x950\x9e\xb7\x8a\xe9W\xb0\xb5\xc09\
\xdb \xfc\xf2\x80\x1e\x9e\x84PP\xe8\xd5Q\xa4D\x99\
\xcb\xb6\xe4\x1eS\xb0\x98\xcbM\x08\xef'\xfb\xdc\x96'\
6s]\x9a^\x22\xa8#o_\x5c\xaa\xedA{\xcd\
\x99a|=U\xb7\xbd\xf4\xee\xf0\xd8\x951\xd9\xa1\x06\
\x18\x8b\xb76\x84\x1bs\xa9OFQ\x83\x1d\x9dvP\
w\xdeS\xceK\x05\xb3\x7ft\xd8g\x02>\xeb\xa4B\
\xa31\x8f\xbc\x19Jw\x17\x04|\xf5\x85\x09\x81\xba\xa2\
\xabJ\xf9&H\xb8\xa9\x0b\x8fN7o\xc8\x07\x03\x1e\
\x19u\xc6\xd6(z3\x1e\xbb\xc3N]\xb3\xe2\x9bJ\
0l\xba\x83+\x8a;\x80Li\xfe\x93V\x8b\x89O\
\x82odN4!\xed\x5cPG\x17\x11\x85\xa8;\x0d\
\x87\xc3OKk\x95\xac\x05\x22\xa0\x01\x06Ei2\x7f\
\xdd\xa4m\xc6,\xc4L\xc2\x1a\xeb<&\xa04\xc4\x82\
\xbb\xbb\x87uE\x00\x042\x84\x0e`4\xd0\xe85\xb4\
\xec\xc4\xaa\x11\xa4\xd29\x89[9|\x02VA\x81:\
\xa5\x1c\x10=\x8b-W\xf2\x8d#\xa7\x02\x93f\x8b\x9a\
F\xb9)\xd1\xb8Bp\xa0\xed\x81\x9a\x8dE\x1a\xb3\xf5\
D\xdd\x9b\xe5l\xd2\xc93\x19\x93\x5c&\xb8\xd7\xac\xd1\
g\xaf\x7f\xbf\xd0]\xb3V\xad\xe1\x92=k\x81\x1a\xd6\
\xde\xfe\xb8\xa75\xe4\xc0\x99A\x1b\x0f\x1a\x9c\xa3\xba\xbe\
z\x10\x9a\xa8\xad\x1c\x18c\x87\x86\x8f`u\x89\x10B\
)\x13L\xb9<B\xd0\xe8\x1e\xa4\x5c\x88\xabf\x85]\
\xcb\x03\x1ex9\xf1.\xe8\x93/\xdcg\x7f\xe0\xc0\x7f\
^\xc1\xec~\xf4#\xe2\xfd\x95\xc6KE\xc7\xbd\x8c\xd0\
\x07\xd4\xff\x9b\xd0O\x92\xb1\xf9w\xe7\xddy\xbf\x01\xba\
\x1cE\xe8\
\x00\x00\x00\xc0\
/\
/ Copyright (C) \
2025 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ap\
ragma Singleton\x0a\
import QtQuick\x0a\x0a\
QtObject {\x0a    p\
roperty bool dis\
abled: false\x0a}\x0a\
\x00\x00\x03\xb9\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aScrollabl\
ePage {\x0a    id: \
page\x0a\x0a    Column\
 {\x0a        spaci\
ng: 40\x0a        w\
idth: parent.wid\
th\x0a\x0a        Labe\
l {\x0a            \
width: parent.wi\
dth\x0a            \
wrapMode: Label.\
Wrap\x0a           \
 horizontalAlign\
ment: Qt.AlignHC\
enter\x0a          \
  text: qsTr(\x22Sw\
itch is an optio\
n button that ca\
n be dragged or \
toggled on or of\
f. \x22\x0a           \
     + \x22Switches\
 are typically u\
sed to select be\
tween two states\
.\x22)\x0a        }\x0a\x0a \
       Column {\x0a\
            spac\
ing: 20\x0a        \
    anchors.hori\
zontalCenter: pa\
rent.horizontalC\
enter\x0a\x0a         \
   Switch {\x0a    \
            enab\
led: !GalleryCon\
fig.disabled\x0a   \
             tex\
t: qsTr(\x22First\x22)\
\x0a            }\x0a \
           Switc\
h {\x0a            \
    enabled: !Ga\
lleryConfig.disa\
bled\x0a           \
     text: qsTr(\
\x22Second\x22)\x0a      \
          checke\
d: true\x0a        \
    }\x0a        }\x0a\
    }\x0a}\x0a\
\x00\x00\x02\xb8\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aScrollabl\
ePage {\x0a    id: \
page\x0a\x0a    Column\
 {\x0a        spaci\
ng: 40\x0a        w\
idth: parent.wid\
th\x0a\x0a        Labe\
l {\x0a            \
width: parent.wi\
dth\x0a            \
wrapMode: Label.\
Wrap\x0a           \
 horizontalAlign\
ment: Qt.AlignHC\
enter\x0a          \
  text: qsTr(\x22Te\
xtField is a sin\
gle-line text ed\
itor.\x22)\x0a        \
}\x0a\x0a        TextF\
ield {\x0a         \
   enabled: !Gal\
leryConfig.disab\
led\x0a            \
placeholderText:\
 qsTr(\x22TextField\
\x22)\x0a            a\
nchors.horizonta\
lCenter: parent.\
horizontalCenter\
\x0a            Acc\
essible.name: qs\
Tr(\x22demo textfie\
ld\x22)\x0a        }\x0a \
   }\x0a}\x0a\
\x00\x00\x06S\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aScrollabl\
ePage {\x0a    id: \
page\x0a\x0a    Column\
 {\x0a        spaci\
ng: 40\x0a        w\
idth: parent.wid\
th\x0a\x0a        Labe\
l {\x0a            \
width: parent.wi\
dth\x0a            \
wrapMode: Label.\
Wrap\x0a           \
 horizontalAlign\
ment: Qt.AlignHC\
enter\x0a          \
  text: qsTr(\x22Co\
mboBox is a comb\
ined button and \
popup list. It p\
resents \x22\x0a      \
          + \x22a l\
ist of options t\
o the user that \
occupies minimal\
 screen space.\x22)\
\x0a        }\x0a\x0a    \
    ComboBox {\x0a \
           enabl\
ed: !GalleryConf\
ig.disabled\x0a    \
        model: [\
qsTr(\x22First\x22), q\
sTr(\x22Second\x22), q\
sTr(\x22Third\x22)]\x0a  \
          anchor\
s.horizontalCent\
er: parent.horiz\
ontalCenter\x0a    \
    }\x0a\x0a        L\
abel {\x0a         \
   width: parent\
.width\x0a         \
   wrapMode: Lab\
el.Wrap\x0a        \
    horizontalAl\
ignment: Qt.Alig\
nHCenter\x0a       \
     text: qsTr(\
\x22ComboBox can be\
 made editable. \
An editable comb\
o box auto-\x22\x0a   \
             + \x22\
completes its te\
xt based on what\
 is available in\
 the model.\x22)\x0a  \
      }\x0a\x0a       \
 ComboBox {\x0a    \
        id: comb\
oBox\x0a\x0a          \
  enabled: !Gall\
eryConfig.disabl\
ed\x0a            e\
ditable: true\x0a  \
          model:\
 ListModel {\x0a   \
             Lis\
tElement { text:\
 qsTr(\x22Banana\x22) \
}\x0a              \
  ListElement { \
text: qsTr(\x22Appl\
e\x22) }\x0a          \
      ListElemen\
t { text: qsTr(\x22\
Coconut\x22) }\x0a    \
        }\x0a      \
      onAccepted\
: {\x0a            \
    if (find(edi\
tText) === -1)\x0a \
                \
   comboBox.mode\
l.append({text: \
comboBox.editTex\
t})\x0a            \
}\x0a            an\
chors.horizontal\
Center: parent.h\
orizontalCenter\x0a\
        }\x0a    }\x0a\
}\x0a\
\x00\x00\x04\xea\
\x00\
\x00\x1a\xf5x\x9c\xd5XmO\xdc8\x10\xfe\x9e_\xe1\
\xa6_@\xd7\x0dmA\xaa\x94Su\x82\xa5*HK\
\xcbKu=\xe9t\x1fL2\xbbk\xe1\xb5S\xc7\x01\
\xb6\x15\xff\xfd\xec\xbc\xe1$N\xe2\xb4\xe8\xc4\xcd\x87\xd5\
\xda3\xf3$\xf3x\xc6\x1ego\x0f\xcdy\xb2\x15d\
\xb5\x96hg\xbe\x8b\xde\xbe~\xf3\x0e}Y\x03\xba\x90\
J\xb3I0\xdb\xa2\x85\x8c\x03oo\x0f]\x9d\x1f\xff\
5[\x90\x08X\x0a\xb3\xd3\x18\x98$K\x02\x22D\xe5\
\xdc%,g\x17r\xa6\xdc6 \x22\x82)\xfa|\x89\
\x8e\xae\x8eg\xfb\xb39\xc5Y\x0a\x9e\x97\x08\xbc\xda\xe0\
\x1c\x993\x05p\x04k|K\xb8\xc28\xe2\x19\x8b=\
\x8f(\x85\x90\xea\xe9\x17\x19\x89nZ\xc3`\x81\xb7<\
\x93i{z\xce\x99\x14\x9c\xa6\x9ew\x8e\x19\xa0\x1f\x1e\
R2\xe74\xdb\xb0\xc2\xa3\x9c\xd2\x92&8\x22l\x15\
\xa2\x83\xd7\xf5\x1cf\xd1\x9a\x8b4X\x12JC\x94`\
\xa1\xde\xac\xa3\x94<9\xc3bEX\xa8H\xf2j\xf5\
\x02_\x035\xf0\x8b9\xfd\xd0\x1c\xee+\x89\xe5:D\
Rd\xd00\xb9\x1389\xe31\x84\x05@\xf0U\x8d\
\x1b\x06\xea\x99\xe4\xbb\x8a\x0b\xd3CJVl\xa3^)\
T\x01\x07\xf9\xe8d\xae\x86 \x1a\x0e\x12\xee\x95\xc5\xb7\
\xf4\x8b\xd8\xf1\x8f\x81\xc2\x0aK@Q\xc9\x0cR1!\
\xb5\x021\xc2)\x8aKm\x8a\x08C\xb7\x04\xeeR\x94\
f\xd1Z\xab\x16$\x95\x7f\xaa\x99\xc0\xdf\xad\xc1\x1f\x1e\
\x83=\xca\xa4\xe4\xec\xa3\xe0Y\xd2\x0a\x99\xc4!\x128\
&\xdc0\xb1!T\x0f\xb0\xb8\xd3R\xd5PD\x94$\
\x16\xfaR\x88$\xe1,H\x04O@\xc8m\x88|\xb9\
M\xc0\xb7\x1aU\xf1\x86\xc8H\x8f\xf6\xd3Kcm\xd1\
\xd1\x0b\xf8\x96\x11\xa1\xc8\xab\x1e\x87R)T\x12UN\
\x1d\x87\xbbb\xd5k65\xc9A>\xd91]\x83\xae\
\xbd\xfa\xf9E6\xa8\xfc\xa6$\x22\xf2$W\xa2\xdft\
\xc6\xb5\x1dm\x89g\x09(7\xb3\x1a\x15\x09c\xc4\x1d\
\xf4\x85\xa3\xa5*\x83(\xcf\xbcS\xd6\xa9\x93J\x1e\xbc\
\xe6hje\x18&'%7\xb9M\xc3h\xa3J\x87\
\x16\x04\xeb*\xb2\xf1\xa0u\x1f(\xe8\xc2A?\x90\xce\
\x0e\x95$\xa7\x126Uq\xf8\xbf\xa3[L3\xa8\x8a\
\xc6\xd4\xbd\xf1w[\x81\xfc2\xe4\xdb\xa7\x87\xdcw\x87\
\xbc\xba#\x09\xf4a6\x94\x13Bw\x07\x9d\x10\xbc;\
\xe8\x84\xf0\xe7k\x88n\xfa@\x1b\xca\x09\xe1\xbb\x83N\
\x08\xdf\x1dtB\xf8\x97zg\xee\x03m('\x84\xef\
\x0e:!|w\xd0i\xc9/\xa3\xf5@N\x19\xdai\
\xe9\xef\x0c;\xad\x00\x9ca\xbb$\xb46\xdc\xc7\xb3o\
\xc1q\x0c\xa2\xe7\xf4\xab\xcc\x0a\xa3_9\xcfR\x9e\x89\
\x08\xea\x06\xef\x11\xba\x9e:\xc3\xc9\xdf:\xd4\x7f<\xf7\
\x836\xa7\xc0\xdd\x5c\xc3;X\xdfbQ\x9c$\x0e\xb6\
D-\x11a1\xdcw\xdf\xba\xb6\xa9\x9b\x1b\xcd\x8f\xd1\
\xd0X\xe0q\xcc\x19\xdd6_\xc5\xc6T\xd8s\xc27\
\xcf\x88\x10\x11cX\xfb\xbf\xb2\xbb6wX\xd5\x02\x98\
\xe31\xe7\xe6\xfe\x14\xa2\xc8\x1c\x8f97\xab\xbb\xec\x18\
'\xbc\xb6Y\x17\xf9{\x1b\x13\xb5\xbb\xa5#\xe9L\xd5\
\xc6\x03\x0d\x94\x95\xd2.\x94\x16s1z\x10\xb5\x00\xc3\
\xd7\x14\x14\xf4\x8b\x8f\x98R\x10[uqY\x92U\x10\
\x934W\xf4:\x16\xadZ\xb3H\x03{MTR\x96\
l\xcb\xc7^\xb3Z\xba\x9b\xd3O\xd2f\xcf&;o\
\x8dL\x1c \xae\x83\xfb\xec)\xeeu\x1a\xa3\xd0\x0cY\
\xc0\x86\xdf\x8eqX\xc9\xa5j\xde1[\xd1!\x1e+\
\x898\xd5w\xee\x06\xfd\xea.\x05\xa9\xbe!\xfe\x81\xfc\
\x97\xfb\xfb\xfb>Rg\xd1\xcb\x83\x83\x03\x7f\x14\xae$\
\xa2\xb8\x10\x0c\xe4\x98)\xd5\xbd\xa7t*\x86\xe3/\xfe\
x\x1d\x1c\xb5mF\xc7\xd9\x5c]\xa9ntj\x8c\xf3\
\xa3\x85,\xd1N;\x1d\xf4\xde\xfe\xe2\xfd{\x94\xa9\x83\
`I\x18\xc4\xbbNPZ,PA~\xf6\x04\xc5*\
\xb7\x9f\x95\x9f5\xe3\xf0\x96\x22m\xcb\xd0E\xb1-K\
\xae\xd6\x22!\xf7@\xaf\xc8wh\x15]\xd0\xd4:\x01\
\x9a_%.\xf3@}7\xce\xca\x14\xf5\xef\xd6j\x1f\
\x1eOA-\xceW\xd4\xb6t\xf7>7\xed\x00\xf9W\
\xaa\x81\xd0\x1f\xc80=dd\x83\xf5\xb5\xda\xb9\xe0k\
\x8f\xe1\xb5=/{\x87\xc3\xc8\x01]\x8b\xc4b\x05\x9d\
Mn\xd4\xcd\xf8\xc6R7\x80\x0a\x02o\xcb\xf5\x1c\x05\
(\xfb\xd7\xceM\xbf-\xc3\xcb\xf0)\xdb\x5c\x83pe\
SK\x15\xaf\xdb\xc9Q\x89\x11n\xb1)\x8d\x07(y\
\x88\xba\x1fh\xda\x028U\xbdiPt\xf9\x1f\x8a\xc1\
)\xfb\x9c\xc9\x8b\x0c\xf7\x9fKZ\x86\x89\xf9\xbf\xe5\xc1\
\x12\xd3\xf4g\x13a\xa0\xe2\xf2U\x0e(,e\xf7\xe0\
\x1c\xf6\x11\xc5I\xe4\xeaT\x07\xcfY\x11y\xa7n\x83\
TQ,w\xec\x1b\xdd\x935Z\xf6\xce\xdbNP\xa3\
k\x7f\x06\x1d\xea\x93\x91`\xbfA\xd8Ih\xdc>\xfe\
c\x12z}\x8c\xef\xe5\xc1J\xff\x0e|G7\xe5)\
\xfbu\xeb-\xaa\xafa7\x8c\x9fi\x22\xd9G\xc5\xbf\
\x07\xef\xc1\xfb\x17\xe7\x1f\xdf\x92\
\x00\x00\x03*\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aScrollabl\
ePage {\x0a    id: \
page\x0a\x0a    Column\
 {\x0a        spaci\
ng: 40\x0a        w\
idth: parent.wid\
th\x0a\x0a        Labe\
l {\x0a            \
width: parent.wi\
dth\x0a            \
wrapMode: Label.\
Wrap\x0a           \
 horizontalAlign\
ment: Qt.AlignHC\
enter\x0a          \
  text: qsTr(\x22Th\
e Dial is simila\
r to a tradition\
al dial knob tha\
t is found on de\
vices such as \x22\x0a\
                \
+ \x22stereos or in\
dustrial equipme\
nt. It allows th\
e user to specif\
y a value within\
 a range.\x22)\x0a    \
    }\x0a\x0a        D\
ial {\x0a          \
  enabled: !Gall\
eryConfig.disabl\
ed\x0a            v\
alue: 0.5\x0a      \
      anchors.ho\
rizontalCenter: \
parent.horizonta\
lCenter\x0a        \
    Accessible.n\
ame: qsTr(\x22demo \
dial\x22)\x0a        }\
\x0a    }\x0a}\x0a\
\x00\x00\x05-\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aScrollabl\
ePage {\x0a    id: \
page\x0a\x0a    readon\
ly property int \
itemWidth: Math.\
max(button.impli\
citWidth, Math.m\
in(button.implic\
itWidth * 3, pag\
e.availableWidth\
 / 3 * 2))\x0a\x0a    \
Column {\x0a       \
 spacing: 40\x0a   \
     width: pare\
nt.width\x0a\x0a      \
  Label {\x0a      \
      width: par\
ent.width\x0a      \
      wrapMode: \
Label.Wrap\x0a     \
       horizonta\
lAlignment: Qt.A\
lignHCenter\x0a    \
        text: qs\
Tr(\x22Frame is use\
d to layout a lo\
gical group of c\
ontrols together\
, within a visua\
l frame.\x22)\x0a     \
   }\x0a\x0a        Fr\
ame {\x0a          \
  enabled: !Gall\
eryConfig.disabl\
ed\x0a            a\
nchors.horizonta\
lCenter: parent.\
horizontalCenter\
\x0a\x0a            Co\
lumn {\x0a         \
       spacing: \
20\x0a             \
   width: page.i\
temWidth\x0a\x0a      \
          RadioB\
utton {\x0a        \
            text\
: qsTr(\x22First\x22)\x0a\
                \
    checked: tru\
e\x0a              \
      width: par\
ent.width\x0a      \
          }\x0a    \
            Radi\
oButton {\x0a      \
              id\
: button\x0a       \
             tex\
t: qsTr(\x22Second\x22\
)\x0a              \
      width: par\
ent.width\x0a      \
          }\x0a    \
            Radi\
oButton {\x0a      \
              te\
xt: qsTr(\x22Third\x22\
)\x0a              \
      width: par\
ent.width\x0a      \
          }\x0a    \
        }\x0a      \
  }\x0a    }\x0a}\x0a\
\x00\x00\x04\xc1\
/\
/ Copyright (C) \
2025 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aPage {\x0a  \
  id: page\x0a    e\
nabled: !Gallery\
Config.disabled\x0a\
\x0a    header: Men\
uBar {\x0a        M\
enu {\x0a          \
  title: qsTr(\x22&\
File\x22)\x0a         \
   Action { text\
: qsTr(\x22&New...\x22\
) }\x0a            \
Action { text: q\
sTr(\x22&Open...\x22) \
}\x0a            Ac\
tion { text: qsT\
r(\x22&Save\x22) }\x0a   \
         Action \
{ text: qsTr(\x22Sa\
ve &As...\x22) }\x0a  \
          MenuSe\
parator { }\x0a    \
        Action {\
 text: qsTr(\x22&Qu\
it\x22) }\x0a        }\
\x0a        Menu {\x0a\
            titl\
e: qsTr(\x22&Edit\x22)\
\x0a            Act\
ion { text: qsTr\
(\x22Cu&t\x22) }\x0a     \
       Action { \
text: qsTr(\x22&Cop\
y\x22) }\x0a          \
  Action { text:\
 qsTr(\x22&Paste\x22) \
}\x0a        }\x0a    \
    Menu {\x0a     \
       title: qs\
Tr(\x22&Help\x22)\x0a    \
        Action {\
 text: qsTr(\x22&Ab\
out\x22) }\x0a        \
}\x0a    }\x0a\x0a    Lab\
el {\x0a        anc\
hors.verticalCen\
ter: parent.vert\
icalCenter\x0a     \
   width: parent\
.width\x0a        w\
rapMode: Label.W\
rap\x0a        hori\
zontalAlignment:\
 Qt.AlignHCenter\
\x0a        text: q\
sTr(\x22MenuBar pro\
vides a horizont\
al bar with drop\
-down menus, \x22\x0a \
                \
+ \x22allowing user\
s to access grou\
ped commands and\
 actions \x22\x0a     \
            + \x22w\
ithin an applica\
tion.\x22)\x0a    }\x0a}\x0a\
\
\x00\x00\x05q\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aPane {\x0a  \
  id: pane\x0a\x0a    \
SwipeView {\x0a    \
    id: view\x0a   \
     currentInde\
x: 1\x0a        anc\
hors.fill: paren\
t\x0a        enable\
d: !GalleryConfi\
g.disabled\x0a\x0a    \
    Repeater {\x0a \
           model\
: 3\x0a\x0a           \
 Pane {\x0a        \
        width: S\
wipeView.view.wi\
dth\x0a            \
    height: Swip\
eView.view.heigh\
t\x0a\x0a             \
   Column {\x0a    \
                \
spacing: 40\x0a    \
                \
width: parent.wi\
dth\x0a\x0a           \
         Label {\
\x0a               \
         width: \
parent.width\x0a   \
                \
     wrapMode: L\
abel.Wrap\x0a      \
                \
  horizontalAlig\
nment: Qt.AlignH\
Center\x0a         \
               t\
ext: qsTr(\x22Swipe\
View provides a \
navigation model\
 that simplifies\
 horizontal page\
d scrolling. \x22\x0a \
                \
       + \x22The pa\
ge indicator on \
the bottom shows\
 which is the pr\
esently active p\
age.\x22)\x0a         \
           }\x0a\x0a  \
                \
  Image {\x0a      \
                \
  source: \x22../im\
ages/arrows.png\x22\
\x0a               \
         anchors\
.horizontalCente\
r: parent.horizo\
ntalCenter\x0a     \
               }\
\x0a               \
 }\x0a            }\
\x0a        }\x0a    }\
\x0a\x0a    PageIndica\
tor {\x0a        co\
unt: view.count\x0a\
        currentI\
ndex: view.curre\
ntIndex\x0a        \
anchors.bottom: \
parent.bottom\x0a  \
      anchors.ho\
rizontalCenter: \
parent.horizonta\
lCenter\x0a    }\x0a}\x0a\
\
\x00\x00\x05S\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aFlickable\
 {\x0a    id: flick\
able\x0a    enabled\
: !GalleryConfig\
.disabled\x0a    co\
ntentHeight: pan\
e.height\x0a\x0a    Pa\
ne {\x0a        id:\
 pane\x0a        wi\
dth: flickable.w\
idth\x0a        hei\
ght: flickable.h\
eight * 1.25\x0a\x0a  \
      Column {\x0a \
           id: c\
olumn\x0a          \
  spacing: 40\x0a  \
          width:\
 parent.width\x0a\x0a \
           Check\
Box {\x0a          \
      id: always\
OnCheckBox\x0a     \
           width\
: parent.width\x0a \
               t\
ext: qsTr(\x22Alway\
s on\x22)\x0a         \
   }\x0a\x0a          \
  Label {\x0a      \
          width:\
 parent.width\x0a  \
              wr\
apMode: Label.Wr\
ap\x0a             \
   horizontalAli\
gnment: Qt.Align\
HCenter\x0a        \
        text: qs\
Tr(\x22ScrollBar is\
 an interactive \
bar that can be \
used to scroll t\
o a specific pos\
ition. \x22\x0a       \
             + \x22\
A scroll bar can\
 be either verti\
cal or horizonta\
l, and can be at\
tached to any Fl\
ickable, \x22\x0a     \
               +\
 \x22such as ListVi\
ew and GridView.\
\x22)\x0a            }\
\x0a\x0a            Im\
age {\x0a          \
      rotation: \
90\x0a             \
   source: \x22../i\
mages/arrows.png\
\x22\x0a              \
  anchors.horizo\
ntalCenter: pare\
nt.horizontalCen\
ter\x0a            \
}\x0a        }\x0a    \
}\x0a\x0a    ScrollBar\
.vertical: Scrol\
lBar {\x0a        p\
olicy: alwaysOnC\
heckBox.checked \
? ScrollBar.Alwa\
ysOn : ScrollBar\
.AsNeeded\x0a    }\x0a\
}\x0a\
\x00\x00\x040\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aScrollabl\
ePage {\x0a    id: \
page\x0a\x0a    Column\
 {\x0a        spaci\
ng: 40\x0a        w\
idth: parent.wid\
th\x0a\x0a        Labe\
l {\x0a            \
width: parent.wi\
dth\x0a            \
wrapMode: Label.\
Wrap\x0a           \
 horizontalAlign\
ment: Qt.AlignHC\
enter\x0a          \
  text: qsTr(\x22Ra\
ngeSlider is use\
d to select a ra\
nge specified by\
 two values, by \
sliding each han\
dle along a trac\
k.\x22)\x0a        }\x0a\x0a\
        RangeSli\
der {\x0a          \
  enabled: !Gall\
eryConfig.disabl\
ed\x0a            f\
irst.value: 0.25\
\x0a            sec\
ond.value: 0.75\x0a\
            anch\
ors.horizontalCe\
nter: parent.hor\
izontalCenter\x0a  \
          Access\
ible.name: qsTr(\
\x22demo range slid\
er\x22)\x0a        }\x0a\x0a\
        RangeSli\
der {\x0a          \
  enabled: !Gall\
eryConfig.disabl\
ed\x0a            o\
rientation: Qt.V\
ertical\x0a        \
    first.value:\
 0.25\x0a          \
  second.value: \
0.75\x0a           \
 anchors.horizon\
talCenter: paren\
t.horizontalCent\
er\x0a            A\
ccessible.name: \
qsTr(\x22vertical d\
emo slider\x22)\x0a   \
     }\x0a    }\x0a}\x0a\
\x00\x00\x05b\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aScrollabl\
ePage {\x0a    id: \
page\x0a\x0a    readon\
ly property int \
itemWidth: Math.\
max(button.impli\
citWidth, Math.m\
in(button.implic\
itWidth * 3, pag\
e.availableWidth\
 / 3 * 2))\x0a\x0a    \
Column {\x0a       \
 spacing: 40\x0a   \
     width: pare\
nt.width\x0a\x0a      \
  Label {\x0a      \
      width: par\
ent.width\x0a      \
      wrapMode: \
Label.Wrap\x0a     \
       horizonta\
lAlignment: Qt.A\
lignHCenter\x0a    \
        text: qs\
Tr(\x22A GroupBox p\
rovides a frame,\
 a title on top \
of it, and a log\
ical group of co\
ntrols within th\
at frame.\x22)\x0a    \
    }\x0a\x0a        G\
roupBox {\x0a      \
      enabled: !\
GalleryConfig.di\
sabled\x0a         \
   title: qsTr(\x22\
Title\x22)\x0a        \
    anchors.hori\
zontalCenter: pa\
rent.horizontalC\
enter\x0a\x0a         \
   Column {\x0a    \
            spac\
ing: 20\x0a        \
        width: p\
age.itemWidth\x0a\x0a \
               R\
adioButton {\x0a   \
                \
 text: qsTr(\x22Fir\
st\x22)\x0a           \
         checked\
: true\x0a         \
           width\
: parent.width\x0a \
               }\
\x0a               \
 RadioButton {\x0a \
                \
   id: button\x0a  \
                \
  text: qsTr(\x22Se\
cond\x22)\x0a         \
           width\
: parent.width\x0a \
               }\
\x0a               \
 RadioButton {\x0a \
                \
   text: qsTr(\x22T\
hird\x22)\x0a         \
           width\
: parent.width\x0a \
               }\
\x0a            }\x0a \
       }\x0a    }\x0a}\
\x0a\
\x00\x00\x04A\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aScrollabl\
ePage {\x0a    id: \
page\x0a\x0a    Column\
 {\x0a        spaci\
ng: 40\x0a        w\
idth: parent.wid\
th\x0a\x0a        Labe\
l {\x0a            \
width: parent.wi\
dth\x0a            \
wrapMode: Label.\
Wrap\x0a           \
 horizontalAlign\
ment: Qt.AlignHC\
enter\x0a          \
  text: qsTr(\x22Ra\
dioButton presen\
ts an option but\
ton that can be \
toggled on or of\
f. \x22\x0a           \
     + \x22Radio bu\
ttons are typica\
lly used to sele\
ct one option fr\
om a set of opti\
ons.\x22)\x0a        }\
\x0a\x0a        Column\
 {\x0a            s\
pacing: 20\x0a     \
       anchors.h\
orizontalCenter:\
 parent.horizont\
alCenter\x0a\x0a      \
      RadioButto\
n {\x0a            \
    text: qsTr(\x22\
First\x22)\x0a        \
        enabled:\
 !GalleryConfig.\
disabled\x0a       \
     }\x0a         \
   RadioButton {\
\x0a               \
 text: qsTr(\x22Sec\
ond\x22)\x0a          \
      checked: t\
rue\x0a            \
    enabled: !Ga\
lleryConfig.disa\
bled\x0a           \
 }\x0a            R\
adioButton {\x0a   \
             tex\
t: qsTr(\x22Third\x22)\
\x0a               \
 enabled: false\x0a\
            }\x0a  \
      }\x0a    }\x0a}\x0a\
\
\x00\x00\x04\x7f\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aFlickable\
 {\x0a    id: flick\
able\x0a    enabled\
: !GalleryConfig\
.disabled\x0a    co\
ntentHeight: pan\
e.height\x0a\x0a    Pa\
ne {\x0a        id:\
 pane\x0a        wi\
dth: flickable.w\
idth\x0a        hei\
ght: flickable.h\
eight * 1.25\x0a\x0a  \
      Column {\x0a \
           id: c\
olumn\x0a          \
  spacing: 40\x0a  \
          width:\
 parent.width\x0a\x0a \
           Label\
 {\x0a             \
   width: parent\
.width\x0a         \
       wrapMode:\
 Label.Wrap\x0a    \
            hori\
zontalAlignment:\
 Qt.AlignHCenter\
\x0a               \
 text: qsTr(\x22Scr\
ollIndicator is \
a non-interactiv\
e indicator that\
 indicates the c\
urrent scroll po\
sition. \x22\x0a      \
              + \
\x22A scroll indica\
tor can be eithe\
r vertical or ho\
rizontal, and ca\
n be attached to\
 any Flickable, \
\x22\x0a              \
      + \x22such as\
 ListView and Gr\
idView.\x22)\x0a      \
      }\x0a\x0a       \
     Image {\x0a   \
             rot\
ation: 90\x0a      \
          source\
: \x22../images/arr\
ows.png\x22\x0a       \
         anchors\
.horizontalCente\
r: parent.horizo\
ntalCenter\x0a     \
       }\x0a       \
 }\x0a    }\x0a\x0a    Sc\
rollIndicator.ve\
rtical: ScrollIn\
dicator { }\x0a}\x0a\
\x00\x00\x05\xcd\
\x00\
\x00\x18\xbax\x9c\xcdXKo\x1b7\x10\xbe\xebWL\
}\xb2\x1b\x8bJ\xec\x02\x05t)\x129i\x0c\xd8\xb1\
\x13;M\x0b\xf4B\xed\x8e$\xd6\x5crMrm+\
\xa9\xff{\x87\xa4\x1e\xab}IH\xda:s\x90\x96\xcb\
\x19\xce\xeb\xe3p\xb8\x83\x01\x8ct>7b:s\xb0\
?:\x80\xa3\xe7/~\x86\xeb\x19\xc2{G3Y\xce\
\xd5\x1c\xce\x5c\xcaz\x83\x01\x5c]\x9e\xfc\xde?\x13\x09\
*\x8b\xfd\xd3\x14\x95\x13\x13\x81f\x08\x8bw\x1fp\xd2\
\x7f\xef\xfa$\x96\xa1I\x04\x97p\xf1\x01^]\x9d\xf4\
\x8f\xfb#\xc9\x0b\x8b\xbd\x9e\xc8rm\x1c-\xfe\xbe\x10\
\xc9Me\xc8\xce\xf8\x5c\x17\xceV_\x8f\xb4rFK\
\xdb\xeb]%\xf4/\xf9X\xe2%\x9f\x22|\xe9\x01\x91\
H\x87\x90\xd3\xb0\x17F\x06y\xaa\x95\x9cCnt\x8e\
\xc6\xcdA(\x07\xe3\xc29\xad>\x89\xd4\xcd\x86p\xce\
\xdd\x8ce\xfca?\xbee\xa4N\x8aD\xb80}\xb8\
\x98\x16\xaaq\x1a~\x84\xa3\xc3\xa0\x8e\xf1;.\x82-\
qb\x00\xc7\x07\x07\xd1\x86\x91\x96E\xa6\x16\xe6y\xb2\
9O\x84\x9a\x0e\xe1\xa7\xe7\xabw\xf7\xd1\x98\x9c\x1b\x0a\
$\x0b\xa3\xdej\xf2\x8c\x8fQ\x96\x16h\x13\xd8\x987\
<?\xd7)\x0e\xa34\xfbD\xe3\x0d\x86\x996\xe23\
\xc5\x92\xcb\x97RLUF\xcb\x0c)\xc8,\x8c\xde\x8e\
h\x88fC\xc0\xe1\x03q\xdc\xdak\xb3\xbfwB\xf9\
\xd4S\x10\x168\xe4:/rp3\xee\xfc8\xd3\xd6\
Q\xbc)\xbf)L\xb4\x01Kz\x5c\x9f\xd6\xca\xc0q\
{caocQO\xcf`\x8f\xab\x14\xc6F\xe0\x04\
\x12\xc2K\xa1D\xc2\x9d\xd0\xca\x92\x9b\x14LG\x08\xa4\
\x05\x0d\xdb;X\x09?\xae\xc3\xf3*d\xa6\x12\x9f\xb2\
\xb5\xe7h-\xe5\xa8$\xed\x89\xab\x84l\xb3l\x1d\x88\
\xe8\xf4*\xa8\xd5\x89\xe6\xf8S\xeeKx\xda\xe0\xd1j\
DX\xb9A\x82d\x16M\x88qc\x84E\xb5\x7f\xd0\
\xdb`^\x84\xf4K-<\xa2*^\xe3@\xe5\x81G\
l?\xfc\xca\xa5D3\xa7=2\x11S\x96\x0a\x1b&\
z5\x89\x87!\xec\x97\xa1\x03\xfd\xe8\xd0\x01\x01\xf7\xa8\
\xc6=_s\xcf0\x14\x87>\xc4\x87\xc8_\x13p\xc2\
I\xacG\xbf\xc6\xd7\x04\xec\xd5\x1a\xa5\x04\x9ei\x83\x19\
\x88\xdc\x16\x19\xa4ZzX\x09\x07<C\xc7\x18\xab\xa4\
\xd5\xd3c\xafy\xb4\x0d4>\xd41\x99\xadX\x0a\xa1\
5Y\x80\xe7S\x02*)\xd9\xf1\xb5\xa8\xaa\xaf\xf1\xdd\
A\xab\xca\x1fy\x87pq\x87F\xf29\xd3\xf1\xbfn\
F\xa6S.\x87\xe0L\x81\xdd\xf0\xecH\xa8'\xeb\xa8\
6q\x93F\xbc\xd8\xe1\x22\xa2\xec\x0f\xb4\xf0\xf7r\xf0\
N\xd7\x0d\xa8\x95\xfd\x8dU\x97G\xc0\xd1\xf3\xc6\xf9%\
\x94&B\xca%|\x1a\x19\xbbvPp\xb5\x04]\x7f\
\x90\xa7:)|\xa9\x87\x19\xb70FT>P\xfe\xe0\
N\xd9\x9f\xeaD\x03\x1d\xb9p\xcfi\xdei\xb0\xfc\x0e\
\xfd\x0b\x03\xc9\x8c\xab)\xda_\x1a\xe2\xe3\xe9\xb1\xf1\xed\
h\x86\xc9\xcd+\xfd\xb0\xa3u\xa4\x5ci\xda\xd4\xf6\x06\
\xf8\x94\x8b\xa6\x5cT\xa3\x13\x1a\x95\xd5\xee\x0a\xa3\x1d\xed\
\xfb\xda\x02Q\xa9\x04\x8e\xf4>q\x11\xf0&|\xc3\xfe\
_\x8b\x7f\xf7[\xff\xbe\xdc\xafQC\x16\x82t\x1f\xbb\
\xb4\xf0\xbc\x16>\xf6\x8dYm\x81\x85\xbbo1\xc2\x86\
|\xd6K\xcdM\xec\xffI\xa9i@\x8c\xa7\xb6*3\
\x92\xdab]\xe3\x1b\x0f\x00\x9f\x84\x96\xbd\xe5\x93;Y\
\xf24r$R\xe4-&{\xda\xb9\xfeTb\x9a\x84\
\xa2\xb7\x88j\xddnO\x9duqi|\x5c\xa7\x95e\
[\xf9\xf4\xd4\xd9QW\xe94[\xdf \xba\xec\xf2\x98\
\xe9dj\xd0\xda\x08\xe62}[\xc9\xa8\x92OYl\
\xfc\x83S\xec\xd2 \xb5\xcew\xf8\xd2\xe6\x98\xb87\xa2\
9\x8fK\xb2T\xee\x13\x92\xddcl \xbc\xbc\x1d\xdc\
\xba~\xd8*\xb9\x9a\xd6{\xf7%=\xb6\xc7v\xdb\x11\
\xe5i\xdbm\xa6J\xbb5\x87\x87\x1e\x9e\x96\xbcFG\
\x87\x18OE.\xac\xc7\x0d\xa0\x14\x8e\xc1\xbbB%\x14\
.%\xc6E\xd3\xb5\xa4JtM\x11\x0a\xd0:\xb8-\
\xe8\xa2#9\xe9\xa5\x06\x14N}JRR\x8e\x0e2\
.\xd1\x16<\xe50!\x03\xe8v\x94Pv\x1d.\xac\
\xcb\x8d\xc8\xc4\xee\xaa&\xbcH\xbcm\x0cF\x85\xe1c\
\xe1\x9d\xc0)i\xf9\xab\xb0\xfex\xa6\xcb\xaf\xbf\x89\x16\
\xa97K\x89l%\x00c1FE&\xb1\xddt]\
\x15\x84\x0dEu\xdd\x22\xdd\xe6\xfc~\x16\x0c~#O\
i1\xda\x87\x90\x14\xc6\xd2\xaa!\x98\xb7\x05\xf7n\x16\
\x86\x1c\x11\xe9\x86\x11;j;\xf1\xd1\x9b\xd0\x06\x96\xc2\
\xd2\xd3\x8c4\x1b4\x94\xb2R\xb2\xca.[\x9e\x0b\xea\
S\x1c\xdd#\xb59\x04\x9cL\x82B\xb3c\x1cS\xa0\
\xa8\x90$(L\xca\xd2\x14G%\xacd\xf0\xd1\xc1\x1d\
*:\xed\x1cY\x83F\x93_\x0a\x94\xc7F.y\x82\
\x86\x13\x8evR\x85\x05p)(@YL\x07\xc5,\
!\xc8\xb9\xa6\x1bJ\x99\xb6^\xd5\xcb\xd4\xdck\xb5l\
\xbf\xf8\x85\xe4\x94RK\xf7im\x18\x9d`\x8e\x1e\xa9\
\xa4Wf:\xb6\xe7\xf2\x18\xdc\xec5\x16\xa3S\x87\xd9\
\xd66\xcd\xe9\xbct\x1c\xf9\xe1V\x91\xb1\xa6s0+\
K\xc57\xffZKX\x939\xe7f*\xd4\x10\xfa\x9b\
~\x86\xb9K\x9e\xa6\xbel<\x83\x17\xffc\x7fy\xaa\
\xf2\xe2I\xbbK\xe1\x0d\xf8\xda\xde\xb2$\xfc\xddw\x96\
;wz\x13\xba=\xd9\x96\xb6i\xe7.\xb0)\xaf\x9e\
\xdaz\xc0\x8b\x9b\xf5EsD\x00@\xd9v\xd9\x8c_\
L\x9f\xfa\xcaI\x87\xeb\xba\x96\xbd\xf6\x83\x0f\x9d\xdb\xb0\
\x8c\xf9K\x89\x9c\x0e\xa1\x00\xdd\xf0\xd9/1\x18>-\
si\x87\x1de4z\x1el_|\xd6mmm\x9b\
\x0b\xe85\x19\xf1F\xa0L;\x1c\xeb\xca\xfe\x92\xc2\x91\
1\xd32Es]\xf2\xeb#\xb5`\x8a:\x82\xa7v\
\xa1\xc5\xbcKn\xed\xbd6i\x87yH\xe8\x88\x87\xd4\
J\x0f[\x8a\xbd\xa6\xb9\x0b\xf5:\xed\xe8-\xbf\xc9\xbb\
\xf6j\x1a\x7f\x1f{\xff\x00\x94ma\x98\
\x00\x00\x03\xd4\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aScrollabl\
ePage {\x0a    id: \
page\x0a\x0a    Column\
 {\x0a        spaci\
ng: 40\x0a        w\
idth: parent.wid\
th\x0a\x0a        Labe\
l {\x0a            \
width: parent.wi\
dth\x0a            \
wrapMode: Label.\
Wrap\x0a           \
 horizontalAlign\
ment: Qt.AlignHC\
enter\x0a          \
  text: qsTr(\x22Ch\
eckBox presents \
an option button\
 that can be tog\
gled on or off. \
\x22\x0a              \
  + \x22Check boxes\
 are typically u\
sed to select on\
e or more option\
s from a set of \
options.\x22)\x0a     \
   }\x0a\x0a        Co\
lumn {\x0a         \
   spacing: 20\x0a \
           ancho\
rs.horizontalCen\
ter: parent.hori\
zontalCenter\x0a\x0a  \
          CheckB\
ox {\x0a           \
     enabled: !G\
alleryConfig.dis\
abled\x0a          \
      text: qsTr\
(\x22First\x22)\x0a      \
          checke\
d: true\x0a        \
    }\x0a          \
  CheckBox {\x0a   \
             ena\
bled: !GalleryCo\
nfig.disabled\x0a  \
              te\
xt: qsTr(\x22Second\
\x22)\x0a            }\
\x0a        }\x0a    }\
\x0a}\x0a\
\x00\x00\x02\xee\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aScrollabl\
ePage {\x0a    id: \
page\x0a\x0a    Column\
 {\x0a        spaci\
ng: 40\x0a        w\
idth: parent.wid\
th\x0a\x0a        Labe\
l {\x0a            \
width: parent.wi\
dth\x0a            \
wrapMode: Label.\
Wrap\x0a           \
 horizontalAlign\
ment: Qt.AlignHC\
enter\x0a          \
  text: qsTr(\x22De\
layButton is a c\
heckable button \
that incorporate\
s a delay before\
 the \x22\x0a         \
       + \x22button\
 is activated. T\
his delay preven\
ts accidental pr\
esses.\x22)\x0a       \
 }\x0a\x0a        Dela\
yButton {\x0a      \
      enabled: !\
GalleryConfig.di\
sabled\x0a         \
   text: qsTr(\x22D\
elayButton\x22)\x0a   \
         anchors\
.horizontalCente\
r: parent.horizo\
ntalCenter\x0a     \
   }\x0a    }\x0a}\x0a\
\x00\x00\x07\xe1\
/\
/ Copyright (C) \
2017 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ap\
ragma ComponentB\
ehavior: Bound\x0a\x0a\
import QtQuick\x0ai\
mport QtQuick.Co\
ntrols\x0a\x0aStackVie\
w {\x0a    id: stac\
kView\x0a    initia\
lItem: page\x0a    \
enabled: !Galler\
yConfig.disabled\
\x0a\x0a    Component \
{\x0a        id: pa\
ge\x0a\x0a        Pane\
 {\x0a            i\
d: pane\x0a        \
    width: paren\
t ? parent.width\
 : 0 // TODO: fi\
x null parent on\
 destruction\x0a\x0a  \
          Column\
 {\x0a             \
   spacing: 40\x0a \
               w\
idth: parent.wid\
th\x0a\x0a            \
    Label {\x0a    \
                \
width: parent.wi\
dth\x0a            \
        wrapMode\
: Label.Wrap\x0a   \
                \
 horizontalAlign\
ment: Qt.AlignHC\
enter\x0a          \
          text: \
qsTr(\x22StackView \
provides a stack\
-based navigatio\
n model which ca\
n be used with a\
 set of interlin\
ked pages. \x22\x0a   \
                \
 + \x22Items are pu\
shed onto the st\
ack as the user \
navigates deeper\
 into the materi\
al, and popped o\
ff again \x22\x0a     \
               +\
 \x22when he choose\
s to go back.\x22)\x0a\
                \
}\x0a\x0a             \
   Button {\x0a    \
                \
id: button\x0a     \
               t\
ext: qsTr(\x22Push\x22\
)\x0a              \
      anchors.ho\
rizontalCenter: \
parent.horizonta\
lCenter\x0a        \
            widt\
h: Math.max(butt\
on.implicitWidth\
, Math.min(butto\
n.implicitWidth \
* 2, pane.availa\
bleWidth / 3))\x0a \
                \
   onClicked: st\
ackView.push(pag\
e)\x0a             \
   }\x0a\x0a          \
      Button {\x0a \
                \
   text: qsTr(\x22P\
op\x22)\x0a           \
         enabled\
: stackView.dept\
h > 1\x0a          \
          width:\
 Math.max(button\
.implicitWidth, \
Math.min(button.\
implicitWidth * \
2, pane.availabl\
eWidth / 3))\x0a   \
                \
 anchors.horizon\
talCenter: paren\
t.horizontalCent\
er\x0a             \
       onClicked\
: stackView.pop(\
)\x0a              \
  }\x0a\x0a           \
     Label {\x0a   \
                \
 width: parent.w\
idth\x0a           \
         wrapMod\
e: Label.Wrap\x0a  \
                \
  horizontalAlig\
nment: Qt.AlignH\
Center\x0a         \
           text:\
 qsTr(\x22Stack Dep\
th:\x22) + \x22 \x22 + st\
ackView.depth\x0a  \
              }\x0a\
            }\x0a  \
      }\x0a    }\x0a}\x0a\
\
\x00\x00\x01?\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00(\x00\x00\x00(\x04\x03\x00\x00\x00~\xd0\xa5^\
\x00\x00\x000PLTE\x00\x00\x00\xff\xff\xffA\xcd\
RP\xcaNA\xcdRA\xcdRH\xcbPB\xccQ\
I\xcbPD\xccQE\xccQE\xccQD\xccQD\
\xccQC\xccQA\xcdR\xb0DE*\x00\x00\x00\x0f\
tRNS\x00\x00\x186Oc\xab\xae\xb8\xbd\xc9\xd6\
\xdf\xe9\xf0r\xc1;\x05\x00\x00\x00\xafIDAT(\
\xcf\x8d\xd2\xb1\x0d\xc20\x14\x04\xd0[\x81\x0d~\x81\x98\
\x81\x01\xe8-e\x01F@\xf2\x14t\xec\xe0E\xd8!\
\xca\x08\xac\x10YXXG\x11'v\xf0\x15\x5c\xf9\xf4\
\x8b\xff\xed\x03\xd6\xb8\x80>.\x99@\x06\x85\xd9\x04r\
T\xd8\x8f:\x92/\x13\xc8A\xe1l\x02\x7fG\x17\x8c\
\x06\x00\xe7{\xc9\x93$I\x0f\x00\x0f\xee\x13M \xbd\
\xc2d\x02\x19\x14&\x13\xc8\xa00+\xe4\xdf8+\x1c\
\x04Nb\xa5\xac.\x1a\xc5\xed\x1f\x03p\xeb/\x07\x8e\
\x97\x92\xeb\xf6F\xddw,\xd59\xacq$\xf96\x81\
\x1e=F\x13X\x06w\xb8\xf5\xa3\xc5\xad\x1e\x0d\xd6\x22\
5X{Tq\xb2\x1eO\xd5\xbe:U\x87P(\x16\
\xe3k\x00\x00\x00\x00IEND\xaeB`\x82\
\x00\x00\x01\xca\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00@\x00\x00\x00;\x04\x03\x00\x00\x009\xa2\x91\xef\
\x00\x00\x000PLTE\x00\x00\x00A\xcdRP\xca\
NA\xcdRA\xcdRF\xccPH\xcbPB\xccQ\
I\xcbPD\xccQE\xccQE\xccQD\xccQD\
\xccQC\xccQA\xcdR\x96\x12X7\x00\x00\x00\x0f\
tRNS\x00\x186Oc\x9d\xab\xae\xb8\xbd\xc9\xd6\
\xdf\xe9\xf0URDE\x00\x00\x01:IDAT8\
\xcb\x85\xd4\xcdM\xc40\x10\x05\xe0\x99\x0e,\x05\x11)\
\x88;\xc7E\x14`QAj\xa0\x02D\x05{B\xe2\
F\x0d[\x81K\xa0\x06D\x019P\xc0\x1e\x12\x09\xf1\
\xb3z\x1c\x9c\xc4\xcex\xd63\xd7|yR</&\
\xd2\xc6\x1f\xa8>\xfe\xc7\x19\x00\x07\x0b\x9c\x9c\x01\xf0n\
\x81z\x84\x07\xf0\xe9\x0c\x80{\x0bL\xce\x00\xb5\x88\x08\
\xbe\x8a\x88\x9b\x97y\xde\x00\x00x\x92\xe0\x15\xdb)\x22\
$(\x22\x0a wV\x00\xb9\xb3\x12\x88\x88\x12\x88\x08\
\x05\x9c\xd2S~\xd0\x00\x12\xe8\xc6:\xe0\xa0\x82i\x05\
\xd7P\xc1\xba0\x1eT\xf0\xe1R\x80\x06R\xef\x9a\xa3\
\x0a\xd6\xe6r\x0f\x0d\xfc\xad\x01\x17\x000\xd2\xe3\xb9c\
\xe4>\x82\xe6n\x9e^,\xa2E\x04\xa2r)`\x1f\
\xcfdy\xff6\x82o\x11\x90\x7f\x9c\xdf\xd4i\x0e\xc8\
f\xe7\x91\x17\xb2\x83\x0aR@P\xc1T\x09\xc0\xce\xe7\
[\x0a*\x98\xf2-i \xfd\x97\x83\x0a\xd2\x9a\x09\x1a\
\xb8\xcc\xda\x1e4\x90W\xbd\xb5@y\x8e\x02h\x11[\
\xa0DlA\xacS\x0d\xc4>U\xc0\x12\xf1\xbb\xdcP\
\xcf\xf2\xe6\x99#F:;\xcd`\x80\xb8\xb1\x1a\xe0\xc1\
\x00te\x01\x0e\x06\xa0\xce\x02\x1c\x0c@\xad\x05xo\
\x00ju\xf0\x0fRR\x00X\xeb\xfc\xd3\xe9\x00\x00\x00\
\x00IEND\xaeB`\x82\
\x00\x00%!\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x03 \x00\x00\x02;\x08\x03\x00\x00\x00#\xc8\x0f\xb6\
\x00\x00\x00\x09pHYs\x00\x004\xba\x00\x004\xba\
\x010P\xac\x88\x00\x00\x00\x19tEXtSof\
tware\x00www.inksca\
pe.org\x9b\xee<\x1a\x00\x00\x03\x00PL\
TE\x00\x00\x00\x00\xff\xff\x00\xff\x80U\xff\xaa@\xbf\
\x803\xcc\x99+\xd5\x80$\xdb\x92 \xdf\x809\xe3\x8e\
3\xe6\x80.\xe8\x8b+\xd5\x80'\xd8\x89$\xdb\x803\
\xdd\x880\xdf\x80-\xe1\x87+\xe3\x80(\xe4\x86&\xd9\
\x801\xdb\x86.\xdc\x80,\xde\x85+\xdf\x8a)\xe0\x85\
'\xe2\x89/\xe3\x84.\xdb\x89,\xdc\x84+\xdd\x88)\
\xde\x840\xdf\x87.\xe0\x83-\xe1\x87,\xdb\x83+\xdc\
\x87)\xdd\x83/\xdd\x86.\xde\x83-\xdf\x86,\xe0\x83\
+\xe1\x86*\xdb\x82.\xdc\x85-\xdd\x82,\xde\x85+\
\xde\x88+\xdf\x85*\xe0\x87.\xe0\x85-\xdc\x87,\xdd\
\x84+\xdd\x87+\xde\x84*\xdf\x86.\xdf\x84-\xe0\x86\
,\xdc\x84+\xdc\x86+\xdd\x84.\xde\x86-\xde\x84-\
\xdf\x86,\xdf\x83+\xe0\x85+\xdc\x83.\xdd\x85-\xdd\
\x83,\xde\x85,\xde\x87+\xdf\x85+\xdf\x87-\xe0\x85\
-\xdd\x86,\xdd\x85,\xdd\x86+\xde\x84+\xde\x86-\
\xdf\x84-\xdf\x86,\xe0\x84,\xdd\x86+\xdd\x84+\xde\
\x86-\xde\x84,\xde\x85,\xdf\x84+\xdf\x85+\xdd\x84\
-\xdd\x85-\xdd\x84,\xde\x85,\xde\x86+\xde\x85+\
\xdf\x86-\xdf\x85-\xdd\x86,\xdd\x85,\xde\x86+\xde\
\x85+\xde\x86-\xdf\x85-\xdf\x86,\xdf\x84,\xdd\x86\
+\xdd\x84+\xde\x85-\xde\x84,\xde\x85,\xdf\x84,\
\xdf\x85+\xdf\x84+\xdd\x85-\xdd\x84,\xde\x85,\xde\
\x84,\xde\x85+\xdf\x86-\xdf\x85-\xdd\x86,\xdd\x85\
,\xde\x86,\xde\x85+\xde\x86-\xde\x85-\xdf\x86,\
\xdf\x85,\xdd\x85+\xdd\x84+\xde\x85-\xde\x84,\xde\
\x85,\xde\x84,\xdf\x85+\xdf\x84+\xdd\x85-\xdd\x84\
,\xde\x85,\xde\x84,\xde\x85+\xde\x86+\xdf\x85-\
\xdd\x86,\xdd\x85,\xde\x86,\xde\x85+\xde\x86-\xde\
\x85,\xde\x85,\xdf\x85,\xdd\x85,\xdd\x85+\xde\x85\
-\xde\x84,\xde\x85,\xde\x84,\xdf\x85,\xdf\x84+\
\xdd\x85-\xde\x84,\xde\x85,\xde\x84,\xde\x85,\xde\
\x86+\xdf\x85-\xdf\x86,\xdd\x85,\xde\x86,\xde\x85\
,\xde\x86-\xde\x85,\xde\x85,\xdf\x85,\xdd\x85,\
\xdd\x85+\xde\x85-\xde\x85,\xde\x85,\xde\x84,\xde\
\x85,\xdf\x84+\xdd\x85-\xde\x84,\xde\x85,\xde\x84\
,\xde\x85,\xde\x86+\xde\x85-\xdf\x86,\xdd\x85,\
\xde\x86,\xde\x85,\xde\x85+\xde\x85,\xde\x85,\xde\
\x85,\xdf\x85,\xde\x85,\xde\x85-\xde\x85,\xde\x85\
,\xde\x85,\xde\x85,\xdf\x85,\xdd\x85-\xde\x84,\
\xde\x85,\xde\x84,\xde\x85,\xde\x86,\xde\x85-\xdf\
\x86,\xdd\x85,\xde\x85,\xde\x85,\xde\x85+\xde\x85\
,\xde\x85,\xde\x85,\xdf\x85,\xde\x85,\xde\x85+\
\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xdd\
\x85-\xde\x85,\xde\x85,\xde\x84,\xde\x85,\xde\x86\
,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\
\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\
\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85\
,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\
\xde\x85\xf8\xdf\x16\xb8\x00\x00\x00\xfftRNS\x00\x01\
\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10\x11\
\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f !\
\x22#$%&'()*+,-./01\
23456789:;<=>?@A\
BCDEFGHIJKLMNOPQ\
RSTUVWXYZ[\x5c]^_`a\
bcdefghijklmnopq\
rstuvwxyz{|}~\x7f\x80\x81\
\x82\x83\x84\x85\x86\x87\x88\x89\x8a\x8b\x8c\x8d\x8e\x8f\x90\x91\
\x92\x93\x94\x95\x96\x97\x98\x99\x9a\x9b\x9c\x9d\x9e\x9f\xa0\xa1\
\xa2\xa3\xa4\xa5\xa6\xa7\xa8\xa9\xaa\xab\xac\xad\xae\xaf\xb0\xb1\
\xb2\xb3\xb4\xb5\xb6\xb7\xb8\xb9\xba\xbb\xbc\xbd\xbe\xbf\xc0\xc1\
\xc2\xc3\xc4\xc5\xc6\xc7\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf\xd0\xd1\
\xd2\xd3\xd4\xd5\xd6\xd7\xd8\xd9\xda\xdb\xdc\xdd\xde\xdf\xe0\xe1\
\xe2\xe3\xe4\xe5\xe6\xe7\xe8\xe9\xea\xeb\xec\xed\xee\xef\xf0\xf1\
\xf2\xf3\xf4\xf5\xf6\xf7\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xeb\x08\xd9\
5\x00\x00 \x97IDATx\xda\xed\x9dy\x80\x8f\
\xd5\xfe\xc7\x9f\x991\xf6m\x0a\xd9\xcb(\x09\x89\x88H\
\x12\x92\x8aV$\x22\xf5+*j\xda\x10mC\xc9T\
7.\xb7M\xbb\xa5\xba\xd9Z\xdc\xeaj,\xddH*\
\x89\xac\x955\xfb\xbe\xaf\xb3|\x7f\xc9u\xb9\xb7\x06s\
>\xe7y\xce9\xcf\xf3z\xfd_\xf3y\x9f\xcf\xe7\xe5\
\xfb\xac\xe7\xf1<\xdf8{m\x0c\xc0q\xf0\x03\xc0\x84\
 \xf8\x01\x08\x82\x1f\x80 \xf8\x01\x08\x82\x1f\x00A\x09\
\x82\x1f\x80 9S\x15?\x00A\xf0\x03\x10\x04?\x00\
A\xf0\x03  A\xf0\x03\x10\x04?\x00A\xf0\x03 \
\x86\x1f\x00\x01\x09\x82\x1f\x80 \xf8\x01\x08\x82\x1f\x00z\
\x05\xa9\xba\x8e\xc5\x04\x04\xc1\x0f@\x10\xfc\x00\xd0)\x08\
~\x00\x82\xe0\x07 \x08~\x00\xe8\x15\x04?\x00A\xf0\
\x03\x10\x04?\x00\xf4\x0ar\x0e~\x00\x82\xe0\x07 \x08\
~\x00h\x15\x04?\x00A\xf0\x03\x10\x04?\x00\xf4\x0a\
\x82\x1f\x80 \xf8\x01\x08\x82\x1f\x00z\x05\xa9\xb9\x89\xa5\
\x03\x04\xc1\x0f@\x10\xfc\x00\xd0)\x08~\x00\x82\xe0\x07\
\x80\x8a \xf8\x01\x08\x82\x1f\x00*\x82\x9c\x87\x1f\x80 \
\xf8\x01\xa0 \x08~\x00\x82\xe0\x07\x80\x8a \xf8\x01\x08\
\x82\x1f\x00*\x82\xe0\x07 \x08~\x00\xa8\x08\x82\x1f\x80\
 \xf8\x01\xa0\x22\x08~\x00\x82\xe0\x07\x80\x8a \xf8\x01\
\x08\x82\x1f\x00*\x82\xe0\x07 H\xce\xd4\xc2\x0f@\x90\
\x9c\xfd\xd8\xcc\x1a\x01\x82\xe0\x07@\xee\x05\xc1\x0f@\x10\
\xfc\x00P\x11\x04?\x00A\xf0\x03@E\x10\xfc\x00\x88\
\xe1\x07\x80\x82 \xf8\x01\x90\xb3 \xf8\x01\x90\xb3 \xf8\
\x01\x90\xb3 \xf8\x01\x90\xb3 \xf8\x01\x90\xb3 \xf8\x01\
\x90\xb3 \xf8\x01\x90\xb3 \xf8\x01\x90\xb3 \xf8\x01\x90\
\xb3 \xf8\x01\x90\xb3 \xf8\x01\x90\xb3 \xb5\xf1\x03 \
GA\xf0\x03 gA\xf0\x03 gA\xf0\x03 g\
A\xf0\x03 gA\xf0\x03 gAjoa)\x00\
r\x12\x04?\x00r\x16\x04?\x00r\x16\xa4\xeeV\xd6\
\xe1x\xec^\xbbx\xd6\xacY\x93\xd2\xd3?\x1b}\x88\
\x09\xe9\xe9\xe9\xdf\xce\x9a\xb5`\xd5\x0e\x96&\x12\x82\xe0\
\xc7\x1f\xd9\xb1p\xea\xa8A\x0ftl\xd5\xb8VrR\
B\xce[#%\x9d~n\xc3\x96\xedS\xd2\x86O\x9c\
\xc7&\xf8a\x15\x04?\x8e\xfd\xb1\xf8a\xcc\xd3\xb7]\
\x5c\xb9\xa0\x97{\xf2Vh\xd0\xe1\x89Q3\xb9\x1a\x18\
2A\xeanc\x11\x0e\xb1m\xea\xa0\xdb/)\xe7\xc9\
9\xa5^\x87''\xacfA\xc3\x22\x08~\xc4\xd6~\
\xf2\xd4\x0d\xc9\x9e^J]\xfe\xf0\xe8_\xb2Y[\xf7\
\x05\x89x\xfe\x85\xaf\xdcT\xd6\xf3\x8bb\xad\x9e\x9d\x99\
\xc1\x8c!\x88\xa3,\x1d\xd6\xb9\x82\xe77\x85\x9a\xa7\xa6\
\xefg\xce\x10\xc41\xf6\xa5\xa7\x94\xf7\x82\xa2`\xeba\
\xeb\x185\x04q\x86\xcd\xa3;\x17\xf5\x82%\xa1N\xea\
\x02\xa6\x0dA\x1c`\xdb+\x8d\x13<#\x9c\xd3o\x19\
\x03\x87 V\x93\x95\xde\xb9\x90g\x90:\xc3\xb8\xfd\x8e\
 \xd62\xff\xa12\x9ei\x0a\xde\xfcy\x16c\x87 \
6\xfex\xb4\xf6\xec\xa0r\xdav\x06\x0fA\xecb\xd7\
\xb0s<{(\x9a\xb2\x92\xd1C\x10{\xf8\xb5Wq\
\xcf.\x12\xdb\x7f\xcb\xf0!\x88\x1dl\xec\x9d\xdf\xb3\x90\
\xe6\xb3\x18?\x041\xcf\xd6\xd4\x22\x9e\x9d\xc4\xb5]\xcc\
\x00\x22\x88Yv\xa7%y\xf6\x12\xdfv)#\x88 \
\xe6\xc8~\xfd4\xcfn\xf2\xf5\xde\xcd\x10\x22\x88!\xe6\
^\xe4\xd9O\xb9\x11L!\x82\x98`Oj^\xcf\x09\
Z/g\x0e\x11$p\xc6W\xf0\x5c\xa1`\xdaA&\
\x11A\x02eGG\xcf%\xea\xfe\xc4(\x22H\x80|\
\x9d\xec\xb9E\x81!\xcc\x22\x82\x04EFZ\xa2\xe7\x1c\
\xd7\xb3\x1f\x0a\x82\x04\xc3\xf2\x86\x9e\x8bT\x98\xca8\x22\
H\x00L,\xee\xb9I\xc2\xe0`\x17j\xdfV5\xb2\
\x11\xc4e\x86%z\xce\xf2\x7f\x07\x82\x5c\xa9g\x15\xab\
\xdc\x80 \x0e\x9f~\xf4\xf0\x5c\xe6\xa2\x0d\x08\x82 >\
\xb2\xa5\xa9\xe76\xc9\xf3\x11\x04A|\xe3\x973=\xd7\
):\x05A\x10\xc4'\x16\x97\xf3\xdc'\xdfG\x08\x82\
 \xbe\xb0\xb0\x8c\x17\x06\xf2\x8eG\x10\x04\xf1\x81\xd9%\
\xbcp\x900\x12A\x10D;\xdf\x9d\xe2\x85\x85\x84\xb7\
\x10\x04At\xfbQ\xc4\x0b\x0f\xf1\xa3\x11\x04A\xb4\xb2\
\xf44/L\xe4MG\x10\x04\xd1\xc8\xa6*^\xb8(\
:\x07A\x10D\x1b{\x1axa\xa3\xecJ\x04A\x10\
Md^\xe3\x85\x8fj[\x11\x04A\xf4\xd0\xcd\x0b#\
\xcd3\x11\x04At\xf0\x8e\x17N\x1eC\x10\x04\xd1\xc0\
\xbc\x82!\x15$\xfeS\x04A\x101\xbb\xce\xf1\xc2J\
\xd22\x04A\x10)\x1d\xbd\xf0Ro?\x82 \x88\x8c\
!^\x98\xb9\x17A\x10D\xc4\xa2\xfc\xa1\x16$n\x22\
\x82 \x88\x80\xccz^\xb89c'\x82 \x88:\x03\
\xbd\xb0\xd3\x1dA\x10\x84\x03\xac\xe3\x1cd}\x8e \x08\
b\xfb\x01V\xe1\x0a5\x1b_\xd3\xb6}\xd7\xae]{\
\xdd\xdd\xb5k\xe7\xb6m\x9a\xd5IN\x8a\x0b\xe6o'\
\xefF\x10\x04Q\xe3\x19\x9fg3O\x95\xebR\x06\x8d\
\x9e\xb1:\xe3\xcf\xff\xfc\xc69\x1f\xbf\xd8\xa7c\x1d\xbf\
\xefS\xde\x87 \x08\xa2\xc4\xaf\x85\xfc\x9b\xca\x22\xcd\xfa\
\xbc;\xe7\xe4\xeeBd-\xfdx`\x9b\xb2>j:\
\x1fA\x10D\x85\xf6>M\xe4i\x9d^\x9a\x93\xeb\x07\
\x05W\xbe{o5\x9f\xeai\x86 \x08\xa2\xc0t?\
N\x02\xe2\xeb\xf4\x9e\x9e\xa5Z\xd1\xf2am}y\xf1\
w\x1c\x82 H\xee\xcf\xd0\xcf\xd3?\x89\xd5\xd3\xd6\x0a\
\xab\xda7\xa1\xad\xfe\xbd\x81+\xeeA\x10\x04\xc9-/\
\xeb\x1e\xc3\xd2}\x17h)l\xc3\xd0\x9a\xbaK\xeb\x8f\
 \x08\x92K\xb6h\xde\x05\xab\xd6\xdb\x1a\x1f\x0c\x9c\xdc\
:^kq\x05W!\x08\x82\xe4\x8e^Z'\xb0\xc5\
T\xcd\xe5\xfd\xdcM\xeb\x91V7\x04A\x90\x5c\xb1N\
\xe7\xdd\x87\x0b'\xfbP\xe1\x8a\xae\x09\xfa*L\x5c\x8a\
 \x08\x92\x1bR\xf4\x0d_\xb5\x09>\xd58\xb7\xa5\xbe\
\x22oE\x10\x04\xc9\x05k\x0a\xe8\x9a\xbc\x02\xa9>\xbe\
\x944\xa1\xbc\xae2\x13\x16#\x08\x82\x9c<\xda\xf61\
i\xb2\xc8\xd7:\xb7\xa7\xe8:\xce\xba\x19A\x10\xe4\xe4\
\x8f\xef\xf3\xea\x99\xba\xfcC|\xff \xe5\x14M\x0f\xa1\
$,D\x10\x049Yn\xd53tU\xe7\x06P\xeb\
\xfa\x16z\x8a\xed\x88 \x08r\xb2g z~@:\
\xed\x0e\xa4\xda\xac\x01Zn\x8a$\xfe\x8a \x08rr\
<\xac\xe5M\xa4\xd4\xc0\xea\x1d\xaf\xe5\x9atO\x04A\
\x90\x93b\xcf\xa9\x1a\xc6-\xef\xc8\x00+\x9eYJC\
\xc5E\xb7#\x08\x82\x9c\x0c\x7f\xd3\xf1\xbe\xc7\x17\x81\x96\
\xbc\xe4\x0c\x0d5\x0fF\x10\x049\x092+\xcbg\xad\
\xd0\xd4\x80\x8b^\xa9\xc1\x90\xd33\x10\x04AN\xccX\
\x0d\x0f\xffM\x0d\xbc\xea\x15\xa7\xcb\xcb\x1e\x8d \x08r\
b\x1a\xc9o\x7f|a\xa0\xec\x9f\xe4\xe7!\x8d\x10\x04\
AN\xc8B\xf9\xf5\xabQF\x0a\xff*\x9f\xb8\xf2\x85\
\x08\x82 'B\xfe\x98b\xaa\xa1\xca\xdf\x17\xbf#\xdc\
\x13A\x10\xe4\x04\xec\x13_\xe3m\x9fm\xaa\xf6\xbe\xd2\
\xd2K\x1d@\x10\x049>\xefJ\x87\xec\xac]\xc6j\
\xcfj*-~,\x82 \xc8\xf1\xb9T8b\xf9f\
\x1b,~\xb5\xf4\xe7\xef\x0a\x04A\x90\xe3\xb2Tz\x1c\
?\xd8h\xf9\xe3\x85\xd5\xc7\xaf@\x10\x049\x1e\xd2\xc7\
\xb0Zd\x9b\xad_\xfa\x1cr\x7f\x04A\x90\xe3\x90-\
\xbc\xddV`\x89\xe1\x00[J\x0a\x9f\xd0G\x10\x049\
\x0e\xd3\x84\xff\x00?i<\xc1k\xc2\x04\xf3\x10\x04A\
r\xa6\x87\xf0\x0a\xd6>\xe3\x09\xb2\x1a\xc8\x22<\x8a \
\x08\x92#\x99ed\xd3\xf5O\x0b2\xcc\x92]f8\
\x0bA\x10$G&\xcb\xfc\xb8\xc4\x8a\x10md!f\
#\x08\x82\xe4DW\xd9l}mE\x88\xc5yD!\
z#\x08\x82\xe4@\x86\xec\x81\xd8\xeb,\x89\xd1I\x94\
\xa2R6\x82 \xc8\x9f3Qv\x93m\xbe%1\x96\
\xc8\xf6\xca\xfa\x1eA\x10\xc4\x8fkX\xd7X\x93\xa3\x9d\
\x1d\x97\xaa\x11$l\x82\xc8\xde\xb5\x9dnM\x8e\xefD\
9.D\x10\x04\xf9S\xe6\x8b\xe6\xaa\xbeEI.\x16\
\x1d*n@\x10\x04\xf93d_}\xfe\xd0\xa2$\x1f\
\x8a\x92\x8c@\x10\x04\xf93\x1aK\xa6\xaab\xa6EI\
\xb2D\x8f\x94\xb5G\x10\x04\xf9\x13\xb6\x8a\xee\x1f\x0c\xb0\
*K\xaa$JR\x06\x82 \xc8\x1f\x11\xbdK\x98g\
\x8dUYV\x89\xae\xf4~\x89 \x08\xf2GD\xf7\xd7\
\xae\xb7,\xccU\x920}\x11\x04A\xfeH9\xd7\x1f\
S<\x96\x0f$a\x1a \x08\x82\xfc\x81\x9f$#U\
:\xd3\xb24\x07N\x91\x1c/\xeeD\x10\x04\xf9_\x86\
I\x04\xb9\xcf\xba8\xb7I\xe2LD\x10\x04\xf9_n\
\x92L\xd47\xd6\xc5\x99h\xfe$\x04AB%\x88\xe4\
[\x7f\x95\xb3\xad\x8b\x93y\x9a OC\x04A\x10\x9d\
\xa7 }-\x0ct\x97 O\xe2.\x04A\x10\x8d\xa7\
 \xdfY\x18\xe8\x9f~_\x94\x9b\xd1\xf5\x04\xd4W\xfc\
\xe3\x9d\xbaja/\x82\xd8r\x0aR.\xdb\xc2@\xfb\
\x0b\x0b\x12\xf59\x89?\xf0\xb6g7\xdb\x11D'\xe5\
\x05\xad\xe8fe\xa2\xeb\x04\x89\x9a \x08\x82\xfc\x17k\
$\xad\xf8\xd4\xcaHo\x08\x12\x15\xceD\x10\x049\x96\
q\x92i\xdage\xa4\x0d\x92\xcf\xa7\xcfG\x10\x049\
\x16\xc9\x9e\xbc\xad,\xcdT[\x90\xe9M\x04A\x90c\
i\x22\xe8\xc4_-\xcd\xf4\xa0 \xd3]\x08\x82 \xc7\
\x90UT\xd0\x89y\x96\x86\xfa\x87 S\x1d\x04A\x90\
c\x98'hD\xa9lKC\xedL\x14\xdc*\xdc\x8b\
 \x08r\x94\xd7\x05\x8d\xe8`m*\xc9>\xd63\x10\
\x04A\x8e\xd2M\xd0\x88\xd7\xadM%\xf9\xa8\xe7\x10\x04\
A\x90\xa3\xd4\x154b\xb1\xb5\xa9>\x11\xa4\xfa?\x04\
A\x90\xa3\xe7\xe8\x85\xd4\xfbpj\xb6\xb5\xb1\xb6\x0a\xbe\
\x84P\x0fA\x10\xe4?H\x1e\xe5mmq\xae\xb3\xd5\
c\x15\xcaB\x10\x049\xc2XA\x1f\x9e\xb68W\x17\
A\xae%\x08\x82 GxB\xd0\x87/,\xce\xf5\x8a\
 \xd7\x07\x08\x82 G\xb8A\xbd\x0d\x09\xbb-\xce5\
G0_\xfd\x10\x04A\x8ep\x96z\x1bj\xd8\x9c+\
#\xbfz\xb0\xb6\x08\x82 \xfff\x8f\xe0\xb9\xd7Na\
\xbd|]\x15A\x10\xe4\xdfH\xbe\xa7\xf1\xbc\xd5\xc9n\
\x17\x1c;\xeeC\x10\x04\x91\xb7z\x8a\xd5\xc9^\xf0\xf1\
\x19L\x04\x89\x8c \x8f\xa8w!n\xab\xd5\xc9\xbe\xf2\
\xf12\x16\x82DF\x90\x1b\xd5\xbbP\xc9\xeed\xbb\x04\
gW\xcf\x22\x08\x82\x1c\xe6\xfcp\xdeG?D%\xf5\
h\xdd\x10\x04A\x0eSL\xbd\x0b\xbd-\x8fv\xa5z\
\xb4\xa6\x08\x82 \xbf\xb3^\xd0\x85\xb7-\xcf\xf6\x80z\
\xb4\x8a\x08\x82 \xbf3M\xd0\x85o,\xcf\xf6\xaaz\
\xb4\xf8\xbd\x08\x82 \x87xSp\x11kG\x88\xe5\x9f\
\x8f \x08r\x08\xc1\x8bw\x15l\xcf\xb6Y0a\x1f\
\x22\x08\x82\x1c\xa2\x9dz\x13\x9a[\x1f\xae\x84o\x0f\x09\
 HT\x04\xb9\xc0\xbfK\xa1\xe6\xa9\xa7\x1e.\x05A\
\x10\xe4\x10\xa5\xd5\x9b\xf0\x8c\xf5\xe1\xda\xab\x87\xbb\x1eA\
\x10\xe47\x0e\x0a\xee6\x8f\xb1>]\x1f\xf5p\x17 \
\x08\x82\xfc\xc6rA\x13f[\x9fNp\x9d\xb7\x0c\x82\
 \xc8o|)h\xc26\xeb\xd3M\x12\xdc\x089\x80\
 \x08\x12\x8b\xbd\xa3\xde\x83S\xedO\xb7L0b\xcb\
\x11\x04Ab\xb1g\xd4{p\xbe\xfd\xe92\x12\xd4\xe3\
}\x89 \x08\x12\x8b\xf5P\xef\xc15\x0et\xa0\x9cz\
\xbcw\x10\x04Ab\xb1k\xd4{\xd0\xdd\x81\x0e\x08n\
\x84\xa4!\x08\x82\x8866x\xda\x81\x0e\x08\xbe\xe5y\
\x0f\x82 \x88\xe8\x18d\x84\x03\x1d\xb8G=\xde\x8d\x08\
\x82 \xb1\xec\xbc\xea=\x98\xe2@\x07\xd2\xd4\xe3]\x8a\
 \x08\x12\xdb&\xe8\xc1\xcf\x0et`\xa4z\xbc\x1a\x08\
\x82 \xb1\x9f\x05=\xd8\xe5@\x07&\xab\xc7+\x85 \
\x08\x22y\xa5\xa8\xa0\x0b\x1d\x10|~1!\x13A\x10\
d\x9cz\x0b\xcep\xa1\x03\x1b\x043\xb6\x09A\x10\xe4\
e\xf5\x16\xd4s\xa1\x03\x99\x82[\xe9\x0b\x10\x04A\xfa\
\xab\xb7\xa0\x95\x13-(\xa9\x1e\xf0\x0b\x04A\x10\xc1}\
\x82[\x9dhA5\xf5\x80\xa3\x8f\xf7\xff\xdd<\xeb\x04\
\xa4(\xfe\xd5\xcfgi!\x13A\xb4 x\xe5\xae\xb7\
\x13-h\xa2\x1e\xf0E\xd1\x1f~V\xf1\xafn\x88E\
\x08\xfb\x05\xb9L}~\xfe\xe2D\x0b\xda\xaa\x07|\x0a\
A\x10D\xb0e\xc3\x9bN\xb4\xe0N\xf5\x80\xbd\x10\x04\
A\xaa\xa8\xcf\xcfx'Z\xf0\xb0z\xc0n\x08\x82 \
\xa7\xa9\xcf\xcfT'Z0P=`{\x04A\x10\xc1\
w.\x7fp\xa2\x05\x82\x1b=W H\xe4\x059 \
\xb8\xd2\xbe\xc2\x89\x16\xbc\xa7\x1e\xb0!\x82D^\x90\x8d\
\xa1\xbf\x15\xf5\x99z\xc0\xea\x08\x12yA\x96\xa8\x8fO\
|\x96\x13-\xf8Z=ay\x04\x89\xbc \xdf\xab\x8f\
O17Z\xb0H=a\x11\x04\x89\xbc S\xd4\xc7\
\xa7\x9c\x1b-X\xa5\x9e0.\x13A\xa2.\xc8\x07\xea\
\xe3S\xc5\x8d\x16l5u\x96\x85 a\x10d\x94\xfa\
\xf4\xd4v\xa3\x05\x92\xebt\xeb\x10$\xea\x82\xbc\xa6>\
=\x17;\xd2\x83D\xf5\x88\xcb\x10$\xea\x82\xfcM}\
zZ:\xd2\x83\xe2>\xbd1\x85 Q\x10\xe4Y\xf5\
\xe9\xb9\xc1\x91\x1e\x086\xfe\xfa\x1eA\xa2.H?\xf5\
\xe9\xe9\xecH\x0f\x04\x8fcNG\x90\xa8\x0b\x22\xf8\x02\
S7Gzp\xbez\xc4t\x04\x89\xba \xf7\xa9O\
\xcf=\x8e\xf4\xa0\xbez\xc4\x09\x08\x12uA\x04\xaf\x13\
\xdd\xefH\x0f.\xf2\xe9\xa5t\x04\x89\x82 \xb7\xa8O\
OOGzp\x89z\xc4\xe1\x08\x12uA\xda\xa9O\
O\x1fGz\xd0\x5c=\xe20\x04\x89\xba W\xabO\
\xcfc\x8e\xf4\xa0\xa5z\xc4\xbf\x22H\xd4\x05\xb9B}\
z\xfa9\xd2\x83V\xea\x11\x9fG\x90\xa8\x0b\xd2B}\
z\x068\xd2\x83k\xd5#>\x8b Q\x17\xa4\x99\xa1\
\xe9\x09\x10\xc1\xc6X\x03\x11$\xea\x824Q\x9f\x9eA\
\x8e\xf4\xe0&\xf5\x88O!H\xd4\x05i\xac>=C\
\x1d\xe9A'C\xa7Y\x08\x12\x06A\x04w\xd1^r\
\xa4\x07\xb7\xaaG|\x1cA\xa2.\xc8\x85\xea\xd3\xf3\x9a\
#=\xb8C=\xe2#\x08\x12uA\xea\xa9O\xcf[\
\x8e\xf4\xe0nC\xf7B\x11$\x0c\x82\xd4Q\x9f\x9e\x91\
\x8e\xf4\xe0^\xf5\x88\xbd\x10$\xea\x82\xd4R\x9f\x9e\xbf\
;\xd2\x83\x07\xd4#>\x88 Q\x17\xa4\xa6\xfa\xf4\x8c\
u\xa4\x07\xbd\xd4#\xde\x87 Q\x17\xa4\x86\xfa\xf4\x8c\
s\xa4\x07}\xd5#\xa6 \x08\xbf \xca\xbc\xefH\x0f\
\x1e\xe6\x10\x0bA\x94\xa9\xad>=\xef8\xd2\x83\x87\xd4\
#\xf6F\x90\xa8\x0bR\xd7\xd0\xdbD\x01\x22x\xab\xb8\
/\x82D]\x10\xc1}\x907\x1c\xe9A\x0f\xee\xa4#\
\x882\x0d\x0c\xbdn\x17 \x82\xd7\xeey\x16+\xf2\x82\
4R\x9f\x9e\x17\x1d\xe9\xc1\xed<\xcd\x8b \xca\x08\x9e\
\xe6\x1d\xe2H\x0f\xba\xa8GLC\x90\xa8\x0br\xa9\xfa\
\xf4\xfc\xc5\x91\x1e\xdcl(\x22\x82\x84A\x90\xe6\x86\xfe\
y\x0d\x90\xf6\xea\x11\x07#H\xd4\x05ia\xe8\x00=\
@\xda\xa8G\xfc\x1b\x82D]\x90+\xd5\xa7'\xd5\x91\
\x1e\x086mx\x09A\xa2.\x88`z\x5c\xd9\x17\xeb\
*\xf5\x88\xaf\x22H\xd4\x05\x11\x1c\xa0\xbb\xb2\xb3\xe2\xe5\
\xea\x11\xdfD\x90\xa8\x0b\x22xa\xdb\x95\xbdy\x05;\
\x1b\xb17o\xe4\x05\x11\xbc\x8fz\x9f#=\x10l^\
\xfd\x1e\x82D]\x10\xc1\xebv\xae|\x1fD\xb0q\xcb\
\xc7\x08\x12uA\x1eQ\x9f\x9e\xdb\x1d\xe9\x81\xe0\x03:\
\x93\x11$\xea\x82<\xa9>=\x1d\x1c\xe9Au\xf5\x88\
3\x11$\xea\x82<\xa7>=\xd78\xd2\x83\x8a\xea\x11\
\xe7!H\xd4\x05yA}z\x9a;\xd2\x03\xc1w\xd2\
\x97!H\xd4\x05yC}z\x1a8\xd2\x83<\xea\x11\
7 H\xd4\x05yW}zj\xba\xd1\x82\xbd\xea\x09\
\xbd\xdd\x08\x12uA>R\x9f\x9e\xcan\xb4`\x83z\
\xc2\xb8l\x04\x89\xba _\xa8\x8fOi7Z\xb0D\
=a\xa1\x18\x82D]\x90\x1f\xd4\xc7\xa7\xa0\x1b-\x10\
$,\x85 \x91\x17d\xa9\xe0\x08}\xbf\x13-\xf8R\
=`%\x04\x89\xbc \x9b=C\xd7x\x02\xe3\x1f\xea\
\x01k H\xe4\x05\xc9\x10\x08\xf2\x93\x13-xO=\
`}\x04\x89\xbc \xb1B\xea\xf3\xf3\x8d\x13-xM\
=`S\x04A\x90\xd2\xea\xf33\xd1\x89\x16<\xaf\x1e\
\xb0\x1d\x82 \xc8\xd9\xea\xf3\xe3\xc6\xf6\xee\xa9\xea\x01\xef\
B\x10\x04\x11<\x0c\xee\xc6\xde\xa3\x0f\xaa\x07|\x04A\
\x10D\xb0\xef\xcf\x00'Z \xd8yt0\x82 H\
\x87\xb0\xbfs{\x8dz\xc0\x91\x08\x82 )a\x7fc\
J\xb0\x7f\xfd\xa7\x08\x82 O\xa9\xcf\xcfeN\xb4\xe0\
L\xf5\x80\xdf\x22\x08\x82\xbc\xaa>?\xb5\x9chA1\
\xf5\x80K\x11\x04A>P\x9f\x9f\xb2.t\xe0@\x9c\
z\xc0\xed\x08\x82 _\xa9\xcfO\xdel\x07:\xb0F\
=_\x9el\x04A\x90\x9f\x05\x0fcmv\xa0\x03s\
\xd4\xe3\x9d\x16C\x10\x04\xd9.\x10d\x8e\x03\x1d\xf8D\
=^u\x04A\x90X,\x9f\xfa\x04}\xe2@\x07\x04\
\xcf*6F\x10\x04\x89\xc5\xca\x85\xfbY\x93~\xea\xf1\
\xda \x08\x82\xc4b\x17\xa8O\xd0\xe3\x0et\xe0\x0e\xf5\
x\x0f \x08\x82\xc4b\xd7\xa9O\xd0m\x0et@\xf0\
\x09\xadA\x08\x82 \xb1\xd8\xbd\xea\x13t\xb9\x03\x1d8\
O=\xde\x18\x04A\x10\xf5>\xfeFU\x07:PJ\
=\xde\x0c\x04A\x10\xd1;\xdb\xf9\xb3\xacO\xb7Gp\
#}5\x82 H,6\xcd36B\x010_p\
#=\x13A\x10$\x16[!\x10d\x9a\xf5\xe9&\xa8\
\x87\xab\x18C\x10\x04\x89\xc5\x0e&\xa8\xcf\xd0p\xeb\xd3\
\x0dU\x0f\xd7\x10A\x10\xe4\x10e\xd5g(\xd5\xfap\
\xf7\xab\x87\xbb\x11A\x10\xe4\x10\xf5\xd4g\xa8\x93\xf5\xe1\
\xae5v\x9f\x10A\xc2\x22H{\xf5\x19\xaag}\xb8\
\x9a\xa6\xb6l@\x90\xd0\x08\xf2\x98\xfa\x0c\x15\xb5\xfd\x8d\
\x90\xac\x02\xc6\xee\x13\x22HX\x04\x19\x1e\xe2\xeb\xbc\x92\
\xcd\xeb\x7f@\x10\x049\x84\xe0\x9dB\xefs\xcb\xb3\x09\
\xde\x06\x89\xdb\x85 \x08r\x88\x8d\x02A\x86X\x9eM\
\xb01o\x85\x18\x82 \xc8\xef\x08\xbe\x93|\xa7\xe5\xd1\
\xba\xaaGk\x8e \x08r\x98\xba\xeaS\xd4\xc8\xf2h\
\x8d\xd5\xa3uG\x10\x049\x8c\xe0:oa\xbb\x1fW\
\xccNR\x8f6\x14A\x10\xe40\x82\xeb\xbc\xde\xcfV\
'[.H6\x11A\x10\xe40\xa3\x04cd\xf77\
B>\x14$[\x81 \x08r\x18\xc1\xd6Q^\x1f\xab\
\x93=\xa1\x1e\xac@\x16\x82 \xc8a\xf6'\xaa\xcfQ\
K\xab\x93\x09>}p^\x0cA\x10\xe4\xdfTW\x9f\
\xa3\x92V?lr\x86z\xb0v\x08\x82 G\xb81\
\xa4g\xe9\x1b\x04\xb9\x1eC\x10\x049\x82\xe0\x1b!V\
\xbf3\xf5\x91 \xd7h\x04A\x10\x1d\x83d\xf3\xbd\xf4\
\x87\x05\xb9\x96\x22\x08\x82\x1ca\x99`\x90jZ\x9c\xeb\
\x12\xf5X\xc5\xb3\x11\x04A\x8e\x90]D}\x92\x12v\
X\x1b+\xa3\xb0z\xacKc\x08\x82 \xff\xa1\xa1\xe0\
'\xc4\xde-\xdeg\x0bR=\x88 \x08r\x94\x14\xb3\
\xa3\xe4\x13\x83\x04\xa9\xdeC\x10\x049\xca;\x82Q\xb2\
\xf7S\x9e\xad\x04\xa9~B\x10\x049\xca\x12\xc1(\xc5\
m\xb4\xf5\x14\xa4\xa8\xe0e\xfb,\x04A\x90c\xce\xd2\
K\x18\xbdc\xe0\x0f_\x0b2]\x12C\x10\x049\x86\
+\x04\xc3t\x87\xa5\x99\x06\x082\xdd\x87 \x08r,\
\x82\xc7^\xbd2\x96>\x8e%\xb8\x0b\xe2\x8d4(\xc8\
z\x04\xb1\x0f\xc1\xee\x1f\x9e\xf7\x8d\x95\x91\xb6\xe6\x11D\
ZlP\x90\xb5\x08b\x1f\x9b\x05\x9f\xd1\xf0\x1e\xb52\
\x92\xe45\xb0\xd2\xd9\x06\x05Y\x8d \x16R%tO\
\x9b\x08\xde\xb4\x17\xef[-\x12d%\x82XHW\xc9\
1\xd62\x0b\x03\x1d\x14lf\xe4\xbdlR\x90\xe5\x08\
b!\xefJ\x04y\xda\xc2@\xe9\x92@\x8bL\x0a\xb2\
\x14A,d\xadd\x9ejX\x18\xe86\xd3\xa7 \xb1\
\xe7B\xb9QLT\x05\x89U\x95\x182\xd7\xba8\xfb\
%GX\xed\xb5\x940\xd8\xe0\x154\x04\xd1N7\x89\
 \xf6\xedm2^\x12\xe7\x15-%\xbc\xa0\xf8\xd7\x7f\
D\x10\x1b\xf9\xbbd\xa2*f\xda\x16\xa7\x9d\xf1S\x90\
\xd8\xb0P\xddV\x8a\xbc \xeb$wB\xbc\x7fX\x96\
f\x8b\xe0\xc39^\x19=5\xbc\xa9\xf8\xe7\xff\x85 \
VRM\x22H+\xcb\xc2\x0c\xf6\x8c\x9f\x82\xc4F(\
\xfe\xf9\x89\x08b%=%3\x15\xbf<D\xb6\xbf\xa9\
\xa7\x06\xd5+\xe7\x1f#\x88\x95|!\x99)\xcb\x1e7\
\x11e\x89[\xa7\xa7\x88\xb1\x8a\x7f\xff}\x04\xb1\x92\x0c\
\xc9\x85Q\xaf\xe4\x1e\x9b\xb2Hv\xc2\xf3\xeaj*B\
u\xe7\xec\xe1\x08b'\xedE?!C-J\xb2L\
\xf2 \xaf\x97\xaa\xa9\x0a\xd5'\xa4_@\x10;\x19)\
\x12\xa4\xc2A{\x92\xdc%J\xf2\x9d\xa6*&*\xfe\
\xfd\x01\x08b'[\x12Ds5\xc2\x9a \x1b$\xd7\
x\xbdr\xba\xde\xff\x9a\xa2X@/\x04\xb1\x94\x8bD\
\x82T\xcd\xb0%G_Q\x8e\x1e\xba\xca\x98\xa6X@\
7\x04\xb1\x944\xd1`y\xafZ\x12c}aQ\x8c\
\xa9\xba\xea\x98\xa9X\xc0M\x08b)KE7\xd3\xbd\
\xb2\x96\x5c\xc8\xea.JQJ\xdbS3\x0b\x14+\xb8\
\x12Al\xa5\x9e\xec'\xc4\x8e\xb3\xcb_\x12E!\xba\
j+d\x8db\x05\x17 \x88\xad\x0c\x92\x09Rd\x95\
\x0d!\xdazv\x1ca\xc5\xf6\xa8^\x0fD\x10[Y\
\x13/\x1b.\x1b\x0e\x0e>\x95E8]\xe3\x87\xdf\xf3\
\xaa\x95\x90/\x1bAl\xa5\xb1l\xba\xbc\xb1\xc6\x13\xec\
\xae$K\xf0\x88\xc6ZJ*\xd6\xb0\x15Al\xe5E\
\xa1 e\x8c\xf7\xf6\x1ea\x82E\x1ak9K\xb1\x86\
\x85\x08b+\x1b\xf2\x08\xe7\xebZ\xc3\x87\x07\x93\x84\x07\
\x89\xf5u\x16s\x81b\x11\x93\x11\xc4ZZ\x09\x05\xf1\
\x06\x1b-\x7f}\x19a\xf9o\xe8\xac\xe62\xc5\x22^\
G\x10k\xf9H*H\xe2W\x06\xab\xcfl&\xac\xbe\
\xd8n\x1b\xae\xa7\xf5E\x10k\xc9('5\xa4\x82\xc1\
\x9d3\x1f\x92\x16\xdfCk9\xb7\x9b}\xa3\x11A\xfc\
\xe0Q\xe9\x8cy\xd5\x8d\x9d\xa8\xbf,\xae\xfdG+|\
\xad\x87 \xf6\xb2\x22^<eM\x0f\x98)}\xac\xb8\
\xf4K\xf4\x16\xf4\x94b\x19\xc5\xb3\x11\xc4^Z\x8a\x05\
\xf1\xda\x18y5dJ~q\xe1\x1f\xe9\xadHu\xdf\
\x9f(\xed\xce\xeb\x9e \xe3\xe4\x82xW\xed\x0b\xbe\xee\
\xcf\x0a\x88\xcb\xae\x92\xa5\xb7\xa4\x8fU\x0b\x19\x8f \xf6\
r\xb0\xa2\x06CZ\x04\xfe`\xef\xb8\xbc\xf2\xaa_\xd6\
\x5c\xd3w\xaa\x85<\x8e \x16\xf3\xbc\x06A\xbc\x86\x01\
\x7fG\xec\x8d<\xf2\x9a\xb5\xef;\xb1Z\xf9\x17\x18A\
,fgq\x1d\x86\x94\xfb6\xc0\x923z\xeb(y\
\xa0\xf6\xb2T\xdfa.r\x10A,F\xcb\xb4y\x05\
\xdf\x0b\xac\xe0M\x97\xea(\xf8\x94\x9d\xda\x0b+\xadZ\
\xcbW\x08b1k\xf2j1\xc4\xbb}g0\xf5N\
\xae\xa8\xa5\xdc\xfe\xfa+\xab\xafZK?\x04\xb1\x99.\
z\x04\xf1\x92\xa7\x05P\xec\xde{\xe3\xb4\x14[|\xbb\
\xfe\xda\x94w\x1a\xab\x85 63/N\x93!\x09\x0f\
\xec\xf0\xbb\xd6igk\xaa5\xcd\x87\xe2\xfa(W\xf3\
\x03\x82\xd8L\x1bO\x17\xa5\xdf\xf6\xf5\xae\xf0\xaf7\xe9\
r\xb9\xfc^\x1f\xcaS\xbeS\xe8\xa5 \x88\xcd,L\
\xd0f\x88w\xe1t\xdf\xca\xdc\xdd\xaf\xa0\xb62\xdf\xf0\
\xa3\xc0\xcf\x95\xcb)\xbc\x06Al\xe6\x16O#\xcd\xbf\
\xf4\xa5\xc6\x1d\x03J\xe8\xab\xb1\x86/\x9f\xc8Z\xa6^\
Pg\x04\xb1\x99\xe5yu\x1a\xe25\x99\x98\xa5\xbb\xc2\
\x8dO$\xe9\xac0\xdd\x97e\xccR\xff\x85\x8b\xff\x0c\
Al\xa6\xbb\xa7\x97\xcai\x1bu\x967\xabk\x01\xad\
\xe5\xdd\xe0\xd32\xd6R/\xa9\xc8l\x04\xb1\x98\xb5\x05\
5\x1b\xe2\xe5\xef8a\xbf\xa6\x03\x97\xb4\xb35\xd7V\
p\x85O\xcbx\x93\xa0\xa8\xb2\xb3\x11\xc4b\x1e\xf5\xf4\
S\xbc\xb3\xdc\x91%\x03\xeb\xe8/\xac\xbf_\xab\xd8O\
RU\x817\x10\xc4^\xf6T\xf4\xfc\xa0`\xcb\xbf\xcc\
U\xbe\xf0\xbbe\xdc]U\xfc(\xaa\xda~\xbfVQ\
\xf8\xee\xc0\xb5\x0b\x10\xc4Z\xc6y~q\xdauOO\
\xca\xedm\xeb\xcc\xf9ow\xaf\x93\xe0O=\xf13|\
[\xc4e\xc2\xd2\x12\xba\xccF\x10[i\xe1\xf9H\x5c\
\xd5\xb6\x8f\xbe3\xebd\x1e\xd6\xca\x5c\xf2\xe9\xa0n\x8d\
\x0b\xfbX\x8b\x8f7\xe5\xb2O\x11WwN\xff9\xb9\
\xb9\x04}p\xeb\xda\xa5?\xce\x9a\x92>z\x0e\x82\xf8\
\xcc\xe2\xbc\x9e\xef\x94\xa9{\xf5\xddO\xbe\xfd\xf1\xbf\xe6\
\xae\xd8\xf6_\x7f{\xef\xbaE3'\xbe;\xf8\xc1\x0e\
\x17\x9f\xe5{\x15\x95v\xfb\xb8\x88\xcdtTX\xa4i\
\xdf\xe1S\x96\x1cs\x1c\xb8k\xeb\xf2%\xb3\xbeI\xff\
d\xf4\xf0a\xcf\xa7\xf5\xe9\xd5\xf5\xb6\xb6W7oX\
\xa7J\xf2)\xc5\x8e\xfeGw\x22\x88\xdf<\xec\x05J\
|RRR\xc5\xe4\xe4\x12II\x05\x83\xfc\xb3\x09\xbe\
>Q\xd9Sc\xa5E\x92*\x9dY+\xb9|\xd2\xc9\
\xbd{\xdf\x01A\xfcfwe/\x02\xf8\xfb\x81\xf71\
\xc6r]\x85 \xbe\xf3UB\xf8\xfd\xa8\xe3\xef\xcb{\
\xeb\x8d\x05k\x84 \xfe\xd33\xf4~\x14\xfd\xd9\xe7%\
<\xd3T\xb2s\x11\xc4\x7f\xf6\xd7\x0c\xb9\x1fqc\xfc\
^\xc2[LE;\x1dA\x02`v\xdep\x0b\xd2\xd3\
\xf7\x15|\xcdT\xb4\xe2\x08\x12\x04O\x85\xda\x8f&\xfe\
\x7f\xda}\x85\xa9l\xf1\xd9\x08\x12\x00YW\x86\xd8\x8f\
36\x04\xb0\x82g\x9bJ\xb7\x0bA\x82`\xeb\x99\xa1\
\xf5\xa3\xc8\x8fA,`\x8a\xa9x\xab\x11$\x10\xe6\x16\
\x0a\xa9\x1f\x09\xc1\xbc\x92\xf4\xa9\xa9|\xf3\x11$\x18\xde\
\x0b\xa9 /\x05\xb3|\xfb\x8a\x18\xca7\x03A\x02\xe2\
\xa1P\xfa\x11\xd8\xdel7\x19\x0a\xf8O\x04\x09\x88\xcc\
kC\xe8G\x8f\xc0\x96o\x9c\xa1\x84\x7fG\x90\xa0\xd8\
\xd74t~t\xc8\x0al\xf5\xf6\x18:\x89\x1b\x86 \
\xc1\xf5\xb8a\xc8\xfch\x9b\x11\xe0\xeau2\x93\xf19\
\x04\x09\x8eM\xd5B\xe5G\x9b \xfd\x88}a&\xe4\
c\x08\x12 \xab*\x86\xc8\x8f\x1b\x03\xf5#\x96}\x96\
\x91\x94\xf7\x22H\x90,\x0b\xcf\x0d\xc3n\x99\x01\xaf]\
\x9a\x91\x98\xb7 H\xa0\xac\xaf\x15\x12?z\x07\xbet\
[\x0a\x9b\xc8y\x1d\x82\x04\xcb\xb6F\xa1\xb8\x7f\xfe\x82\
\x81\xa5\xeba\x22iS\x04\x09\xfaZV\x08\x1e\x5c,\
\xfc\xa1\x91\xe3\xd3<\x06\xa2\xd6A\x90\xa09\xd8\xd9u\
?N\x9fkf\xe5:\x1a\xc8z\x16\x82\x04\xcf_\xf3\
8\xedG\xd0\x1f\xa7\xfe\x0fK\x0c\xbcxV\x0aA\x0c\
\xf0ei\x87\xfd\xe8z\xc0\xd8\xbau\x0f>m>\x04\
1\xc1\xca\xba\xae\xeaQ\xe4}\x93\xd7\x00\x8b\x05\x1fx\
\x1f\x82\x98`\xdf\xadn\xfaQ{\x91\xd1e{)\xf8\
\xc4\x1b\x10\xc4\x0ccJ\xb8\xa7G\x5c\xca~\xb3\x8b\x96\
\x15\xfc\xe3l\xbf \x88!\xd6\xb5v\xcd\x8f\x0aS\x8d\
/\xda\xfc\x02A\x87\xfe\x0eA\x8c1:\xc9-A\xaa\
\xec7\xbff\xaf\x06\x1dz2\x82\x98c\xc5en\x19\
\xd2\xdf\x825\x0b\xfa\xd5\xc2\xf1\x08b\x92\x09\xc9.\x09\
\x92o\xb1\xf9\x15\xdb{a\xb0\x99\xdfF\x10\xa3\x1c\x18\
R\xd8!C\x9aX\xb0\x8f\xda\xa6`\x9f{\x1f\x82 \
\xa6\xef\x89\xb4s\xc8\x90\x91\x16,\xd8\xe22Q;\xac\
\x8c\xb6 \xb1\xd8\xf4\xe6\xce\x08Rj\x8b\x05\xeb\xf5S\
\x85\x00\x13?\x84 \x160\xcd\x19E\xee\xb0a\xb9\x96\
U\x8eX\xe0\xc8\x0b\x12\x8b\xcdp\xe4\xaeH\xdct\x1b\
VkK\xb3\xc0\x02\xdf\x88 \x96\xf0U\x1b'\x1e\xf2\
\xadq\xd0\x86\xc5:xwPy[\x22\x885\xacM\
+\xe7\x80!\x03\xedX\xacq\xa7\x06\x13\xb7\x01\x82X\
\xc4\xfe\x91\xf5\xad\x17\xa4\xe0R;\xd6j\xcd\xe5\x81\xc4\
\xad\x86 v\xf1\xfd}e,7\xc4\x9a\x83\x8e\xf7\xcb\
\x07\x90\xb6<\x82\xd8F\xe6\xa4\xdb\x8a[m\x885\xfb\
\xd5\xeez\xd8\xff\x8f\xc1\x17A\x10\x1b\x0f\xb5\xc6\xb7\xb1\
\xf8\x06{\x99\xed\xd6,\xd4\xba{\xf3\xfb}\xd9.\x13\
A\xect$\xfd\xfe*\xb6\x1ar\xb7E\xeb\xb4\xfaa\
\x9f\xcf\xd6\xb7!\x88\xb5,\x19zy~\x1b\x05\x89\x9f\
i\xd3*\xed}\xb5\xb6_A\x13kv|f/\x82\
\xd8L\xc6\xac!m\xed{\xf7\xf0\xdc\x83v\xad\xd2\x82\
\xd43tGLj\x942l\xfa^G\xa6$\xc2\x82\
\x1c\x22{\xc1\xb0\xceU\xe2\xad2\xc4\xba\xaf\x02dN\
\xef\xadk\xef\xfc\xfc\xe7w\x194i\xa3S\x13\x12q\
A~g\xf7\xccaw7*\xaa\xe7\xbc3N\xfc\xbf\
(\xb4\xc2\xc6#\xd2a\x1d\xca\x8aR\x95h\xd0e\xc0\
\x98\x05\x19\xee\x0d\x07\x82\x1c\xf9-Y\xfa\xe1sw]\
~f\xa2@\x8e*\x9dG\xac\xd3\xf01\x9a\xd6\x96\xae\
\xd0O#R\x1a\xe5\xfecT\x85k\xb7{t\xc4\xcc\
-\xce\xce\x05\x82\xfc\xcf\x89\xc9\xb2I\xc3\xfav\xb9\xb2\
v\xd9\xdc<\xbdU\xe8\xdck\x1f|\xf1\xf3\xdf\xaf\xc9\
\xac\xd7p\x9fe\x9c\xbd\xcb\x93\xf9\xf3G\xcf\xdcvQ\
\xb9\x938(-pv\xb3.O\xbc>u\xb5\xeb\x03\
\x81 9\xfd\xa2\xac\xffq\xe2\xa8\x97\xd3\xfa\xf4\xb8\xe5\
\xfa\xe6u\xab&'\x9f\x9a\x94td\xf6\x93\x92\x92\xca\
%'W\xa9\x7fE\x87\xee\x8f>\xff\xe6\x87\xd3\xd7\x1d\
\xfb\xdf\xbd\xa8\xe1\x0e\xf3N\xdb\x17\xe7\xe0\x8ai\xef\x0e\
}\xa2\xfb\x8d\xcd/</\xb9dR\xbe\xc3e\xe7K\
*Q\xb9N\xb3\xebo\xed5h\xd4\xa4y\x1b\xc32\
\x07\x08\xa2\x99,\x0d\x8f|\xa5\xb0\x8c\x08\x12Zf%\
\x84\xecf\x08\x82\x80V\xee\x92\xff\x84\xd4\xc9d\x19\x11\
$\xacl\xd7\xf0\xcc\xf0\x10\x96\x11AB\xcbp\x0d\x0f\
\xba\xaeb\x19\x11$\xacd_*7\xe4z\x96\x11A\
B\xcb\x82D\xb9!\x1f\xb1\x8c\x08\x12Zz\xc9\x05\xa9\
\xb8\x8beD\x90\xb0\xb2G\xc3\x13\xb0\x0f\xb1\x8c\x08\x12\
Z\xc6\xcb\x05\xc93\x9beD\x90\xd0\xd2Jn\xc8\x05\
\xdc\x0cA\x90\xd0\xb2\xa2\x90\xdc\x90\x17YF\x04\x09-\
\xfd\xe5\x82\x14]\xcd2\x22HX9P5\x22\x9b\xd7\
\x22\x08(\x91\xae\xe1\x05\xc5\x09,#\x82\x84\x96\xf6r\
AN\xdf\xcd2\x22HXYWLnH\x1f\x96\x11\
AB\xcb`\x0d7C\xe6\xb0\x8c\x08\x12V25\xec\
\xbaV?\x8buD\x90\xb0\xf2\xad\x86\x1d\xb7^e\x19\
\x11$\xb4\xdc\xaea\x1b\xc2\x0d,#\x82\x84\x95-%\
\xe5\x86\xdc\xcc2\x22Hhy]\xc3\xcd\x90t\x96\x11\
A\xc2J\xf6%rA\xce\xdc\xc7:\x22HX\x99\xa7\
\xe1\xe5\xc2\xc7YF\x04\x09-\xf7\xcb\x05\xc9\xbb\x90e\
D\x90\xb0\xb2S\xc3\xf7\xa7\x1bg\xb3\x8e\x08\x12V\xde\
\xd7p\x9e\xfe\x16\xcb\x88 \xa1\xa5\x85\x5c\x90\x12\x9bX\
F3lC\x10\xdf\xf9E\xc3\xc7\x10oe\x19\x8d\xb0\
\xbd\x1e\x82\xf8\xcf\xe3\x1a\xbe\x5c5\x99e4\xe3\x07\x82\
\xf8\xcf~\x0d\x1f\x9d\xae\xc2\xcd\x10\x03\xc7W\xbf\xf9\x81\
 \x010Q\xc3yz\x7f\x96\xd1\x88\x1f\x08\x12\x047\
\xc8\x05\xc9\xb7\x98e4\xe1\x07\x82\x04\xc1\xaa\xc2rC\
\x9ap3\xc4\x84\x1f\x08\x12\x08\xcfj8\xc8\x1a\xc52\
\x1a\xf0\x03A\x02!\xe3<n\x86\xb8\xe9\x07\x82\x04\xc3\
\xb48\xb9!w\xb0\x8c\x81\xf9q\x81\x87 \xc1\xd2E\
\xc3\xcd\x90\xe9,c\xe0~ H@l.!7\xa4\
\xfaA\xd61h?\x10$(^\xd6p\x9e>\x90e\
\x0c\xda\x0f\x04\x09\x8a\xac\x0b\xe5\x82\x14X\xc2:\x06\xec\
\x07\x82\x04\xc6\xf7\x09rC.g\x19\x03\xf6\x03A\x82\
\xa3\x87\x86\x83\xac\xf7Y\xc6`\xfd@\x90\xe0\xd8QV\
.H\xe9m\xacc\xa0~ H\x80\x8c\xd2\xf0\x13r\
7\xcb\x18\xa8\x1f\x08\x12$M\xe5\x82\xc4\xcf`\x19\x83\
\xf4\x03A\x82dQ^\xf9\xf7\xd3\xc7\xb0\x8c~\xb1\xe5\
|\x0fA\xcc\xd2G\xba\x89\xdc\x90\xfd,b\x90\xbf\x1f\
\x08\x12,{+I\xf4\xa83\x82\x8f!\x04\xed\x07\x82\
\x04\xcb\xa7\xeaOb\xb5\xe6Q,\x03~ H\xc0\x5c\
\xad\xa6Gb\xe7\xf9\xac\x9d\x09?\x10$`V\x16R\
\xd0\xa3P\xcaJV\xce_?\xeaz\x08b\x07\x03r\
\xff\xa6T\xeaf\x96\xcd\x94\x1f\x08\x124\x07\xce\xc9\xe5\
\xa7\xa0\x87\xeca\xd1\xcc\xf9\x81 \x81\xf3\xaf\xdc\xbc\x5c\
x\xee\x88\x0cV\xcc\xa4\x1f\x08\x12<\x1dOZ\x8fF\
\x13\xd8\xca\xc4\xb0\x1f\x08\x12<\xeb\x8b\x9f\xdcC%\xad\
y\xaa\xc4\xbc\x1f\x08b\x80\xa1'\xf3\xd5\x9c\xce\x0bX\
(\x0b\xfc@\x10\x03d\xd5;\x91\x1e\x85S~e\x99\
\xac\xf0\x03AL\xf0\xdd\xf1_.,\x99\xba\x855\xb2\
\xc4\x0f\x041\xc2\x9d\xc7\xe9H%\xae\xeb\x06\xc6\xd6\x13\
\xfa\x81 f\x1aS*\xa7~\x9c\xc7u]\xab\xfc@\
\x103\xbc\xc5u]7\xfc@\x103d7\xf9\xb3\xeb\
\xba_\xb30\xb6\xf9\x81 \x86\x98\x9f\xf8\x87\xeb\xba\x8b\
X\x15\xfb\xfc@\x10S\xf4\xfc\xaf6\x14IY\xcd\x92\
\xd8\xe8\x07\x82\x98b\xcf\x19G\x9bP*u+\x0b\x12\
\xb0\x1fu<\x04\xb1\x9b\xb1GZ\x90<d/\xaba\
\xab\x1f\x08b\x8e\xab~o@\xad\x11\x99,\x85\xbd~\
 \x889\x96\xe4?t]\x97u\xb0\xda\x0f\x041\xc8\
\x80\xf6\xb3Y\x04\x03l\xca\xcd\x07\xf1X.\xe0\xf7\x03\
A\x00\x94\xfc@\x10\xc0\x0f\x04\x01P\xf3\x03A\x00?\
\x10\x04@\xcd\x0f\x04\x01\xfc@\x10\x80XlcM\x0f\
A\x00t\xfa\x81 \x80\x1f\x08\x02\xf8q\xae\x87 \x00\
\x9a\xfd@\x10\xc0\x0f\x04\x01\xfc@\x10\x00\xed~ \x08\
\xe0\x07\x82\x00~ \x08\x80v?\x10\x04\xf0\x03A\x00\
?\x10\x04@\xbb\x1f\x08\x02\xf8\x81 \x80\x1f\x08\x02\xa0\
\xdd\x0f\x04\x01\xfc@\x10\xc0\x0f\x04\x01\xd0\xee\x07\x82\x00\
~ \x08\xe0\x07\x82\x00h\xf7\x03A |l\xd0\xe7\
\x07\x82@\xf8\xfc\xa8\xe1!\x08@\x10~ \x08\xe0\x07\
\x82\x00~ \x08\x80v?\x10\x04\xf0\x03A\x00?\x10\
\x04\xf0\xc3C\x10\x80\x00\xfd@\x10\xc0\x0f\x04\x01\xfc@\
\x10\xc0\x0f\x04\x01\x08\xd2\x0f\x04\x01\xfc@\x10\xc0\x0f\x04\
\x81\xc8\xfaQ\xddC\x10\x00\x03~\xfc?\xf8\x1a\xb7\x9e\
cx\x86\x0f\x00\x00\x00\x00IEND\xaeB`\x82\
\
\x00\x00\x02\xbb\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00\x80\x00\x00\x00v\x08\x03\x00\x00\x007+k\xfd\
\x00\x00\x003PLTE\x00\x00\x00A\xcdRP\xca\
NA\xcdRA\xcdRF\xccPH\xcbPB\xccQ\
I\xcbPD\xccQE\xccQJ\xcbOE\xccQD\
\xccQD\xccQC\xccQA\xcdRz\x18s\xcb\x00\
\x00\x00\x10tRNS\x00\x186Oc\x9d\xab\xae\xb8\
\xbd\xc9\xcd\xd6\xdf\xe9\xf0\xb8\x82#\xe9\x00\x00\x02'I\
DATh\xde\xc5\xda[v\x830\x0c\x04PT\xd3\
\x90\x07\xa5\xda\xffj\xfb\x91\xa4!`\x9f\x18i\xa4\xd1\
\x06\xe6\xf2\x01#\xc0\xc3\x101E\x17\x19\x98ST\x97\
\x22\x5c\x80\xea(d\x00Sp\x07\x10\x05\x0f\x00O\xf0\
\x04\xe8$d\x00K\xf0\x02\x90\x04+\x00G\xb0\x06\xe8\
U\xc8\x00\x86\xe0\x1d@\x10l\x00:\x0b\x19\x90^\x8e\
;@\xb6`\x0fH\xae\xe7\x0a \xb7\x18\xaa\x80LA\
\x1d\x90(h\x00\xf2\x04-\x00\xa4\x18\xa4|\x9eQ\x03\
\x05\xdf\xea\x1a\xbf\xe0\xacd\x81\x17\xe0\xae&7\xc0+\
\xf0\x03\x9c\x02\x00\xc0W\xcf\x08\x80\xab\x1c!\x00\x8f\x00\
\x03p\x08@\x00\xfd\xb5.\x08(\x80\xb9\x9ap\x00\xa3\
\x00\x08\xb0\x09\x90\x00S1@\x01:\xb1\x01\xda\xbf\x86\
\xdc\x84\x09\x90Q\xb5\x10\x012j\x10`\xec\xcf\x8f\x00\
t\xbe.\xdd\xf3\x03\x00\x9d} \x93\xc6\x00\x0e\xe6\xc3\
\x01\xf3\xc1|4\xa0s/\x93\x8b\xc6\x00z\xf3\xaf\x1a\
\x030\xe4C\x01\x93!\x1f\x09\xe8\xcd\xff\xd1\x18@o\
\xfe\xa21\x00c>\x0c\xd0\xb7\x06I\xd9\xe6\xa3\x00\xe3\
\x91\xc7\x7f\x0d\xf0u\xfe<sD\xfe\x13\xd03%\x22\
\x1f\x018V\xbfx\xc0\xd1\xfaC\x03\x9c\xf9n\xc0\xec\
\xcc\xf7\x02\x8e\xd7\xef\xee\x0a:\xee\xbf\x93\xb4\x00\x96\xfa\
\xb3Li\x00\xb2\xf2[\x80)+\xbf\x01\xb0\xd5/\x0e\
`\xad?\x14 3\xbf\x060\xd7/\x08\xe0\xa9\x1f\x04\
 9\x7f\x07\xc8\xce\xdf\x00|\xf5\xeb\x07x\xeb\xcf\x0b\
`\xe4\xaf\x013#\x7f\x05\xe8\xfd+q\xd1\x18@\xf7\
_\x11\x8d\x00\xc8\xed\xd4\xfd\x09t\x89\x00\x1c\x19Y\xc8\
\x00\xb0\xc0\x00@\xac!>\x00T`\x02 \x056\x00\
P`\x04\xe0\x9e\x87V\x00L`\x06\xa0\x04v\x00H\
\xe0\x00`\xf6\x12\x0f\x00\x22p\x01\x10\x02\x1f\x00\xf0v\
\xe0\x04\xb8\xab\x09pj\xaf-\x18;N0!\x8ej\
5\x05eH\x9aV1\xa4\x01Z\x82<@C\x90\x08\
\xa8\x0b2\x01\xd5bH\x05\xd4\x04\xb9\x80\x8a \x19\xb0\
\x17d\x03v\xd5\x94\x0e\xd8\x0a\xf2\x01\x1b\x01\x01\xf0^\
\xcf\x0c\xc0[5q\x00+\x01\x09\xf0\x12\xb0\x00\xff\xc5\
@\x03<\x05<\xc0C@\x04\xdc\x05L\xc0 \x13\x19\
0\xc8\x14\x04\xf8\x03L-q\xaa\x9e\xb6\xb9\xda\x00\x00\
\x00\x00IEND\xaeB`\x82\
\x00\x00\x02\xf6\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00\x9e\x00\x00\x00\x9d\x04\x03\x00\x00\x00\xb7?\xdc\xe6\
\x00\x00\x00$PLTE\x00\x00\x00\xff\xff\xffA\xcd\
RA\xcdRA\xcdRH\xcbPB\xccQE\xccQ\
J\xcbOD\xccQC\xccQA\xcdRE\xef\x01u\
\x00\x00\x00\x0btRNS\x00\x00\x18Oc\xab\xae\xc9\
\xd7\xe9\xf0\xc1\xad8\xf5\x00\x00\x02vIDATh\
\xde\xcd\xda\xdbM\xc40\x10\x85\xe1\x15i\x00:\x88D\
\x07\xc0\xdb\xbem))!=\xd0\xc0\xd6\xb2J\xb4\xeb\
\xe6x\x08\xb98\xbe\x8dg\xfe\x07\xa7\x80OG \x0e\
\xe3\xb1/\x17\xc1\xd7=\xfb\x0b\xf9u\xee\x0e{\xaf/\
\xd6s\x8f\x9e\xf5\xdc\x0d\xf6\xc8\x80\x9ds\xce\x0d\xb07\
\xf5\xac\x07\x06\x5c<.\xe0\xe2\xb9\x11\xf6\xe6\x9e\xf5\xb0\
\x80\xabG\x05\x5c=\xaa\x166\x0f\xea\xad\xcd\x83\x02\xee\
\x1e\xd3[\xbb\xc7\xd4\xc2\xc1Cz\xeb\xe8\x11\x01\x8f\x1e\
Q\x0b\x9e\x07\xd4\x82\xe7\x01\x01}\xcf\x1e\xd0\xf7\xec\xb5\
p\xf2\xcc\xb5p\xf2\xcc\x01\xcf\x9e5\xe0\xd9\xb3\xd6B\
\xe0\x19{+\xf0\x8c\x01C\xcf\xd6[\xa1g\xab\x85\x88\
g\xea\xad\x98g\x09\x18\xf3,\xb5\x10\xf5\x0c\xb5\x10\xf5\
\x0c\x01\xe3\x9e>`\xdc\xd3\xd7B\xc2S\xd7B\xc2S\
\x07Ly\xda\x80)O[\x0bI/\xd2[\x9f\xdf\xe5\
\xef\xc7\xc9\x03^\x9d\xe5\x0b{\xcb\xe6\x85\xb5`\xf4\x82\
\xde\xb2z\xe7\x80V\xef\x5c\x0bf\xefT\x0bf\xef\x14\
\xd0\xee\xf9\x01\xed\x9e_\x0b\x80\xe7\xd5\x02\xe0y\x01\x09\
\xef\x18\x90\xf0\x8e\xb5\x80x\x87\xdeB\xbcC@\xc6\xdb\
{\x8b\xf1\xf6Z\x80\xbc\xad\xb7(o\x0dHyk-\
`\xde\x7f-`\xde\x7f@\xce[\x02r\xdeR\x0b\xa0\
7\xf7\xac\xe7F\xd8\x9b{\xd6sw\xd8{\xf6\xac\xe7\
\xee\xb0\xf7\x82=\x07{\xcf\xb6\x7f~3\xfc\xfb\x1d\xdb\
\xfe\xfb\x80\xff~'\xb8_\x86\xb6\xfb\x19\xfe\xff\xf1\x80\
\xff\xbf\xdd\x9a\x9e\x0f\xb6\x01\xebJ\x15\x01\xea\xed\x03j\
\x93\xf3\xe9\x0c\xcf\xcfc\xdb\xe7\x0f\xf8|4\xc1\xe7\xb7\
\xa1\xed\xf3/|>\x7f\xc0\xfb\x83[\xd3\xfb\x97`\x81\
u\xa5\x8a\x00\xf1\xc2\x05 \xbd\xff3\xed'\x95\x0b\xd4\
N\xb4\xc9io\xbf\x0b\xef\x9f'x?>\xb0\xfb{\
\xfa~!\x1e\xef#\xff\xbd'\xbd\xc4\x05\x92\xdaK\x5c\
pi\xbd\xd4\xfd\x96\xd2K^\x10*\xbdd\x11\xe8\xbc\
\xf4\x05\xab\xceK\xf7\x94\xca\xcb\x14\x81\xca\xcb\x14\x81\xc6\
\xcb\xf5\x94\xc6\xcb\xf5\x94\xc2\xcb\x16\x81\xc2\xcb\xf6T\xbd\
\x97\xef\xa9z/_\xa3\xd5^\xe1\xa2\xbb\xda+\x5c\xc4\
\xd7z\xa5{\xf8J\xaf\xf8\x90\xa1\xd2+\x0e,u^\
\xf9!H\x9dW\x9e\xa7\xaa<\xc1\xc0R\xe5\x09\x06\x96\
\x1aO2O\xd5x\x92y\xaa\xc2\x13\x0d,\x15\x9eh\
\x9e\x92{\xb2yJ\xee\xc9\xc6=\xb1'|\x90#\xf6\
\x84\x0f\x86\xa4\x9e\xf4\xbd\x90\xd0\x13?\xb8\x12z\xe2\x83\
\x95\xcc\x93?X\x93y\xf2s\x9f\xc4{\xfb\x15\xc7\xfb\
\x03\xfd\xb9\xba%(\xb7\xb7:\x00\x00\x00\x00IEN\
D\xaeB`\x82\
\x00\x00\x02c\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00w\x00\x00\x00w\x04\x03\x00\x00\x00\xcb\x17\xc2\xb9\
\x00\x00\x00'PLTE\x00\x00\x00\xff\xff\xffA\xcd\
RA\xcdRA\xcdRB\xccQD\xccQE\xccQ\
E\xccQD\xccQD\xccQC\xccQA\xcdR\xeb\
\xfeC\xda\x00\x00\x00\x0ctRNS\x00\x00\x18Oc\
\xae\xbd\xc9\xd6\xdf\xe9\xf0v\x89\xbe\xf7\x00\x00\x01\xdfI\
DATX\xc3\xc5\xd8\xbbm\xc30\x14\x05P\xc1Z\
 \xd9\x80e\x902I\xe9\x22\xd0\x08\x9c\xc4\xf0H\xd2\
\x22)(\x07R\xdeP)\xa2\x0fI\x89|\x9f[\x84\
\xfd\xc1\x05l]>\x92MS^\xed\xb7k\xcc\xab\xa5\
\x1e\xc1\xb3\x030\x10\xdd\x12\xfd\xbc\x01\x98\x82\x030u\
\x08\xb6F\xb7DD\xe4\x11<:\x00\x1b\xa3\x17l\x8b\
^0\xdd\x11l\xea\xc7\x8aM\xd1\x1b\xb6Do\xd8\xd2\
\x8f\x1d\x1b\xaa\xb9cCt\x84\xf5\xd5\x8c\xb0\xbe\x1f1\
VW3\xc1\xda\xe8\x04k\xfb\x91be?R\xac\x8c\
\xce\xb0.:\xc3\xba~\xe4X\xd5\x8f\x1c\xab\xa2\x0fX\
\x13}\xc0\x9a~\x1c\xb1\xa2\x9aG\x9cG_\xde\x8b\xeb\
\xe3\x88\xb3j\xbe\x92j\xa5\xfd\xb8\xeapZM-N\
\xa2\xb58\xe9\x87\x1a\xc7\xfdP\xe38Z\x8f\xa3h=\
\x8e\xfaa\xc0{?\x0cx\x8f\xb6\xe0-\xda\x82\xb7~\
\x98\xf0ZM\x13^\xa3mx\xa9\xa6\x0d/\xfd0\xe2\
\xbfjZqp\x00&\x8f\xe0\xd1\x01\x98<\x82G\x07\
`\xba!\xf8\xeb\xdf\xf0'\x80\x03\xf2\x83u\xc0_5\
\x00\x1f\xc9\x8c|\xdb=\xd0\xaa\x09\xe9\xf3\x1d\xd8I\x96\
\xcd\xf7\x0a\x04\xdb\xf0\x03\xd9z=\xb0\xe9\x8f\xc8\xb8\xf1\
\xc0\xa0\x0b\xc8\x88\xed\x80\xe1\x1e\x90cE\x07\x1ch\x06\
\xe0(5#'\xc0\x1e8{N\xc9\xa9\xf7\xc5z\x06\
\xd3\x1f\xd6\xe5W\x94\x96\x0b\xd6\xe1\x07rA\xf1\xc0\xd5\
hD.e\x1e\xb8\x0e\x06\xe4\x22z|sx.\xad\
\xa7\x1c\x9f\x04\xcb\xf1\xc9c\x87\x18\x0f\xce\x8eO/\xb0\
R|zu\x16\xe2\xc9\x01\xf8\xbc\x112\x5c\xa8\xa2\x0c\
\x17\xaa(\xc2\xa5*\x8ap\xa9\x11\x12\x5c\xac\xa2\x04\x17\
\xab(\xc0\xe5*\x0ap\xf9\xf9\x8f\xc7\x95=\x80\xc7\x95\
wG\x16\x0f\xce\x8e\xaboI\x1c\xae\xbeb1xr\
\x00\xae\x0f\xa7:f\xa6b\x1d3S\xb1\x8a\xb9\xa9X\
\xc5\xdcp\xaaav*V\xf0\x85{\x8b\xff\x05\xc8d\
f\xae\xfe\xc1\x88\xef\x00\x00\x00\x00IEND\xaeB\
`\x82\
\x00\x00\x1cV\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x02X\x00\x00\x01\xad\x08\x03\x00\x00\x00\xfd\x0f\xfa\xd9\
\x00\x00\x00\x09pHYs\x00\x00'\x8b\x00\x00'\x8b\
\x01q}\xe4H\x00\x00\x00\x19tEXtSof\
tware\x00www.inksca\
pe.org\x9b\xee<\x1a\x00\x00\x03\x00PL\
TE\x00\x00\x00\x00\xff\xff\x00\xff\x80U\xff\xaa@\xbf\
\x803\xcc\x99+\xd5\x80$\xdb\x92 \xdf\x809\xe3\x8e\
3\xe6\x80.\xe8\x8b+\xd5\x80'\xd8\x89$\xdb\x803\
\xdd\x880\xdf\x80-\xe1\x87+\xe3\x80(\xe4\x86&\xd9\
\x801\xdb\x86.\xdc\x80,\xde\x85+\xdf\x8a)\xe0\x85\
'\xe2\x89/\xe3\x84.\xdb\x89,\xdc\x84+\xdd\x88)\
\xde\x840\xdf\x87.\xe0\x83-\xe1\x87,\xdb\x83+\xdc\
\x87)\xdd\x83/\xdd\x86.\xde\x83-\xdf\x86,\xe0\x83\
+\xe1\x86*\xdb\x82.\xdc\x85-\xdd\x82,\xde\x85+\
\xde\x88+\xdf\x85*\xe0\x87.\xe0\x85-\xdc\x87,\xdd\
\x84+\xdd\x87+\xde\x84*\xdf\x86.\xdf\x84-\xe0\x86\
,\xdc\x84+\xdc\x86+\xdd\x84.\xde\x86-\xde\x84-\
\xdf\x86,\xdf\x83+\xe0\x85+\xdc\x83.\xdd\x85-\xdd\
\x83,\xde\x85,\xde\x87+\xdf\x85+\xdf\x87-\xe0\x85\
-\xdd\x86,\xdd\x85,\xdd\x86+\xde\x84+\xde\x86-\
\xdf\x84-\xdf\x86,\xe0\x84,\xdd\x86+\xdd\x84+\xde\
\x86-\xde\x84,\xde\x85,\xdf\x84+\xdf\x85+\xdd\x84\
-\xdd\x85-\xdd\x84,\xde\x85,\xde\x86+\xde\x85+\
\xdf\x86-\xdf\x85-\xdd\x86,\xdd\x85,\xde\x86+\xde\
\x85+\xde\x86-\xdf\x85-\xdf\x86,\xdf\x84,\xdd\x86\
+\xdd\x84+\xde\x85-\xde\x84,\xde\x85,\xdf\x84,\
\xdf\x85+\xdf\x84+\xdd\x85-\xdd\x84,\xde\x85,\xde\
\x84,\xde\x85+\xdf\x86-\xdf\x85-\xdd\x86,\xdd\x85\
,\xde\x86,\xde\x85+\xde\x86-\xde\x85-\xdf\x86,\
\xdf\x85,\xdd\x85+\xdd\x84+\xde\x85-\xde\x84,\xde\
\x85,\xde\x84,\xdf\x85+\xdf\x84+\xdd\x85-\xdd\x84\
,\xde\x85,\xde\x84,\xde\x85+\xde\x86+\xdf\x85-\
\xdd\x86,\xdd\x85,\xde\x86,\xde\x85+\xde\x86-\xde\
\x85,\xde\x85,\xdf\x85,\xdd\x85,\xdd\x85+\xde\x85\
-\xde\x84,\xde\x85,\xde\x84,\xdf\x85,\xdf\x84+\
\xdd\x85-\xde\x84,\xde\x85,\xde\x84,\xde\x85,\xde\
\x86+\xdf\x85-\xdf\x86,\xdd\x85,\xde\x86,\xde\x85\
,\xde\x86-\xde\x85,\xde\x85,\xdf\x85,\xdd\x85,\
\xdd\x85+\xde\x85-\xde\x85,\xde\x85,\xde\x84,\xde\
\x85,\xdf\x84+\xdd\x85-\xde\x84,\xde\x85,\xde\x84\
,\xde\x85,\xde\x86+\xde\x85-\xdf\x86,\xdd\x85,\
\xde\x86,\xde\x85,\xde\x85+\xde\x85,\xde\x85,\xde\
\x85,\xdf\x85,\xde\x85,\xde\x85-\xde\x85,\xde\x85\
,\xde\x85,\xde\x85,\xdf\x85,\xdd\x85-\xde\x84,\
\xde\x85,\xde\x84,\xde\x85,\xde\x86,\xde\x85-\xdf\
\x86,\xdd\x85,\xde\x85,\xde\x85,\xde\x85+\xde\x85\
,\xde\x85,\xde\x85,\xdf\x85,\xde\x85,\xde\x85+\
\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xdd\
\x85-\xde\x85,\xde\x85,\xde\x84,\xde\x85,\xde\x86\
,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\
\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\
\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85\
,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\
\xde\x85\xf8\xdf\x16\xb8\x00\x00\x00\xfftRNS\x00\x01\
\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10\x11\
\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f !\
\x22#$%&'()*+,-./01\
23456789:;<=>?@A\
BCDEFGHIJKLMNOPQ\
RSTUVWXYZ[\x5c]^_`a\
bcdefghijklmnopq\
rstuvwxyz{|}~\x7f\x80\x81\
\x82\x83\x84\x85\x86\x87\x88\x89\x8a\x8b\x8c\x8d\x8e\x8f\x90\x91\
\x92\x93\x94\x95\x96\x97\x98\x99\x9a\x9b\x9c\x9d\x9e\x9f\xa0\xa1\
\xa2\xa3\xa4\xa5\xa6\xa7\xa8\xa9\xaa\xab\xac\xad\xae\xaf\xb0\xb1\
\xb2\xb3\xb4\xb5\xb6\xb7\xb8\xb9\xba\xbb\xbc\xbd\xbe\xbf\xc0\xc1\
\xc2\xc3\xc4\xc5\xc6\xc7\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf\xd0\xd1\
\xd2\xd3\xd4\xd5\xd6\xd7\xd8\xd9\xda\xdb\xdc\xdd\xde\xdf\xe0\xe1\
\xe2\xe3\xe4\xe5\xe6\xe7\xe8\xe9\xea\xeb\xec\xed\xee\xef\xf0\xf1\
\xf2\xf3\xf4\xf5\xf6\xf7\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xeb\x08\xd9\
5\x00\x00\x17\xccIDATx\xda\xed\xdd{\x9cM\
\xe5\xfe\x07\xf0\xb5gh\x90\xeb\xb8D\xe4\xb8F\xb9\x95\
q\x8b\x944'\xe4\x1aM\xa1\xa2NE\xc2\xe8\xa2\xa6\
R\x19Q8(\xa4N\xa3\x9bD\x0e\x91J9\x95\xe6\
\xd0\xa9\x8e\x5c\x0e\xe1'\x84\xdc\xefL\x0d&\xc3\xcc\xec\
9\xbf\xdf\xef\x9c\x84\xb9\xec\xfdYk}\xd7\xf7Y3\
\x9f\xcf\xdf^\xf3|\xd7\xf7y\xdb{\xed\xb5\x9e\xf5,\
\xcbr\x9eJ\xff\x93\xcd0\xe7\x87\xae\x18CaU\xda\
\xc0.2\xee\xc3\xa2+F\x02\x16]1\x12\xb0\xe8\x8a\
\x91\x80EW\x8c\x04,\xbab$`\xd1\x15#\x01\x8b\
\xae\x18\x09Xt\xc5H\xc0\xa2+F\x02\x16]1\x12\
\xb0*\xd2\x15#\x00\xab\xe2zv\x8eq\x1f\x16]1\
\x12\xb0\xe8\x8a\x91\x80EW\x8c\x04,\xbab$`\xd1\
\x15#\x01\x8b\xae\x18\x09Xt\xc5H\xc0\xa2+F\x02\
\x16]1\x12\xb0\xe8\x8a\x91\x80EW\x8c\x04,\xbab\
$`\xd1\x15#\x01\x8b\xae\x18\x09X\x15\xd7\xb1U\x8c\
\xfb\xb0\xe8\x8a\x91\x80Un5\x1b\xc5\xb8\x0f\x8b\xae\x18\
\x09Xt\xc5H\xc0\xa2+F\x02\x16]1\x12\xb0\xe8\
\x8a\x91\x80EW\x8c\x04,\xbab$`\xd1\x15#\x01\
\x8b\xae\x18\x09X\xe5V\xb1=\x8c\xfb\xb0\xe8\x8a\x91\x80\
U\x96\xae\x18\x01Xt\xc5H\xc0\xa2+F\x02\x16]\
1\x12\xb0\xe8\x8a\x91\x80EW\x8c\x04,\xbab$`\
\xd1\x15#\x01\x8b\xae\x18\x09XeW\xb2'\x8c\xfb\xb0\
\xe8\x8a\x91\x80EW\x8c\x04,\xbab$`\xd1\x15#\
\x01\x8b\xae\x18\x09Xt\xc5H\xc0\xa2+F\x02\x16]\
1\x12\xb0\xe8\x8a\x91\x80EW\x8c\x04\xac\xb2+\xd8\x08\
\xc6}Xt\xc5H\xc0*\xc0\xcfy\xa5\xa6\xa4\x1c\xdb\
\xfe\x7f9\x90\x92\x92\x92\xc9\xd9\xf6\x14V\xc1r\x95\xb6\
\xf5\xeb\xb9\x93\x13\xfa\xdfrcL\x9dJ\xc5.\xb8\xc3\
\x1eQ\xaef\x93\xb6\x9d\xfb\xc4\xbf0\xe3o\xeb\x0fr\
\xea\x85a\x15\x94\xf5W\xc7V\xcc\x1a\xd9\xb7y\xb9\xb0\
\xf7\xad/Z\xf7\xe6a\xd3>\xdf\xce\x8f1)X\x05\
\xc0\xd5\xce\x85\xcfv\xadf\xd9K\x89V\x0f$\xad:\
E\x07\xee\xc3\xf2w\xf9\x19\xdf\x8d\xef\x1cm9M\x91\
\xab\xe2\x17\x1c\xa6\x05\xc2\xfao\xd6\xbc\xd0\xa1\xa4\xe5V\
\x02\x0d\x06/8A\x0f\x85\x1eV\xe67\xc3.\xb3\xdc\
N\xb1\xd8){H\xa2\x10\xc3\xca\xfc\xa4wiK&\
\x81f\x93\x0ePE\xe1\x84\xf5CbuK2\x11\xb1\
3\xd3\x08\xa3\xb0\xc1J\x7f\xb3\x99%\x9frC\xb7\x92\
Fa\x82ux\x5cU\xcb\x9bD\xc4.\x22\x8e\xc2\x02\
k\xeb}\xc5,\x0f\xd3\xf4\xafY\xf4Q\x08`\x1dJ\
\x88\xb2<N\x83yA\x0a)\xe0\xb0\x8e%\x14\xb7\x14\
\xd2*\x99D\x0a2\xac\xcc\xa9e,\xa5t\xdfE$\
\x05\x16\xd6\x9a\x16\x96^J$\x9e&\x93\x02\x09\xeb\x97\
\xc1\x11\x96j\x1a~C'\x05\x10\xd6\xf2Z\x96v\x02\
\xc3\xf8\xa1U\xd0`e$FZ\x06\xa4\xd9\x8f\xa4R\
\xa0`\xedhm\x99\x91R3h\xa5\x00\xc1J\x8e\xb6\
\x8c\xc9\xe0\x0c\xd7\x0fo[R\xb8\xd9AXnfz\
Q\xcb\xa0\xfc1\xc5\xed\xe3\x9b\x13\xf6\xd8\x0b\x09\xcb\xc5\
\x8bW\x09\x96Y\xa9\xb3\x89\xb0\x0a\x00\xac_o\xb6L\
K\xf4*\xc2\xf2=\xac\xb4X\xcb\xbc\x94\xf9\x96\xb0|\
\x0e\xebd{\xcb\xc4\x5c\xfc%a\xf9\x1a\xd6/\xad,\
3Sb\x09a\xf9\x18\xd6\xe9\x1b-SSb9a\
\xf9\x16VV\x9cen*l!,\xbf\xc2z\xc82\
9\xb5\x0f\x12\x96?aM\xb2\xccN\xcbS\x84\xe5G\
XK#\x0d\x87e\x0d ,\x1f\xc2:X\xc52>\
o\x13\x96\xef`e\x5cg\xbe+\xab\xf8Z\xc2\xf2\x1b\
\xac\x04\xcb\x0f\xa9w\x92\xb0\xfc\x05\xeb\xdb\x08_\xc0\xb2\
\x86\x12\x96\xaf`\xa5\xd5\xf5\x87++b\x19a\xf9\x09\
\xd60\xcb/\xa9y\x82\xb0\xfc\x03\xebk\xf7\xbe\x08#\
\xab\xb6\xbe\xe5\x9eGFOKJzs\xde\xec\xa4\xa4\
\xa4\x09O\x0d\xeas\xd3\x95\xeem\xd4f\xc5\x13\x96o\
`\x9d\xae\xe7\xca\xf2\xf4\xb6\x03_N\xde\x95\xc7B\xe2\
\x94\xb5sF\xf4\xac\xe3\xca\x97\xe1j\xc2\xf2\x0b\xac\x09\
\x8eg\xbbJ\xff\xd7\xd6\x85\xb1\x11\xf2\xe1\x8f\x9fl\xed\
\xf82l\x9b a\xf9\x03\xd6\x01\x87\xdb\xf45\x1f\xb3\
\x06\x98\xeb\xa3\xb3\xfb\x96r6\xde,\xc2\xf2\x07\xac\xbb\
\x1d\x9dL?\xb3\x19\xff\x0d:\xbb\x93\x93\xcf\xad\xaa'\
\x08\xcb\x0f\xb0V88so\xff\xb1\xcd\xbd\xacv>\
\xea`\xbb\x91\xa7\x09\xcb\x0f\xb0\xda\xd9\x9d\xdf@\xcf\xef\
\x1d\x0c{|b\x05\xdb\x0b\x95\x0f\x11\x96\xf9\xb0\xbe\xb4\
;\xbd79}v&u\xa4\xdd\x93\xadG\x08\xcb|\
Xmm\xfe\x0e\x9c\xe7\xc2\xd8\xfb\xfb\xd9\x1b\xbc\xd8^\
\xc22\x1d\xd6b{\xdf\x82C\x8f\xbb3\xfc\x87\xf6\x16\
\xeb\x0c%,\xc3a\x05m\xed\xb2]q\xb1k\x05\x1c\
\xeah\xa7\x80\xa8=\x84e6\xac/\xecL\xebu\xfb\
\xdc\xa4=\xd6\xce\xa5\x87\xc7\x09\xcblX7\xd9\x98\xd4\
\xdb\x5c~\x1d\xdc\x87%\xf0\x1aJ\xa7\x12\x96\xc9\xb06\
\x04\xf09}\xc6\xf5\xbd\xb2WT\xc2\xab\x98BX&\
\xc3\xb2q\xd1}\x8c@\x19\x9b*\xc3e\xd4\xc8$,\
sa\x1d\xc0\xdf\x0d\xf0\x82H!\xeb\xca\xc3\x85\xcc'\
,sa\x8d\xf2\xf6\xa49\xbfoC\xf8=\x05\xb1\x84\
e,\xac\xac?\xa0\xb3y\xb3\xd8K\xc2\xdfG\xcf\xf6\
\x02\xdb\x08\xcbTX\x9f\xa1\xae\x9a\x9c\x94+\xe6i\xef\
nE\x13\x96pz\xa17\x7f7I~|\xa27\xc3\
\xabf\x12\x96\x99\xb0\x8e\xa0\xa7\xeeo\x88\x96\xb3\x07\xdd\
\xaa\xf9S\xc22\x13\xd6Dp\x22\xbb\x0b\xd73\x13\xac\
\xe7V\xc22\x13\xd6\xd5\xd8<\x96\xd8!]\x10\xb8M\
e\xb1T\xc22\x11\xd66\xf0\x03b\xbcxE\x1b\xc1\
\xfd\xe5g\x13\x96\x89\xb0\xc6`\xb3X\xf7\x8c|I\xc3\
\xbc\xf9n&,\xd14\xc4fq\xae\x07%\x1d\xc6\x96\
\x94F\xfdBX\xe6\xc1\xda\x84\xb9j\xec\xc9\x1b\xc0\x9f\
\xc0\x8az\x87\xb0\xcc\x83\x05\xde\xceY\xe4IQ\xc7J\
z\xf1]HX\x92\xc1\x96\x8e^\xe1\xd1{\xe5\xe3\xb1\
+\xb6\xe9\x84e\x1a\xac\x03\xd8\xbd9\xaf^\x1b\xb8\xa3\
\x08T\xd6\x17\x84e\x1a\xac7\xa1\x09\xbc\xd4\xb3w\xe8\
\xde\x0a\xd55\x8c\xb0L\x83\xd5\x13\x9a\xc0\x91\x9e\xd5\x85\
=\xe6X\x9b\xb0\x0c\x83u\x1a\xfaa\x1f\xb9\xd3\xb3\xc2\
\x82\xd8^G?\x12\x96Y\xb0\x92\xa1\xe9\xeb\xeaae\
/@\x95\xbdLXf\xc1\xc2V?}\xe8ae\xfb\
\xa0=Jz\x11\x96Y\xb0\xaeEf\xafl\xba\x97\xa5\
A\xeb\xb2\xcag\x11\x96I\xb0\xd2\xa0\xa5X\xf7yZ\
\xdb_\xa0\x0f\xd3\xf5\x84e\x12\xac%\xd0\xe4%{Z\
\xdb\x11\xe8R\xd6T\xc22\x09\xd6\x08\xe8\xeb&\xd3\xdb\
\xe2n@\x8a\xeb\x99\xf7\xdf\xc9L\xca#\xf7\x85\xfd\xd7\
\x07%\xd9\xc8w\x85\x18\x16t\x8a\xd5\xcf\xe3\xe2\xa0\xd7\
\xdbU\xc8\xfb^S\xba\xa5\x93\xc7\x0a/\xac\xd3\xd0)\
\xd6|\x8f\xab\xdb\x0cM\xe3V\xc22\x07\xd6\x0ah\xd5\
\xd3q\xaf\xcb\x83\xde\xbf2\x9b\xb0\xcc\x815\x0d\xe9S\
{\xcf\xcb\x1b\x82\x94\xf7\x10a\x99\x03\xeb.\xfd\xdd\x1a\
\xf2\xcb\x07Hym\x08\xcb\x1cX\x97#}Z\xe1y\
y?#[\xb1\x15?CX\xa6\xc0\xfa\x19Y\x8bU\
&\xd3\xfb\x02c\x90y\x5cKX\xa6\xc0\x82\xf6\x87\xec\
\xacP\xe0\xc3\xae<\x9fMX^\x07z)\xd3(\x85\
\x02\xdfse\xb1\x1fay\x1dhw\xf5\xc5\x0a\x05\xfe\
\xe8\xca\xafV\xc2\xf2:\xd0\xb3\xf5\x87\x15\x0a\x0c\x22\xfb\
\x83T ,C`e\x22\xbb\xe7\xd5T)1\x16\x99\
\xc8\x83\x84e\x06,\xe8Q\xd5\xdbUJ\x84\x1e\x5c]\
BXf\xc0\x9a\x87ti\xa2J\x89\xef#%\xbeH\
Xf\xc0JD\xba\xb4L\xa5\xc4\x9dn\xacC$,\
\x8f\xd3\x17hR\xc4q\x9d\x1a\x91W\x0a\xb4#,3\
`\xb5\x00\x9aT_\xa9\xc6\x0e@\x8dU\x09\xcb\x0cX\
\xe5\x80&uW\xaaq(Pc\xe0$a\x99\x00\xeb\
\x88\x1f\x9a4\xd5\x85\x07*\x08\xcb\xdb\xfc\x13i\xd2t\
\xa5\x22\xff\x86\x14\xf9\x01a\x99\x00\x0b\xda\x9cx\x99R\
\x91\xd0\x06\xa9\xe3\x09\xcb\x04X\xcf\x22M\xda\xa7Td\
\x06\xb2\xcf\xed\x00\xc22\x01V\x7f\xa0G%\x83ZU\
\x22k\x11;\x11\x96\x09\xb0\x90\xdd\xd4\x9b\xaaU\xd9\x19\
\xa8\xb2!a\x99\x00\x0by\x06\xa6\xb7Z\x95\x0f!\x8b\
\x5c\x09\xcb\x04X\xc8\x1b\x98\x9fU\xab\xf2\x15d*\x8f\
\x13\x96>\xac\xa3H\x8ff\xa8\x95\x09\xbd\xf2n#a\
\xe9\xc3Z\xe3\xc6\x8a\x14\xf9\xacG\xca\xfc\x9c\xb0\xf4a\
}\xec\xc25m\xd3\xee\x0f\xbcAX\xfa\xb0^Cz\
tD\xad\xcc r!k\x0ca\xe9\xc3B\xdeHq\
QP\xafC\xd5\x80:\x87\x11\x96>,dc\x84\xea\
\x8a\x1dB\xde\x9c\xd1\x87\xb0\xf4a\xdd\x0e\xb4\xa8\xa5b\
\x87\xba\x02u\xdeHX\xfa\xb0\x90\xfd\xf2z(v\xe8\
~\xa0\xce\xc6\x84\xa5\x0f\x0byM\xe1\x03\x8a\x1dz\x06\
\xa8\xb32a\xe9\xc3\xba\x04hQ\xa2b\x87\x90K\xef\
Er\xfd\x91\x919.\x8f\xf4\x09\xfb\x0f\xf7\x1bg#\
\x7f/\x94\xb0\xb2\x90-\x82\x92\x14;\x04m\x92u\x0c\
\xfa\xd3\xdc5Y ?\xbb\xb04\xd3\x93|\x83\x14\xba\
\x8d\xb0\xb4a\xedD\xe6+Y\xb1C\x1b\x90B\xd7\x12\
\x966\xacu\xc8|\xadT\xec\x10\xf4?`\x19ai\
\xc3\xfa\x072_\x9b\x15;\x94\x82\x14\xfa\x11ai\xc3\
Z\x84\xcc\xd7~\xc5\x0ee \x85\xce$,mX\xb3\
\x90\xf9:\xa9\xd9\x22d\xb3\xa5\x97\x09K\x1b\x16ry\
(2\xa8\xd9\x22\xe4\x82\xdb\x18\xc2\xd2\x865\xd6\xf9Z\
r\x8f\x82\xac\xcd\x7f\x8c\xb0\xb4a!\xef\xfd\xaa\xae\xda\
\xa2\xa6@\xa5C\x08K\x1b\xd6\xa3\xc0t5PmQ\
;\xa0\xd2{\x09K\x1b\xd6\x83\xc0t]\xa3\xda\x22d\
\xddL_\xc2\xd2\x86u\x0f0]\x1dT[t\x07P\
iO\xc2\xd2\x86\xd5\x07\x98\xae\x1e\xaa-\x1a\x08T\xda\
\x89\xb0\xb4a\xf5\x00\xa6+N\xb5E\x83\x81J\xdb\x11\
\x966\xac\x8e\x8e\x97\x92{\x15\xe4!\xfbV\x84\xa5\x0d\
\xebz`\xba\xeeRm\xd1p\xa0\xd2&\x84\xa5\x0d\xab\
\x150]\xf7\xa8\xb6\x08y\x89\xc0\x15\x84\xa5\x0d\xab9\
0]\x03T[\xf44P\xe9\xe5\x84\xa5\x0d\x0b\xb9\x9e\
\xfd\xa0j\x8b\x12\x81Jk\x13\x966\xac&\xc0t\xc5\
\xab\xb6h\x0cPiM\xc2\xd2\x86\x85<\xfd\xf5\x88j\
\x8b\xc6\x89\xdd\xd5$,\x81\x5c\x09LW\x82j\x8b&\
\x02\x95V%,mX\xf5\x80\xe9\x1a\xa1\xda\xa2)\x8e\
\x9fX%,\x0fS\x07\x98\xae\x91\xaa-B\x96$V\
\x22,mX\xb5\x80\xe9zN\xb5E\xd3\x81J\xcb\x13\
\x96\x9f`%\xaa\xb6\xe8u\xa0\xd2\x8a\x84\xe5\xa7\xaf\xc2\
gT[\xf4*Pi\x15\xc2\xf2\xd3\xc9\xfb\x93\xaa-\
B^\x00V\x8d\xb0\xb4a]\xe1\x97\xfdx\xb2_\x04\
*\xadAX\xda\xb0\x90\x0b\xa4\x0f\xab\xb6h<Pi\
\x1d\xc2\xd2\x86\x85\xdc\xd2\x19\xaa\xda\xa2\xe7\x81J\xeb\x11\
\x966\xac\xab\x81\xe9\x1a\xa4\xda\x22d{\xe7\x06\x84\xa5\
\x0d\x0b\xd9\x8c\xf8~\xd5\x16!\xefUlLX\xda\xb0\
Z\x02\xd3u\xb7j\x8b\x9e\x02*mJX\xda\xb0\x90\
\xa5\xc9w\xaa\xb6\xe81\xa0\xd2\x16\x84\xa5\x0d\xab\x030\
]\xbdU[\xf4\xb0\xd8\xa3\xb5\x84%\x90[\x80\xe9\xea\
\xa5\xda\xa2\xa1|\xfc\xcbO\xb0\x90\x07V\xbb\xaa\xb6h\
\x10Pig\xc2\xd2\x86\x85<b\x7f\x83j\x8b\x90\x97\
\xa2\xdfFX\xda\xb0\x90MA\x9a\xa9\xb6\xa8\xa7\xd8\x83\
j\x84%\x10d\x1b\xa3z\xaa-\x8a\xe5\xfeX~\x82\
\x85<\xadWE\xb5E\xc8\xa3\xb5\x09\x84\xa5\x0d\x0by\
\xf6\xa5\xa4j\x8b\x90\xc7>\x9e#,mX\xc8\xf2\xb9\
@\xa6f\x8b.\x03*\x9dDX\xda\xb0f\x03\xd3e\
\xa5j\xb6\xa8,P\xe8k\x84\xa5\x0d\x0bz\x81\xc0\x1e\
\xcd\x16\x15\x01\x0a}\x97\xb0\xb4aA\xaf<\xd9\xa8\xd8\
\xa1_\x91B\x17\x10\x966,\xe8%M\xdf)v\xe8\
\x10R\xe8R\xc2\xd2\x86\xb5\x03\x99\xaf%\x8a\x1d\xda\x86\
\x14\xba\x81\xb0\xb4aA/\xd5\x9a\xab\xd8\xa15H\xa1\
\xfb\x08K\x1bV\x109'\x9e\xac\xd8\xa1\xcf\x10X\xe9\
\x84\xa5\x0d\x0bz\xf7\x91\xe6\x83\x853\x80:Ke\x13\
\x96:\xacF>Y\x9b\x8c\xdc\x22\xa8IX\xfa\xb0n\
\x04&\xac\xa3b\x87\x90\x05\xa4\xcd\x08K\x1fVo`\
\xc2\xaeR\xecP\x1f\xb9\xff\x00\x84%\x91x\xb1\xfd\xcc\
\xdc\xcd\x0d@\x9dw\x10\x96>\xac\xd1\xc0\x84Ed\xe8\
u\x08\xd9d\x22\x9e\xb0\xf4a%!?\xe3\x15\xdf6\
\x1e\x0d\x94\xf9<a\xe9\xc3\xfa\x04\x81\xb5F\xad\xcc\xf4\
\x80\xd8=h\xc2\x12\xc9\xf7\x08\xac\xc5je\xeeF\xca\
\xfc;a\xe9\xc3:\x86\xcc\xd8\xabje~\x87\x94\xb9\
\x95\xb0\xf4ae_\xec\x8b\x1d\xb2\x90\x05\x89\x81S\x84\
e\x00\xac\xcb\x81)\xeb\xa2Ve\x22Pe\xc5l\xc2\
2\x00\x16r\xe9\xfdr\xb5*\xef\x14\xdbk\x86\xb0\x84\
\x82<a\x5c\xf4\x8cV\x95\xc8\xc3_\xdd\x08\xcb\x04X\
#\x05O\x8b\xddKy\xa0\xc8\xc1\x84e\x02\xacw\xfd\
p\xbd\xe1g\xa4\xc8\xb1\x84e\x02\xac\xe5\xc8\x9cMQ\
*r%R\xe4|\xc22\x01\xd6Qd\xce\x86(\x15\
\x09=\xfe\xb8\x81\xb0L\x80\x95]\x0e\x98\xb3?*\xd5\
\xf8\x1cr\xab\xfc\x14a\x19\x01\xab\x85\xe0%\x22\xb7\xd2\
Wn\xfd(a\x990i\xd6N\x9d\x1a\xebJ\xaes\
%,\xfd\xaf\x19\xeb}\x95\x12\x7fA\xd66\x0c#,\
3`}\x80\xc0z\x5c\xa5\xc4/E\xef\x94\x13\x96L\
\xb6\x22\xb3\xa6\xb3\x0f)\xf2\x88\x8e\x95LXf\xc0\xca\
B\xd67\x94\xce\xd2(1\x0e\x81\xb5\x97\xb0\xcc\x80\x95\
\xdd\x1c\x99\xb6M\x1a\x15\xd6\x00\x0a\xac\x90MX\x86\xc0\
\xfa\x13\x02k\xa6B\x81G\x90\x02;\x10\x96)\xb0^\
2\xfd\xda;\xb4o\xc3\x93\x84e\x0a\xace\xc8\xbc5\
R(0Q\xf6\x82\x08a\x09\xe5$\xb2\xe3L\xe0\x80\
\xf7\x05^\x87\xc0\xdaAX\xa6\xc0\xcan\x8cL\xdc\x1c\
\xcf\xcbK\x8b\x02\xca\x8b\x0e\x12\x961\xb0\xeeC`\xdd\
\xebyy\x9f#\xe5\xc5f\x13\x961\xb0\xa6#3w\
i\xd0\xeb\xf2\x86\x0b\xdf\x1a ,\xa9@;\xdcZ+\
\xbd.\xaf\xbe\xe4*?\x08\xd6<\xc2\x82\x92Y\x12\x99\
\xba\xa7=\xae\x0e\xba\xe5\x148,\x09k6aaA\
^\xe0\x0b\xbe\x22\xde\xe3\xcbl\x0d\xb3%a\xcd ,\
,c\xa1\xef\xc2\x1f\x0c\xbe\xd80D\x14\xd6\xeb\x84\x85\
\x05z\xa0\xc2z\xc6\xd3\xdavG\x08\x9fbe\xcf\x0d\
\xfb\xaf\xbfBXX2J!\x93W\xdb\xd3\xdf\x85\xe3\
\xa1S\xacC6F\x08\x7fA\xda\x14\xc2\x02\xd3\x11\xfa\
\xc8\xfa\xd6\xcb\xd2\xae\x96>\xc5\x02^T5\x81\xb0\xc0\
@+\xe9\xac~\x1eV\xf6/K\xfa\x14\x0b\xb8\xc7\xfd\
\x02a\x81Y\x0bM_\xf1\xa3\xdeU6\x00\xaa\xecC\
;C$\x87\xfd\xe7G\x11\x16\x98`5h\xfe&y\
V\xd8q\xe8\xec\xafx\x9a\x9d1\xbe2\xf6\x12\x9e\xff\
ae\x0f\x84`U\xf7l\xd7\x99\xa9P]\xddl\x8d\
\xf1\xcf\xb0\xff\xfe\xa3\x84\x85\xe6ch\x02\xadw\xbc\xfa\
\xb9\xfa\x07\xa8\xac7m\x0d\xb2*\xec\xbf\x7f7a\xa1\
I+\x06\xcd`\x03\x8f\x9e\xa9\x80\xb6\xc2\xb1\x22\x0e\xd9\
\x1a$\xfc\xfd}\xbb\x12\x16\x9cN\xd8G\x967w\xcd\
2\x1b@E\xb5\xb67\xcaF\xe9\x01\x0a5\xac71\
X\xb5N{Q\xd4\xdbXQ\xe3\xed\x8d\xf2S\xd8\x03\
\xd4#,8)\x17a\x93\xe8\xc5K1OU\xc7j\
\xdal\xf3\xd0\x8d\xdf\x13\xc5\xcf\xb0\xb2\xbb`\x93X\xd6\
\x83\xd7\x9f\x8c\xc6Jje\xf7\x0b7\xec\x9d!\x22\xb3\
\x08K\xf8<\xd9\xb2\xe2\xc4+\xda\x82\xfd\xa0\xb0\xa6\xdb\
\x1d\xa8t\xd8C\xa4\x10\x16~)\xb28(Kz\x9d\
nV[\xac\x9e\x8bS\xed\x8e\x14\xfe7\xeeV\xc2\xc2\
\x13\x07\xc2\x8a\xde%[\xcf\xf3`=\xfdm\x8f\x14\xfe\
SJ_\x11\x16\x9e\xcf\xc0\x89\xb4Z\x88\xfe2\x5cV\
\x04,\xc7\xfe\xa4\x87\xbf\x94\xf0m\xc2\xb2\xf1\xd5S\x03\
\x95u\xbf`5\xbb\xab\x80\xc5\xd4\xb1\xbfJ\xac\xab\xa1\
k\x1c\x0b\x08,\xf4G\xd8\xff\xe69\xb1Z\x8e]\x89\
\xd62\xd6\xfe`w\x85=\xc8\x1d\x84e#{\xd1/\
\x1f+0]\xa8\x94\xb4k\xd1RJ9\xf8\xc1\xf6\xb0\
\xb1\x0f\x92\x14\x0cX\xd9\xdd\xe0\x8f\xac\xc04\x91BR\
\xdb\xc2\x95\x0cw0\xdc\xa4\xb0G)\x9aNX6\x92\
l\xe1\x19+P\xc7\xd1\xe6p\x19Q{\x1d\x8c7/\
\xfcqV\x13\x96\x9d4\xb5!\xeb\x8e_\xdd\xaebs\
}\xbc\x0aG\xbf#\x80\x87\x94&\x13\x96\x9d\xcc\xb6\x01\
\xcbj\xbd\xcf\xdd\x22\xe6\x97\xc4k\x88\xd8\xe2\xe8\xe42\
\xfc\x81\xba\x10\x96\x9d\x80\xeb\xea~\xbbR:\xd7\xc5\x12\
N\x0c\x08\xd8(\xe1VGcf\x16\x0d\xff7\xc2)\
\xc2\xb2\x93\x17-[\xe9{\xc8\xad\x02\xbe\xaaeg\xfc\
\xc8\xf5\xceF\x05\xfe?\xcd%,;9Y\xd9\x9e\xac\
2/\xb9\xb2\x0a~w\xef\x80\xad\xe1\x07:\x1c\x17\xb8\
\xb8\xd1\x81\xb0le\xaae3\xf5\xdfs\xbc\xa4\xe4\xc8\
S%\xec\x8d]\xda\xe9\x07&\xf2:\xa1e\x84e'\
\xe95,\xdb\xb4f:\xbaw\xb8\x7fxI\xbb#\x8f\
sz\xd4c\x80\xc1\x9a\xfcJXv\xf2\x96e?\x95\
\x12\xb6\xdb\x1c5kI\x5cQ\xdb\xc3\xd6t|B\x0d\
=\xa3\xd4%\x93\xb0\xec\xfcB\xba\xc2\x81,+\xe2\xfa\
W\xf0]\x95\x83+\x1f\xab\xe1dP\xe7\xfb\xec\xed\x80\
\xc6\xeb\xfe3a\xd9\xc8\xe7\x96\xb3D\xb6\x1d\xbd\x1c\xf8\
?}l\xc1\xa0\x1a\xce\x06\xbc\xc9\xf9\xe67\xc1\xd2\xd0\
\x88\xb5?!,\x1b\xe9i9N\xd9N\xcf.:\x18\
\xfa\xc3q\xfd[\x83\x9aF:\x1d\xab\xccn\x17\x8e\xb9\
58h\xdb\xf9\xa7\x08\x0b\xcd\xce\x12\x96+)\xdf\xe6\
\xde?\xcf\xf9\xfa\xa7\x5cn\xdb\x1e\xdd\xf0\xe9\x1bO\xf4\
l\x10\xe5\xca0o\xb9q\xcc\x03q\xcf}^^\x9d\
\xf3\xd02R\x0el\xdf\xb0z\xe9\x92\xf7\xff\x9a4m\
\x5cb\xc2\x90\x01\xbd\xe3b\x7f\x22\xac\xfff\xb4\xe5j\
J]\xd6\xf0\xda\xd8\x1eqqw\xc6\xc5\xc5\xdd\x1c\xdb\
\xacn\xa5(7\xff\xba;\xf7X\xa6\xd9\x1a;P\xa5\
e\xb7\xb8?\x0d\xb87\xae{l\x9b\x98z\xb5\xca\x97\
\xcd\xf5_}KX\xbf]rh`\xf9&\xd1\xee<\
\x85\xb6B\xb0\xc4\xc5\x84\xf5[V\x17\xf5\x0d\xac\xf7\xdc\
9\xe2\x8c2r%\xce!\xac\xb3\x19\xe5\x17W\x0f\xba\
u\xc4\x9d\xe5jL\x22\xac\xdf\xff\x03\xb7\xf0\x87\xab6\
\xae='4Q\xae\xc8\x09\x84\xf5{6\x95\xf4\x83\xab\
\xcb\x5c[T\x91\xbdF\xae\xca\xa7\x09\xeb\x9c,\x0c\x98\
\xef\xaa\x98\x8bo\xf5\xc9*/Vf<a\x9d\x9b\x04\
\xf3a\xcdr\xf3xo\x11+\xb3?a\x9dwY\xfc\
&\xd3]\x8dq\xf5xg\x88\xd5\xd9\x83\xb0\xce\xcb\xb1\
\xfaf\xbbz\xd2\xdd\xc3E\xf7\x07\x0b?7\x10\xd6\xf9\
\xd9}\x99\xc9\xae\x5c?s\xe9$Ui\x0ca]\x90\
\xad\x97\x98\xeb\xaa\xbf\xeb[\xa0\xcd\x91*\xb5\x0ea\xe5\
\xb8\x02_\xdaTW}\xdd_lw*Z\xa8\xd6\x8a\
\x84\x95#\xcb\xa3\xcdt5Xb\x11g\xbcP\xb1Q\
\x84\x953\x1b\xab\x1a\xc8*\x90(r\xac?E\x0a\xd5\
\x9bNX9\xb3\xa3\x8eq\xae\xa2\xa4\xf6\x98\xef%T\
\xf0!\xc2\xca%\xfbcL[(\xf3\x0f\xb1SJ\xa1\
\xbb\x0d[\x08+\xd7\x93\xda~F\xb9j\xb4Y\xeeP\
o\x93)y\x15a\xe5\x9e)E\xccq5P\xf2\xa9\
\xbe-2\x17I\xbf$\xac<\xb2\xb4\x8a!\xac\xca\x08\
\xef\x9c0B\xa4\xea\x05\x84\x95W\x8e\xden\x84\xab\xe6\
\xdb\x85\x8f\xf3T]\x89\xb2\xdf&\xac|\xaeK\x1bp\
E\xab\xc8.\xf1\xc3\x5c)\xf1e8\x99\xb0\xf2\xc9\xbe\
\x1e\xfa\xb2z\xcb\x1f\xe6K\x02e\x8f\x22\xac|\x93\xdc\
P]\x96\x07O ?\xe8~\xd5\xc3\x09+\xff\x9c\x99\
\xa8}\xef\xb0\xb6\xfcN/\x19]\x5c\xaf\xfa~\xc2\x0a\
\x95\x83\x0f\x15\xd7\x955B\xfe\x18\xd3\xbb\xbb]\xf4m\
\x84\x15:\x07ti]\xb4\xd1\x83\x0f\xe6\xbb].\xba\
\x03a\x85E\xeb\x912\x8a\xb2\xae\x0bzp\x88\x13\xdd\
\xbd\x1d}\x0da\x85\x97\x13I\x8d\xf4d\xbd\xe5\xc5\x11\
.w\xf5\xd6\xfb\x95\x84\x15\xfe/\xc4^\x17)\xc1*\
\x7f\xc4\x8b\xe3K\x1d\x18\xe1^\xc9U\x09\x0bH\xca\xf4\
\xeb#Td\xf5\xf7\xe6\xf8V\xb6t\xad\xe2R\x84\x85\
e\xcf\x84\x96\x0a\xb6\x02K\xbd9\xba\xe0GM\xdc\xaa\
8\x93\xb0\xd0\x1c~\xb7\x8f\xe7\xf7z\xea{\xb5\x223\
\xf8Y'WVh\x95J%,\x1b\xc9\xfcv\xcc\xcd\
\xde\xfeN|\xce\xbb\x83\xdb\xfalM'\x95F\xd4\xb9\
u\xd4\xc2\xed\xc1l\xc2\xb2\x99\xacu\xd3\xfa\xd6\x8f\xf4\
\x0aV\xb1\x1f=<\xb4\xe0\xf2'm\xdd\xca*{\xdd\
\x90\xe9+N\xaa\xcf\x8c\xcfa\xfd\x7f\xd2VN\x1f\xd2\
\xb6\x22:\x01\x17\xc7\xe2\xf3\x16\xeb\xf1\x91\xed\x9du\x7f\
\xe3\xb0\xf7\xa2\xbb\xa8^\xb7\xe1IK\xf7\x192)\x05\
\x01\xd6\x7fr|\xed\x82??\xd0\xb1\xd1%a\x9c\x9d\
Tn\xd3o\xd2\x8a\x8c\xec\x0d\xf8\xfe\x81\xb3\xbc?\xae\
3\xeb\xde}\xbcC\xb5|~\xacDT\x89\xe9:x\
\xf2\xe2m\x19FMG\xc1\x81u\xf6\xd4k\xff\xdaO\
\xdf\x9966a\xd0\x9d\xddcccbb\xae\xaaU\
\xabV\x83\x98\x98\x16\xb1\xb1]\xee\x8a\x1f9\xf9\x9d\x8f\
\xd7\x9d\xfd\x9e\x18\x0e\xc3\xba$E\xe9\xa82\xf6\xad^\
\xf4\xfa\xa8\x07o\x8f\xeb\x14\xdb&\xa6Q\xad+c\xda\
w\x8a\xbb\xe7\x81\x11S\x17.\xdf\x9ba\xe44\x14<\
X\xc8w(\xfe\x96\xc4\x81\xd9\x0ca\x85\xcc\xfb\xf8\xcf\
\xadoh\x86\xb0B\x07\xdfZ\xb6\xe1\x19\xa2!\xac\x90\
\xd9V\x0c\x965\x9eh\x08+t\x12aX%\xb6S\
\x0da\x85Lz=XVG\xaa!\xac\xd0\xb1\xf12\
\xbb\xf9dCX\xa1s+\x0c\xab\xca/tCX!\
\xb3\x07\x7feA<\xdd\x10V\xe8L\xc0/f-g\
\xd7\x08+d2\xf0uu\x8d3\xd86\xc2\x0a\x99\xaf\
\xf1Uu\x93\xd95\xc2\x0a\x9d\xfe\xf8\xda\xcc=\xec\x1a\
a\x85\xccQ\xfcEI=\xd95\xc2\x0a\x9dW\xf1\x8b\
Y\x1f\xb1k\x84\x152Y\xad`X\xd5O\xb0m\x84\
\x152\xab\xf1u\xf3\xc3\xd95\xc2\x0a\x9d\xc1\xf86\x7f\
k\xd85\xc2\x0a\x99T|\x1b\xdd\xe6Yl\x1ba\x85\
\xccL\xfc\xfc\xfd/\xec\x1aa\x85N{\x18V\xe9}\
\xec\x1aa\x85\xccF\xfca\xb0\xde\xecZ^YEX\
g\xf38\xfee\xf8)\xbb\x96{\xd6D\x13\xd6\xd9\xa4\
\xd5\x80a\xd5\xf9\x95m\xcb\xc3\x15a\xfd\x9e\x05F\xee\
y\xebSW\x84uN\xf0\x0d\xb1\xbd\xd8\xf3\xd6\x9f\xae\
\x08\xeb\x9c\xec,\x01\xcbj\x1bd\xdbruEX\xe7\
f\x94\xa1{\xde\xfa\xd0\x15a\x9d\x9b\xd3\xf8\xc3`\xd1\
\x87\xd9\xb6\xf3\x5c\xfd\xb6\x04\x89\xad87_\xe0\x1fY\
w\xb3k\xb9\xb9\x22\xac\xf3\x83\xbfD7\x90\xcc\xae\x9d\
\xcd\xda\xdf\x97L\xb2\x19\xe7\xe5\x00\xbe\xab\xe9\xe5\xa7\xd8\
\xb6\x9c\xae\x08\xeb\x82L2z\xcf[\xff\xb8\x22\xac\x0b\
b\xe3a\xb0\xa8\xcdl[\x0eW\x84ua\xbe\xc1\x1f\
\x06k\xc7\x8bY9\x5c\x11V\x8e\xdc\x03\xc3j}\x90\
]\xcb\xfe\xd7\x05/t`G.\xcc\xd1\x0a\x18\xabk\
\x17\xb1g9>\xaf\x08+\x97\xbc\x86l\xe3\xd0u\x15\
\x1b\x96\x9b+\xc2\xca\x99\xack\xc2>m\xef\xf7#\xdb\
\x95\xbb+\xc2\xca%\xeb\x8a\x84\xb74y\xd8~\xf6*\
/W\x84\x95[\x86\x86\xc1\xaar\x22\xf7_\xcb\xc7\x15\
a\xe5\x96\xd4KC\xae\x1d\x9d\xc2\xeb\xed\xf9\xba\x22\xac\
\x5c3+\x7fV133\xd9\xa3\xfc]\x11V\xeei\
\xcf\xeb\x0b\xce\x5c\x11V\xee\xd9\x12\xc5\xeb\x0b\x8e\x5c\x11\
V\x1ey\x82\xd7\x17\x1c\xb9\x22\xac<\x92V\x93\xd7\x17\
B\xe7\xfb\xbc\xb7\xaccs\xf2\xc8B^_\x08\xed*\
\x9f\xbb_\xecN^\xe9z\xc1\xf5\x85t\xb6\x04pE\
Xyf\xd7\xc5\xbc\xbe`\xdf\x15a\xe5\x9d\xd1\xbc\xbe\
`\xdf\x15a\xe5\x9d\xd3\xf5\xffs}a5[\x81\xbb\
\x22\xac|\xb2\x84\xd7\x17l\xbb\x22\xac\xfc2h\xe4\x11\
6\xc1\x9e+\xc2bD\x5c\x11\x16#\xe2\x8a\xb0\x18\x11\
W\x84\xc5\xa0Y\x17\xd6\xd3&\xec\x13#\xe1\x8a\xb0\x18\
\x11W\x84\xc5\x88\xb8\x22,F\xc4\x15a1\x22\xae\x08\
\x8b\x09??T\xb6\x08\x8bQuEX\x8c\x88+\xc2\
bD\x5c\x11\x16#\xe2\x8a\xb0\x18\x11W\x84\xc5\x84\x93\
M\xa8+\xc2bD\x5c\x11\x16#\xe2\x8a\xb0\x18\x11W\
\x84\xc5\x88\xb8\x22,F\xc4\x15a1\x22\xae\x08\x8b\x11\
qEXL\xbe\xae\xaaX\x84\xc5\x18\xe4\x8a\xb0\x18\x11\
W\x84\xc5\x88\xb8\x22,F\xc4\x15a1\x22\xae\x08\x8b\
\x11qEXL\xae\xd9\xec\xd0\x15a1\x22\xae\x08\x8b\
\x11qEX\x8c\x88+\xc2bD\x5c\x11\x16#\xe2\xea\
\xdfK\xb1\x10m(`\xee\x87\x00\x00\x00\x00IEN\
D\xaeB`\x82\
\x00\x00\x01\xdc\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00O\x00\x00\x00O\x04\x03\x00\x00\x00X\x1a\x85\xb5\
\x00\x00\x000PLTE\x00\x00\x00\xff\xff\xffA\xcd\
RP\xcaNA\xcdRA\xcdRF\xccPH\xcbP\
B\xccQI\xcbPE\xccQJ\xcbOE\xccQD\
\xccQC\xccQA\xcdR\x000]\xbe\x00\x00\x00\x0f\
tRNS\x00\x00\x186Oc\x9d\xab\xae\xb8\xc9\xcd\
\xd6\xe9\xf0\xbb\x00_o\x00\x00\x01LIDATH\
\xc7\xc5\xd6\xbdM\xc4@\x10\x86\xe1+\x01:\x98\x80\x8c\
\x0aH\x11\xf9\x227@\x09HT\x80\xdc\x05\x11\x92K\
\xa1\x01$\xebJ\xa0\x00\x12K\xd6\xc9\xd6\x10\x9c\x7f\xd6\
\xbb\xb3;o\xc6\xc4\x8f>\xe9\xee\xbe\x9d\x9b\xd3)\x9b\
\xd0\x9f\xd8\x84\xf9\x11B\x1d\x04Bm(d\x91Aa\
dP\xd5Q \xd4\x96B\x12\x19\x14F^\xe1E \
\xd4\x8eB?r\x81\xfaE\xe1,\x10jO\xa1W\xb7\
\x0dz\xdd\xd8\xa1\xd3\x8d\x08\xd6##X\x8f\x8ca\xd4\
\x8d\xbb\xa7t^\x22\x18u\xe3U\xab\xb3G~\xd6\xe1\
\x1e\xe9\xc1\xad\x1b\x1e\xdc\xea\xe6\xc25\xd2\x85k\xdd|\
\xb8\xd4\xcd\x87K\xdd\x00\xbcF\x02\xa8?\x02\xa16\x14\
\x0e\x02\xa16\x14\x8e\x02\xa1\xbeS\xf8\xfb\x7f\xf0\x19\xc2\
3\xfc\xd43\xfd\xc2{\xf8\x13N\xb4\x14\x1d\xac\xd9\x85\
\xf6\xb1\x83\x0d\x1f\xe9\x9bi\xe1+\x1c\xe9\xbbn\xe1\xa6\
\x18\xe8\xee\xd9v\xee\xc3G:\xdff\xa0\xb3\x9a\xabK\
<\x1c\xeb\xb5\xcem:7!\xdbO>\x8c\xff\x0dk\
p\x12\x08\x0fG@\x05\x1eo\x80\x0a<^\x15e\x98\
\x9c>e\x98\x5c>E\x98\xdeRE\x98\x9eR%\x98\
\xb5\xa1\x04\xb36\x14`^\xaf\x02\xcc\xebe\xc3\xb30\
h][&\xb4\x8e-\x0bN\x02\xa1yc\x1a\xd0>\
1sx\xfff\xb9?\x9e\xd1\xe7wR\xc2\xd3\xb8\x00\
\x00\x00\x00IEND\xaeB`\x82\
\x00\x00\x13H\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x01\x90\x00\x00\x01\x1e\x08\x03\x00\x00\x00\x93\xdb\xb3V\
\x00\x00\x00\x09pHYs\x00\x00\x1a]\x00\x00\x1a]\
\x01\x87\x7f\xcb\xb2\x00\x00\x00\x19tEXtSof\
tware\x00www.inksca\
pe.org\x9b\xee<\x1a\x00\x00\x02\xfdPL\
TE\x00\x00\x00\x00\xff\xff\x00\xff\x80U\xff\xaa@\xbf\
\x803\xcc\x99+\xd5\x80$\xdb\x92 \xdf\x809\xe3\x8e\
3\xe6\x80.\xe8\x8b+\xd5\x80'\xd8\x89$\xdb\x803\
\xdd\x880\xdf\x80-\xe1\x87+\xe3\x80(\xe4\x86&\xd9\
\x801\xdb\x86.\xdc\x80,\xde\x85+\xdf\x8a)\xe0\x85\
'\xe2\x89/\xe3\x84.\xdb\x89,\xdc\x84+\xdd\x88)\
\xde\x840\xdf\x87.\xe0\x83-\xe1\x87,\xdb\x83+\xdc\
\x87)\xdd\x83/\xdd\x86.\xde\x83-\xdf\x86,\xe0\x83\
+\xe1\x86*\xdb\x82.\xdc\x85-\xdd\x82,\xde\x85+\
\xde\x88+\xdf\x85*\xe0\x87.\xe0\x85-\xdc\x87,\xdd\
\x84+\xdd\x87+\xde\x84*\xdf\x86.\xdf\x84-\xe0\x86\
,\xdc\x84+\xdc\x86+\xdd\x84.\xde\x86-\xde\x84-\
\xdf\x86,\xdf\x83+\xe0\x85+\xdc\x83.\xdd\x85-\xdd\
\x83,\xde\x85,\xde\x87+\xdf\x85+\xdf\x87-\xe0\x85\
-\xdd\x86,\xdd\x85,\xdd\x86+\xde\x84+\xde\x86-\
\xdf\x84-\xdf\x86,\xe0\x84,\xdd\x86+\xdd\x84+\xde\
\x86-\xde\x84,\xde\x85,\xdf\x84+\xdf\x85+\xdd\x84\
-\xdd\x85-\xdd\x84,\xde\x85,\xde\x86+\xde\x85+\
\xdf\x86-\xdd\x86,\xdd\x85,\xde\x86+\xde\x85+\xde\
\x86-\xdf\x85-\xdf\x86,\xdf\x84,\xdd\x86+\xdd\x84\
+\xde\x85-\xde\x84,\xde\x85,\xdf\x84,\xdf\x85+\
\xdf\x84+\xdd\x85-\xdd\x84,\xde\x85,\xde\x84,\xde\
\x85+\xdf\x86-\xdf\x85-\xdd\x86,\xdd\x85,\xde\x86\
,\xde\x85+\xde\x86-\xde\x85-\xdf\x86,\xdf\x85,\
\xdd\x85+\xdd\x84+\xde\x85-\xde\x84,\xde\x85,\xde\
\x84,\xdf\x85+\xdf\x84+\xdd\x85-\xdd\x84,\xde\x85\
,\xde\x84,\xde\x85+\xde\x86+\xdf\x85-\xdd\x86,\
\xdd\x85,\xde\x86,\xde\x85+\xde\x86-\xde\x85,\xde\
\x85,\xdf\x85,\xdd\x85,\xdd\x85+\xde\x85-\xde\x84\
,\xde\x85,\xde\x84,\xdf\x85,\xdf\x84+\xdd\x85-\
\xde\x84,\xde\x85,\xde\x84,\xde\x85,\xde\x86+\xdf\
\x85-\xdf\x86,\xdd\x85,\xde\x86,\xde\x85,\xde\x86\
-\xde\x85,\xde\x85,\xdf\x85,\xdd\x85,\xdd\x85+\
\xde\x85-\xde\x85,\xde\x85,\xde\x84,\xde\x85,\xdf\
\x84+\xdd\x85-\xde\x84,\xde\x85,\xde\x84,\xde\x85\
,\xde\x86+\xde\x85-\xdf\x86,\xdd\x85,\xde\x86,\
\xde\x85,\xde\x85+\xde\x85,\xde\x85,\xde\x85,\xdf\
\x85,\xde\x85,\xde\x85-\xde\x85,\xde\x85,\xde\x85\
,\xde\x85,\xdf\x85,\xdd\x85-\xde\x84,\xde\x85,\
\xde\x84,\xde\x85,\xde\x86,\xde\x85-\xdf\x86,\xdd\
\x85,\xde\x85,\xde\x85,\xde\x85+\xde\x85,\xde\x85\
,\xde\x85,\xdf\x85,\xde\x85,\xde\x85+\xde\x85,\
\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xdd\x85-\xde\
\x85,\xde\x85,\xde\x84,\xde\x85,\xde\x86,\xde\x85\
,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\
\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\
\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85\
,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85\xbf\
\x5c\xdf:\x00\x00\x00\xfetRNS\x00\x01\x02\x03\x04\
\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10\x11\x12\x13\x14\
\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f !\x22#$\
%&'()*+,-./01234\
56789:;<=>?@ABCD\
EFGHIJKLMNOPQRST\
UVWXYZ[\x5c]^_abcde\
fghijklmnopqrstu\
vwxyz{|}~\x7f\x80\x81\x82\x83\x84\x85\
\x86\x87\x88\x89\x8a\x8b\x8c\x8d\x8e\x8f\x90\x91\x92\x93\x94\x95\
\x96\x97\x98\x99\x9a\x9b\x9c\x9d\x9e\x9f\xa0\xa1\xa2\xa3\xa4\xa5\
\xa6\xa7\xa8\xa9\xaa\xab\xac\xad\xae\xaf\xb0\xb1\xb2\xb3\xb4\xb5\
\xb6\xb7\xb8\xb9\xba\xbb\xbc\xbd\xbe\xbf\xc0\xc1\xc2\xc3\xc4\xc5\
\xc6\xc7\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf\xd0\xd1\xd2\xd3\xd4\xd5\
\xd6\xd7\xd8\xd9\xda\xdb\xdc\xdd\xde\xdf\xe0\xe1\xe2\xe3\xe4\xe5\
\xe6\xe7\xe8\xe9\xea\xeb\xec\xed\xee\xef\xf0\xf1\xf2\xf3\xf4\xf5\
\xf6\xf7\xf8\xf9\xfa\xfb\xfc\xfd\xfeV\xf3|\x8f\x00\x00\x0e\
\xc2IDATx\xda\xed\xddy|NW\x1a\x07\xf0\
\x9b}A\x88\x18\xc5\x08\x95\xc6\xbe7(e*\x94Z\
\x83\xb6t\xaa\x22cmP\x8dA\xd1N-E\x8d)\
UK-U\xc5\xd8F\xa2C\xec%j,\xa1*v\
\xad6\xda\xd4:\x12%!MR\x917y\xcfg\x92\
\x98\x96\x90<\xe7\xde{\xce\xbd\xce\xb9\xf7\xf9\xfd\x9d\xf7\
<\xf7\x9co\xde\xf7n\xe7\x9e\xab(\x1a\xe2\xfb%\xc1\
\x18\x1c\xf4\x90\x17\xc4w\x0f\x0e\x97H \xe8!\x16\x08\
z\x88\x05\x82\x1eb\x81\xa0\x87X \xe8!\x16\x08z\
\x88\x05\xe2\x83\x1eB\x81\xf8\xc4\xe10\x89\x04\x82\x1eb\
\x81\xa0\x87X \xe8!\x16\x08z\x88\x05\x82\x1eb\x81\
\xa0\x87X \xe8!\x16\x08z\x88\x05\x82\x1eb\x81x\
m\xc3\xc1\x11\x09\x04=\xc4\x02A\x0f\xb1@\xd0C,\
\x10\xf4\x10\x0b\x04=\xc4\x02A\x0f\xb1@\xd0C,\x10\
\xf4\x10\x0b\xc4k+\x8e\x89H \xe8!\x16\x08z\x88\
\x05\xe2\x89\x1eB\x81\xa0\x87X \xe8!\x16\x08z\x88\
\x05\x82\x1eb\x81\xa0\x87X \xe8!\x16\x08z\x88\x05\
\x82\x1eb\x81xn\xc1q\x10\x09\x04=\xc4\x02\xf1\xda\
!\xdcF\xa5\xa7\x15&\xdb\x96 ^\xdb\x85\xd8\x8e\xbc\
\x0b\xbbWL\x1f\xf9\xf2sM\x82\xfc]\x1f\x98\xacW\
\xa9V\xb3N\x11\xe3\xe7F\x1fM\xb3\x0d\xc8\xe3\xf7\xb8\
\x14;\xb9W]/\xda\x83*\x01-\x07\xce?\xf8\x8b\
\x0d@\x1e\xef\xf7\xe2\xe4\xdc^\x154,\xb7\xe2Zo\
\xd8\xbf\xae!\x88Q{\x8a\x98~\xe5\x15\x1d\xa9?\xe1\
p\x1e\x82\xf0\xce\x9d\xf5]=\x15\xdd\xa9\xf8F\x02\x82\
\xf0\xcc\xe9\xc8r\x0ac\xea\xcf\xbe\x85 |\xe2\x8c\x0b\
sQ8\xa4\xf4\xd0s\x08\xc2!\xb1\x0d\x14^q\xed\
\x9f\x84 \x8c\xd9\xd7R\xe1\x19\x8f\xc8\x14\x04aHJ\
\x84\x8b\xc29\xe5\xe6\xe5\x22\x88\xde\x9d\xc7\x92r\x8a\x01\
iq\x16At\xe5F\x0f\xc5\x98x\xcfs\x22\x88\xf6\
\xec\xad\xa2\x18\x96\xb0[\x08\xa25\x8b\xdc\x15\x03S\xfb\
\xbc\xae\x8d:\xe2_r&Y\x1a\xc41B16\xe5\
u-\xf2\x1c\x0f\xb4\xf8\x96\x95A\x1c\xfd\x14\xa3\xe3\xb9\
\x09AT\xe7\xee\x8b\x8a\xf1\xf1\xd8\x80 *\x93\xdbC\
1#\xee[\x10D]F(\xe6\xc4'\x1eA\xd4d\
\xaabV*\x9cG\x10z\xb6\xb9\x98\x06\xa24\xc8B\
\x10Z.\x05(&\xe65\x04\xa1$\xa7\x85bjV\
 \x08\x9c\xf7\xcc\xf5P\xca^F\x10(gt\xde8\
/S\xde\xdfK\xdf'\xbb\x22\x08t\x06\x12\xa2\xed\xde\
F\xcb\xf0)k\x8e$\xa5f\xde\xfbtZrb\xdc\
\xe21=\xebk\xbb\x0a\xb6\x06AJ\xceb\xf5\xe3X\
\xaa\xc3\xcc\xf8\x9c\xe2[\xc9\x8c\x1b\xdf\xc6CuC\x95\
\xd2\x11\xa4\xa4\xa4\xa9\x9d\x04Wv\xe0^\x07\xdcT\xea\
\xb2PW\x95\x8d\xbd\x8d %%J\xdd\x08\x06\x7f\x9c\
\xa1\xa6\xb5\xc4a\xbe\xea\xeeW]@\x90\x12NAT\
\xed\x97\xab/W}S\xfc\xda\x08U\xc7\x08\x83\x10\xa4\
\xf8\x0cVs\x8d6*SK\x93\xe7\x9fW\xd1\xa6[\
\x22\x82\x14\x97\x1fU\xec\x88\xeb\x9e\xd6\xd8\xa8s\x8e\x8a\
/I\x04\x82\x14\x97H\xfa\xc8\xf5\xcd\xd4\xde\xecQ\xfa\
\xbdy\x8f\xcb\x08\xf2hn\xf8P\x07\xee\x1d]\xb3E\
.\xd3\xa7>\x8eE\x10=\x17M\xa6\xe9l\xf9\xd6\xd3\
\xb4\x96\xfdn#\xc8\xc3\xb9[\x896j\x13u\xb7\xfd\
sMZ\xdb\xf3\x11\xe4\xe1l\xa0\x8dY\x1f\x86\xd9m\
\xdf\xfbS\x1ao\x88 \x0f\xa73\xed^\xd2\xaf,\xad\
o\xa6\xdd\xf5J@\x90\xa2\xb9H\xb9\xd0\xe1\xcd8!\
w\x18\x05$\x12A\x8af\xbaA;\xf4\xdf\x92Q\x95\
2q.\x07A\x8a\xa4!e\xea'\xf3\xa2\x00\xb4}\
\xd4v\x04y0\xdfQ\x86+\x9a\xbdD3\xb8\xc2_\
\x10\xe4\xc1L\xa6\x5c1\xe1\xf0\x5c\xf3\xbf)W\xf4\xb3\
\x11D\xfd/\xd6Z\x0e%\x9c\x94\x13\xf68\x04y\xe0\
\xea\x06|T\x1a\xec\xe0Qd\x0d\x0c2\x0aA\xee\x87\
r\xebv!\x97\x22\x8e\xca`\x91\x9a\x08r?a\xf0\
9\x08\xa7\xa5}\xc6\xc0\xec\xe7\x11\xe4\xf7\xff\xdd2\xe0\
H\xfd\x99S\x99\xb30\xc8b\x04\xf9-\x87\xe1\x91\xe2\
\xb6bZcVw\xbb\x80\xcc\x00\x07\xea\x09\x07\xaf:\
\xb3\xc1:\x15\x9d\x08\xf2\xfft\x02\x07j8\xbf\x0bf\
\xf07\xf1\xd1\xa5P\x16\x0d-\x1a\xe8I\xa2\xc6C\xe9\
q\xc8\x01\x92\x0b\xefB8\xae\xb9Y\x07,\xf4\xe9#\
\x7f\xdf\x8b\xf3L\xe2l9@N\xc3Ogf\xf0\xab\
\xf4&Xi(\x82\xdc\xcb\xa7`':p\xac\xb4\x0d\
\xac\xd4\x04A\xeee\x08\xd8\x89Y\x1c+e\x82S\xf1\
\xdc2\x11\xa40M\xc0N\x9c\xe6Y*\x14,u\x08\
A\x0a\x92\x03Ne+\xcdu5\xa5\xf1\xe0x-A\
\x90\x82\x9c\x01\xfb\xd0\x96k-\xf86\xd5\x08\x04)\xc8\
Z\xb0\x0f|\xcf\xb7.\x80\xb5\x9eC\x90\x82L\x00\xfb\
\x10\xc3\xb7\xd8\x1f\xa0Z\xfe\x08R\x90\x9e`\x1f~\xe2\
[\x0c\x9emt\x1dA\xf2\xd3\x08\xeaB\x00\xe7b\x7f\
\x03\x07\xec\x08\x82\xe4\x07\xbcp\xd2\x9as\xb1\xd5\xe0\x80\
\xadG\x10Bn\x82]\x18\xc0\xb9\xda\x11\xb0\xdaL\x04\
!$\x01\xec\xc2\x0c\xce\xd5\xd24\xcd_\xb4%H\x0c\
\xd8\x85\xcfy\x97\x03_\xb0\xd0\x09A\x08\xf9\x00\xec\xc2\
\x19\xde\xe5\x9e\x81\xaa\xd5B\x10\xca4h\x97_y\x97\
\x0b\x87\xcay\xe5!\x08\xe9\x02\xf5\xa0\x0a\xf7r\xf0\x1c\
\xc9\xab\x08B\xeaB=\x08\xe1^n\x098b\xf1\x08\
B\xfc\xa0\x1et\xe7^.V\xcb1\x84\x1dAr\xc0\
Y\xa4C\xb8w\x00>\x11\xf9\x04AR\xc0\x1e\xf0_\
\xbf\x1b\x9ey\xf2w\x049\xc7:\x9bPc\xb2\xc1o\
\xe48\x04\x89\x07{\xb0\x91\x7f\x0f\xc03\xc3\xc1E\xff\
\xf6PL\xd1L\x03>\xda#\x86\x9e<\x09@\xb6\x80\
 \x87\xf9\xf7\xa0\x1eT\xef%\xfd\xff=V\x99\xb9\xb8\
\x02\x04\xf9\x96\x7f\x0fZA\xf5B\x11\x04\x9eo{\x85\
\x7f\x0f\xc0y\xab\x8d\x10\x04\xbec\x94\xce\xbf\x07\xbd\xa1\
zU\x11\x04\xbe\x94e\xc0^p T\xd0\x17A^\
\x85\xc6\xa7\x8c\x01=\x18\xc5p`j\x07\x10p\x8a\xc3\
\x1f\x0d\xe8\xc1D\x10\xe4\x17\xdb\x83t\x85\x86\xa7\xae\x01\
=\x80\xef\xbf\xdc\xb4=HGhx\x9a\x19\xd0\x03\xf8\
r\xef5\xdb\x83\xb4\x85\x86\xa7\x95\x01=\xf8\x0c\x04\xb9\
d{\x10\xf0<\xedO\x06\xf4\xe0\x9f \xc8\x0f\xb6\x07\
\x01\xd7\x84io@\x0f\xd62\x5c\x1a\xb0\x03\x088o\
\xf1\x05\x03z\x10\x0d\x82\x9c\xb4=\x08\xf8\x1cfW\x03\
z\x00\xaf\x0at\xd4\xf6 OA\xc3\xd3\xc3\x80\x1el\
\xd1rS\xdd\x86 \x81\xd0\xf0\xbcl@\x0fv\x80 \
{m\x0f\x02.\xd7\xfb\x8a\x01=\x88\x03A\xbe\xb0=\
\x08\xb8fR\x1f\x03z\xb0\x07\x04\xd9e{\x90j\x0c\
7\xf0te'\x08\xf2\xa5\xedA\x82\xcd\xde\xa9o\x05\
A\xf6\xd9\x1e\xa4\xae\xd9\x87\xbd\x1b\xf1(\x0bLc\xb3\
O\x0c\xe1\xc7\x1f\xbe\xc2K'f_:Y\x07\x82$\
\xd8\x1e\xe4YM\x0f\x8es\xc8*\xbct\x02&\xd4\xec\
\xcb\xef\xf0\xbc\xa3\xb3\xb6\x07y\x01\x1a\x9e\xa7\x0d\xe8\xc1\
R\x86\xc7\xe2\xed\x00\xd2\x0d\x1a\x9e\x9a\x06\xf4`\x0e\x08\
\xf2\xb3\xedA\xc0\xf9\xcc\x95\x0c\xe8\xc1{ H\x96\xed\
A^c\x98&\xa5+c\x19&\x82\xd9\x01\xe4\x0dp\
|r\xf9\xf7`(T\xb0\x14\xb1=\xc8$\xf0\x17\xe4\
\x16\xff\x1e\xf4\x85\xeaUD\x90\x8f\x18&\x81\xe8\x0a\xb8\
\xce|\x10\x82\xc0\x93@\xbe\xe1\xdf\x03p\xdeQs\x04\
\x81/\xbe\x1e\xe0\xdf\x83\xa6Z\xd6\xd6\xb0!\xc8!\x10\
$\x86\x7f\x0f\xc0;b\xe1\x08\x02\xbf\x0el\x1e\xf7\x0e\
\xe4\xb9C\xf5\xa2\x10\xe4:\x08\xf2\x0e\xf7\x0e\xc0\x8fa\
OE\x10\x07\xf8\x98\xf2@\xee\x1d8\x05\x82,D\x10\
xi\x8d.\xdc\xcb\xedb\xd9g\xd9\x02\x04|L\xb9\
)\xf7r+\x19n\x18\xda\x03\xa4\x1b\xc3\x99\xb3\x8e\xfc\
\x03\x04\xb9\x8c d\x84\xb9\xd7N^\x87\xaa\xb99\x10\
\x84\xccb\xb8\xc7\xad#\xcfC\xd5\x02\x09\x82P\xd6\xc7\
_\xc7\xbb\x5c5\x96[\xc6\xb6\x009\x06\x82L\xe1\x5c\
\xed\x8e+\xcb\xd4U[\x80\xc0\x0b)\xf7\xe3\x5c\xed[\
\xa6\xf3P{\xbc6\xcf\x8f\xe1\xf2\xab\xe6l\x06AV\
\x22\x08\xa1<\xd4\xe6\xcb\xf9\xd5\x7f3YNCl\x02\
\x02/\xdbv\x8ao1p\xe9\x19%\x0dA\x08m=\
\xa0e|\x8bUg:\x0d\xb5\x07\xc8z\x10\xe4u\xae\
\xb5n(L\x8f\xc5\xdb\x03\x04>\xf0\xe1\xbb\x942\xfc\
\x80\xe1H\x04)\x88\x03|\xdb\xa3\xe7\x1d\x9e\xb5\xa6\x81\
 \xcb\x11\xa40\xcd\xc1Q\xda\xcd\xb3Tg\xb6\x03\x08\
\x9b\x80\xc0\x97\x17\xc7\xf0<O\xf7\x85*y\xe7 H\
a\xe0\x99@\xf58V\x82\xefN\xb5 \x08R\x18x\
\x9e\x03\xcf\xc9r\xa3\xc1B\xc3\x99@F[\x07\xc4\x09\
.6\xfd\xe8\x1bj\xf5\xa7>Xh=\x13\xc8H\xeb\
\x80P^-\xc9\xef\xc9O\xf8\xc5\xab.\xc9L \x91\
\x16\x02\x81\xe7\xf7\xbaq[My*X\xa7\x0e\xbd\x81\
\xc3\xc0\xc7\x07Y\x08\xe4\x04\xbc\x13y\x9f\xd7O#\xb8\
\xf2\x902\x8c\xde\x02\xf4\x8a\xbf\x08\x0b\x81\xe4\x05\x80#\
U\xd3\xc9\xa7\xcc~\x85q\x17\x02N\xea\xeak!\x10\
\xf81*E9\xc8\xa7\xca\x00\xb0\x88{*\xbd\x85o\
\x80\xcf\xf7\xb6\x12\x08\xbc\x0e\x22\xa77Q]/\x05\x16\
i\xa7\xa2\x89D\x93\x97\x01yl i\xee\xb0\xc8\xd7\
<\x8a\xbc\x05\xd7\x98\xab\xa2\x89$\xe0\xf3\xcfX\x09\x84\
<\x07\x0fVW\xe3\xbf \xaa^\xdc~\x05\xf8|\xb0\
\xa5@\x16\xc0\x83\xf5\xc8\x8b\xce\xf9\x7fA\x1a\xa9i\xe3\
6\xd0\x80\xbf\xa5@R\xdc\xe0\xe1j\xc1\xfc8n\x92\
/\x5ca\x96\xaa\xc3A`\x12\x91k\xae\x95@\xe0\x09\
\x85\xf9\xf9\x88\xf5\x1c\x84R\xc0#EU3\xfe@\x13\
7,\x05\xb2\x9c\x02\xe2}\x82\xad\xfd\x0f)\xed\xf7R\
\xd7L\x0d\xa0\x89\xe3\x96\x02\xc9\xf4\xa3\x8cX0\xd3?\
\xe0A\x0fJ\xf3[\xd4\xb5\x03=1\xba\xc6R $\
\x922bJ+\x867y\x9f+Oi<P\xe5\xf4\
\xafv@\x1b\xefZ\x0b\xe48\x0dD\xe9\x90\xa9\xb7\xed\
\xc4\xaa\x0a\xa7]T_\xe1N\xd5\x0d\x03!m\xa8\x22\
mt\xfej\x9dx\x82\xd6r\x80Z\xebqlW\x8b\
\xa5\x02\xd9L\x05Qj\x9c\xd6\xd3\xf0:_j\xc3\xaa\
\xe7\xd8/`\xbb\x9f\x22\x15H^\x1d\xba\x88\xf7l\xcd\
\xef\xd1K\x1f@o\xb6\xf4M\xb5\xadm\x82\x9aYa\
-\x10\xea\x91\xef\xbd+FG\xb5\x9d~\xac\x0fT\xd1\
\xe84\xd5\xed\x81\x0f\xb3\x84Z\x0c\xc4Q[\x8d\x88\xeb\
\xab\x1a\xde\x8f\xbb\xebY5MVS\x7f\xfc\x06OE\
\xddk-\x10\xca\xbbo\xee\x93\xf4\xd8\xa1\xea\x87+\xeb\
\xb3f\xea\x1a\x5c\xaba\x1b\xc1\x95R\xaa\xff\xd7Z \
\xce\x10Ee\x02G\x1d\xa4Lj\xcb\x88\x0d\xf7S\xd9\
XK-7$\xc1ET\x95\xa7NY\x0a\x84\x1cv\
QT\xc7\xb7\xfd\xe4=%\x1c\xac&\x7f\x1e\xd5\xcc]\
uC^\x9a\xd6\xe4\x1aC\xb9$\x16\xf9\x9d\x95@\xc8\
\x00E[\xaa\xb4\x0e\x9f\xb8|s\xdc\x91\x93I\xd7.\
\xfep,>n\xe3\xc7c{\x87\x04hkc\xa6\xa6\
-\x5cIm\xaf\xc9\xc8\x95\xfb\xcf'\xa7\xa5&%\xe6\
oPl\xcc\xb2Of\xbe?\xfe\xafC\xfb\xf7\xe9\xd2\
\xa1y\x7f\xe9@\xaek\x1cL\x0ei\xa9\xed\xaa\xf9i\
\x96Z\xb5\xa5\x03Q\xbb_\xe7\x97\xd2\xdfk<[b\
\xf9\x97\xa9,\x1f\x08\xfc6o\xfeq\x89\xd6\xba\x81/\
1T\xf3\x95\x10$\xb5\x86\xa9 \x135o\xe0\x02\x96\
r9\xf2\x81\x90\x93\xbe&z\xf4\xd0|%\x06\x9c\x9a\
E\xcdM\x09A\xc8j\xf3<Ze\xe8\xd8\xbe:\x0c\
\x05\x93d\x04\xa1L\x89\xe6\x98\xc6\xa9fo\xdeI)\
Ah'_\xbcR;E\xd7\xd6]p\xd1_r\x9f\
\x9c \xce!fx4\xd5{\xfb\xa2\xb5\xfe\x9a\x9b\xe5\
\x04!\xce\xf1\xc6{tL\xd7\xbbu\x1b\xf5\x17]-\
)\x08!\x1f\xb8\x1a\xec1H\xff\xa26\xce\xc6\xba\xab\
.\x90\x16\x84l+g$\x87'\xd3\xbc\xbb\x0d\xba\xeb\
N\x97\x17\x84$60\xce#\x88m%\xc7\xbc\xb6z\
\x0b\x8f\x93\x18\x84\xdc\x19c\xd4\xcfV\xcf\xdb\x8c\x9b\xf6\
S\x19\x9d\x95#e\x06!d\x7fMc@\xd8_\x0a\
\xbeTg\xe5\xber\x83\x90\x9cy~\x86\x88le\xde\
2\x9d\x87\x81\xdd$\x07!\xe4\xea\x00w\x03@jd\
1\x1f\x98G\xea*\xdcFz\x10B\x92\x06z\xf0\x17\
\x99\xc0\xbcYy\x13\xf4\x9c\xb07\xb4\x00\x08!\x97&\
T\xe0\x0d\xe2q\x96}\xb3\xb6\xf8k\xaf[\xcd\x12 \
\xf9\x07\x5c+B9\x1fq\xb5\xe6\xf0\xe8\xfb\xc5\xde\x9a\
\xbf$e-\x02\x92\x9f+\xb3\x9b\xb9\xf0\x14Y\xcac\
\xa3\x0e\xb5\xd2X\xd5\xd5i\x19\x90\xfc$/\xef\xcd\xef\
\xf4\xbd\xfcu.\xdbt \xdc[}M\xb7\xda}\xb2\
\xad\x04R\xb0+=\xb3(<\x98\xcf\xafW8\xa7M\
J]\xd8\xbd\x94\x0a\xffvo.K\xc82bH\x1e\
/Ha2\x0e/\x1e\xde\xb9\x96'x\xabc\xd0p\
\xea\xf4\x86=\xdc6\xe8\xee\xdew\xc3\x9e,i\x1a^\
\xfd\x17\xc7-\xdb\x9fb\xdch\x08\x00r/\xb9\x97\x0e\
mZ8)\xb2_\xcf\xf6!\x0d\x83\x82\x82*V\x0d\
\x0a\xaa\x13\xd2&,\x22jFtBZ\xfew\xa9\x05\
M\xa4\x16\xd7\xe5N\xc9\xed\xf85\x1f\x8e\xee\xd7\xb1y\
H\xed\xa0\xca5\xea\x84\x84v\x8ax{\xfe\xa6\xaf\xae\
\xe6\x19=\x0e\xc2\x80\xd0r\xdc\x8d&2\x99X!\xd2\
\x80\xc0\xef\x05/\xbc\x06\x7f\x0eA\xccLz\x15\x9aH\
['\x82\x98\x19\xfa|\xa2U\x08bj\xda\xd3@\x02\
n \x88\x99I\xf4\xa2\x89\x0cF\x10SC\xbdk\xe1\
\xf2\x1f\x0413YO\xd2D\xea\xe7 \x88\x99\xa1O\
\xa1\x9a\x81 \xa6\xa6;\x0d\xc4\xe7G\x0413\x17\xa9\
\x17\xfe:!\x88\xa9\xa1\xcfU\x8fF\x103s\x97\xba\
<D\xa5[\x08bfvS\xbf\x22#\x10\xc4\xd4\xbc\
B\xbd\xaf\xfa\xb5\xc4\x1e\xdb\xe5\x03I.K\x13i$\
\xef\xc9\xc8No\xf9@\xa8\x0b\x92*\xca\x1cY=\xbe\
\xf0V$\x04q4\xa1>?~A^\x0f\x09AH\
<u\x02QWy=d\x04Q\xb1\xa6M\xac\xb4\x1e\
R\x82\xdc\xa4\xceE\x0d\xcc\x90\xd5CJ\x10\xb2\x84\xfa\
\x15\x19-\xab\x87\x9c y\xd49\x9fn'$\xf5\x90\
\x13\x84\x9cp\xb3\xd6\xf9\xfa\xce\xfb3X%=b\x1f\
Iy\x92f\xabT\xbd\xd9\xf5\xc0\x8cbIA\xa0I\
A\xaea\x09DZ\x0fYA\xc8\x9a\x928\xbc\x22\xce\
\x13\x89=\xa4\x05)aR\x90_\xd45\x22\xb5\x87\xbc\
 \xc5M\x0a\xaa4\xe5\xb6t\xfd\xd8\xe5\xa3X\x03\x84\
Lx\x98#x^6\x91\xdeCb\x90\xac\xa2\xcb9\
\x86\xac\xca%\x16\xf0\x90\x18\xa4\xc8\xbb&$;\xce-\
\xd9Cf\x10\x12\xf6\xfbq\xee1b\x15\x0f\xa9A.\
\x95\x92\xf38\x17\xf2\x90\x1a\x84LS\x94\xb2Q\xc9\xc4\
J\x1er\x83dw\x98\x9bI\xac\xe5!7\x88\xcc\xd9\
]\xbc\x07\x82\x08\xe6\x81 \x82y \x88`\x1e\x08\x22\
\x98\x07\x82\x08\xe6\x81 \x82y \x88`\x1e\x08bv\
\xf6S\x1e\x02\xc3\x11\x12\xcb\x03A\x04\xf3@\x10\xc1<\
\x10D0\x0f\x04\x11\xcc\x03A\x04\xf3@\x10\xf3<J\
+\x08\x22\x9f\x07\x82\x08\xe6\x81 \xe6\xe4\x80Z\x0f\x04\
\x11\xcc\x03A\x04\xf3@\x10\xc1<\x10D0\x0f\x04\x11\
\xcc\xe3\x7f\xb9\x0bd\xb5'\xfb\xb9\xcc\x00\x00\x00\x00I\
END\xaeB`\x82\
\x00\x00\x04\xbb\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x01\x00\x00\x00\x00\xeb\x04\x03\x00\x00\x00t\xa8U\xa0\
\x00\x00\x000PLTE\x00\x00\x00A\xcdRA\xcd\
RA\xcdRH\xcbPB\xccQI\xcbPD\xccQ\
E\xccQJ\xcbOE\xccQJ\xcbOD\xccQD\
\xccQC\xccQA\xcdR\xaa`?\xbc\x00\x00\x00\x0f\
tRNS\x00\x18Oc\xab\xae\xb8\xbd\xc9\xcd\xd6\xd7\
\xdf\xe9\xf0\x91\xee\xa9J\x00\x00\x04+IDATx\
\xda\xc5\xda\xc1m\x13A\x18\xc5\xf1\xddT`%Bp\
 R\x02'N\x08\x1ap\x0dT`q\xa1\x0f*\xa0\
\x0b*\xa0\x91T\xe0\x1e@\x96|q\x86\x83c\xc7\xbb\
;\x93\xcc|\xef\xcd{\xd3\xc0\xfe\xf4\x0e\xfb\x9f\x95v\
\x18z\x9c\xab\xc3j\xb0\x9e\xab\xf4\xe0\x06<\xde\x9b\x01\
\xe9\xdf\xca\x0cH_\xdc\x00\xeb\x04W)\xa5\xf4\xcd\x0d\
\xd8\xad\xcc\x00\xe7\x04G\x80q\x82# \xfdt\x03\xf6\
+3\xc07\xc1\x09`\x9b\xe0\x04\xb05\xe9\x0cpe\
\xf9\x0cpM\xf0\x0c0e\xf9\x19`j\xd2\x05\xc0\x93\
\xe5K\x80e\x82K\x80\xa5I\x13\x80\xa3I\x13\x80c\
\x82)\xc00\xc1\x14`h\xd2\x0c\xa0o\xd2\x0c\xa0\x9f\
`\x0e\x90O0\x07\xc8\x9b\xb4\x00\xa8\xb3\xbc\x00\xa8'\
X\x02\xc4Y^\x02\xc4M\xca\x00\xb4Y\xce\x01\xa4\x13\
\xe4\x00\xd2&e\x01\xca&e\x01\xca\x09\xf2\x00\xe1\x04\
y\x80\xb0I\x05\x80\xaeI\x05\x80n\x82\x12@6A\
\x09 kR\x11\xc0\xc8\xf2\xf5\x87\xd7\xcf\xc7\xd4q\x82\
\xcf\x099\x84,c\x00B\x93@\x00\x9ee\x14\x00O\
\x80\x02\xe0&\xc1\x00\xb4I0\x00\x9d\x00\x07\x80\x13\xe0\
\x00\xb0I\x04\x00\xd6$\x02\x00\x9b\x80\x01\x80&`\x00\
\xa0&Q\x00H\x96)\x00d\x02\x0e\x00\xc82\x07\x00\
4\x89\x04\x88g\x99\x05\x08O\xc0\x02\x84\x9bD\x03D\
\x9bD\x03D'\xe0\x01\x82\x13\xf0\x00\xc1&\x11\x01\xb1\
&\x11\x01\xb1\x09\x98\x80\xd0\x04L@\xa8IT@$\
\xcbT@d\x02.\xe0\xb1\xfa\xb9\xe3\xa7U\x0f@\xaa\
\x06\xbcIw=\x00\x87\xea\x016}\x00\x0f\xf5\x03t\
\x01T\xbf\x08\xc6M\x1f@\xf5\xab\xf0]\xea\x02\xa8\x1f\
\xe0W\x1f@\xd3\x00\x1d\x00\xbb\xa6\x01:\x00\xaa\xafD\
\xb7\xa9\x0b\xa0~\x80m\x1f@\xe3\x00t@\xf5\x87\xc1\
\xf5\xb6\x0f\xa0\xf6\xd3h\x5c\xa7.\x80\xea\x01nR\x17\
@\xf5\xe7\xf1y\x002\xa0\xa5B=\x00\x87\x96\x0a\xf5\
\x00\x04\x06\xa0\x02\xf6\x81\x01\xa8\x80\xb6\x0a\xf1\x01m\x19\
\xee\x00\x08\x0d@\x04\xecB\x03\x10\x01\xad\x15b\x03Z\
3L\x07\x04\x07\xa0\x01\x9a3\xcc\x064g\x98\x0ch\
\xcf0\x17\x10\xc80\x17\x10\xa9\x10\x13\x10\xc9\xf0\x0c \
\xfa\x7f`9\xc0\x09\x00\xfdA\xb1\x07\x06\xa0\x00b\x15\
\xe2\x01b\x19&\x02\xa0\x01\x08\x80\x1d4\x00\x01\x10\xad\
\x10\x0b\x10\xcd0\x0d\x00\x0e\x00\x03\xc2\x19f\x01\xc2\x19\
&\x01\xe2\x19\xe6\x00\x80\x0cs\x00H\x85\x9e\xce\xd7\x8a\
\x0c\xde\x97\x00H\x86[\xce\xdf\x12\x800\x00\x04\xd8\x8b\
\x06(\x02\xb0\x0a\xe1\x00,\xc3\x04\x80l\x80\x02`'\
\x1b\xa0\x00@+\x84\x02\xd0\x0c\xc3\x00\xe1\x00Y\x00\x9c\
a\x14\x00g\x18\x04\xe0\x19\xc6\x00\x84\x0cc\x00U\x85\
J\x00U\x86\x8b\x00\xf1\x00\x0b\xc0^<\xc0\x02\xa0\xab\
P\x1e\xa0\xcbp\x01 \x1f`\x06\xd8\xc9\x07\x98\x01\x94\
\x15\xca\x01\x94\x19\xce\x02\x0c\x03L\x00\xd2\x0c\xe7\x00\xd2\
\x0cg\x00\xda\x0c/\x01\xe2\x0c/\x01\xea\x0a\xcd\x01\xf5\
\xff\xe9m\xfa\x00\xea\x7f\xd3K=\x00\xe3\x9f\x95\x17\xd0\
r\xd6n\x00\xf35\x14\x020_\xc41\xc0\xb85\x03\
\x86\xf7n\x00\xef:\x12\x04\xf0.dQ\x00q\x82\x18\
`x\xeb\x06\x8c\x1b3\x80\x97\xc4(\x806A\x14@\
\xbb\x16\x85\x01\xac{Q\x18\xc0\x9a \x0e M\x10\x07\
\x90\xb2\x0c\x008YF\x00\x94,#\x00J\x96!\x00\
\xa3I\x10\x80\x91e\x0c@\x98\x00\x03\x10\xb2\x0c\x02\xf0\
&\x81\x00<\xcb(\x00\x9e\x00\x05\xc0M\x82\x01h\x93\
`\x00:\x01\x0exa\x82\xdf?^?\xdfa\xc0\x0b\
Y\xbe\x1b4\xe7\xd6\x0d(fY\x05(fY\x06(\
5I\x06(eY\x07(L\xa0\x03\x14\xb2,\x04\xe4\
\x9b$\x04\xe4\xb3\xac\x04d'P\x02\xb2M\x92\x02r\
M\x92\x02r\x13h\x01\x99\x09\xb4\x80L\x96\xc5\x80e\
\x96\xd5\x80E\x96\xd5\x80E\x96\xe5\x80y\x93\xe4\x80y\
\x96\xf5\x80\xd9\x04z\xc0,\xcb\x06\xc0\xb4I\x06\xc04\
\xcb\x0e\xc0d\x02\x07`\xd2$\x0b\xe0\xb2I\x16\xc0\xe5\
\x04\x1e\xc0\xc5\x04\x1e\xc0E\x96M\x80\xe7,\xbb\x00\xe7\
,\xbb\x00\xe7,\xdb\x00\xa7&\xd9\x00\xa7,\xfb\x00O\
\x13\xf8\x00OY6\x02\x8eM2\x02\x8eYv\x02\xc6\
\x8d\x190\xdc\xb8\x01\xe3\xda\x0c\x18n\xdc\x80qm\x06\
\x0c\xd7\xdb>\x80\xffx\xf2iE\x15\xa8J\x8b\x00\x00\
\x00\x00IEND\xaeB`\x82\
\x00\x00\x0b!\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00\xc8\x00\x00\x00\x8f\x08\x03\x00\x00\x00\x88\xdbdx\
\x00\x00\x00\x09pHYs\x00\x00\x0d.\x00\x00\x0d.\
\x01\xb5\xcb\x89]\x00\x00\x00\x19tEXtSof\
tware\x00www.inksca\
pe.org\x9b\xee<\x1a\x00\x00\x02\xacPL\
TE\x00\x00\x00\x00\xff\xff\x00\xff\x80U\xff\xaa@\xbf\
\x803\xcc\x99+\xd5\x80$\xdb\x92 \xdf\x809\xe3\x8e\
3\xe6\x80.\xe8\x8b+\xd5\x80'\xd8\x89$\xdb\x803\
\xdd\x880\xdf\x80-\xe1\x87+\xe3\x80(\xe4\x86&\xd9\
\x801\xdb\x86.\xdc\x80,\xde\x85+\xdf\x8a)\xe0\x85\
'\xe2\x89/\xe3\x84,\xdc\x84+\xdd\x88)\xde\x84.\
\xe0\x83-\xe1\x87,\xdb\x83+\xdc\x87)\xdd\x83/\xdd\
\x86.\xde\x83-\xdf\x86,\xe0\x83+\xe1\x86*\xdb\x82\
.\xdc\x85-\xdd\x82,\xde\x85+\xde\x88+\xdf\x85*\
\xe0\x87.\xe0\x85-\xdc\x87,\xdd\x84+\xdd\x87+\xde\
\x84*\xdf\x86.\xdf\x84-\xe0\x86,\xdc\x84+\xdc\x86\
+\xdd\x84.\xde\x86-\xde\x84-\xdf\x86,\xdf\x83+\
\xe0\x85+\xdc\x83-\xdd\x83,\xde\x85,\xde\x87+\xdf\
\x87-\xe0\x85-\xdd\x86,\xdd\x86+\xde\x84+\xde\x86\
-\xdf\x86,\xe0\x84,\xdd\x86-\xde\x84,\xdf\x84+\
\xdf\x85+\xdd\x84-\xdd\x85-\xdd\x84,\xde\x85,\xde\
\x86+\xde\x85+\xdf\x86-\xdf\x85-\xdd\x86,\xdd\x85\
+\xde\x85+\xde\x86-\xdf\x85-\xdf\x86,\xdd\x86+\
\xdd\x84+\xde\x85-\xde\x84,\xde\x85,\xdf\x84,\xdf\
\x85+\xdf\x84+\xdd\x85-\xdd\x84,\xde\x85,\xde\x84\
,\xde\x85+\xdf\x86-\xdf\x85-\xdd\x86,\xdd\x85,\
\xde\x86,\xde\x85+\xde\x86-\xde\x85,\xdd\x85+\xdd\
\x84+\xde\x85,\xde\x85,\xdf\x85+\xdf\x84+\xdd\x85\
-\xdd\x84,\xde\x85,\xde\x84+\xde\x86+\xdf\x85,\
\xdd\x85,\xde\x86,\xde\x85+\xde\x86,\xde\x85,\xdf\
\x85,\xdd\x85,\xdd\x85+\xde\x85-\xde\x84,\xde\x85\
,\xdf\x85,\xdf\x84+\xdd\x85-\xde\x84,\xde\x85,\
\xde\x84,\xde\x85,\xde\x86+\xdf\x85,\xdd\x85,\xde\
\x86,\xde\x85,\xde\x86-\xde\x85,\xde\x85,\xdf\x85\
,\xdd\x85,\xdd\x85+\xde\x85-\xde\x85,\xde\x84,\
\xde\x85,\xdf\x84+\xdd\x85-\xde\x84,\xde\x85,\xde\
\x84,\xde\x86-\xdf\x86,\xdd\x85,\xde\x86,\xde\x85\
,\xde\x85,\xde\x85,\xde\x85,\xdf\x85,\xde\x85,\
\xde\x85-\xde\x85,\xde\x85,\xde\x85,\xdf\x85,\xdd\
\x85-\xde\x84,\xde\x85,\xde\x84,\xde\x85,\xde\x85\
,\xdd\x85,\xde\x85,\xde\x85,\xde\x85+\xde\x85,\
\xde\x85,\xde\x85,\xdf\x85,\xde\x85,\xde\x85+\xde\
\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xdd\x85\
-\xde\x85,\xde\x85,\xde\x84,\xde\x86,\xde\x85,\
\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\
\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85\
,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85,\
\xde\x85,\xde\x85,\xde\x85,\xde\x85,\xde\x85\x83p\
\xec\x06\x00\x00\x00\xe3tRNS\x00\x01\x02\x03\x04\x05\
\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10\x11\x12\x13\x14\x15\
\x16\x17\x18\x19\x1a\x1b\x1d\x1e\x1f!\x22#$%&'\
()*+,-./01234567\
89:;<=>?@ABDEFHI\
JLMNPQRUWXYZ[\x5c]^\
_`abdefgijklmnop\
qrstuvwxyz{|}\x80\x81\x82\
\x84\x86\x87\x88\x89\x8a\x8b\x8d\x8e\x90\x91\x92\x93\x95\x96\x97\
\x98\x99\x9a\x9b\x9d\x9e\x9f\xa0\xa1\xa2\xa3\xa4\xa5\xa7\xa8\xa9\
\xaa\xab\xac\xad\xae\xaf\xb0\xb1\xb3\xb4\xb5\xb6\xb7\xb8\xb9\xbb\
\xbd\xbe\xbf\xc0\xc1\xc3\xc4\xc5\xc6\xc7\xc8\xc9\xca\xcc\xcd\xce\
\xcf\xd0\xd1\xd3\xd5\xd6\xd7\xd8\xd9\xda\xdb\xdc\xdd\xde\xdf\xe0\
\xe1\xe2\xe3\xe4\xe5\xe6\xe7\xe9\xea\xeb\xec\xed\xee\xef\xf0\xf1\
\xf2\xf3\xf4\xf5\xf6\xf7\xf8\xf9\xfa\xfb\xfc\xfd\xfe.\xcby\
/\x00\x00\x07\x07IDATx\xda\xdd\xdc\xf9_\x94\
E\x1c\x07\xf0\xd9\x85\xf5@E\x17\x0d\xf3B\x84\x0c\x8f\
\x14\xb7,\x8d4O\x14+\x0c\x0f4M\xad\x14\xad\xc4\
\xbc\x22q\xb3\x8c\xf2H\x10I3\xcb\x83\xd4\xb4C\xcd\
\xf2\xca#\xb5T\xa4\xbc\xc8\xa3\x02\x95\x92c\xbf\xffH\
?\xa8\xaf\xf8~\x9fk\xf6\xc1yf\xf6\x99\x1fe>\
\xcf\xcc\xdb=\x9e9\x9eY\xc6\xf4JT\x09DZ\xd1\
ux#\xcf\xa1\x0b\xf1\xae\x03W@\x22\xd2\x01nq\
\x80[\x1c\xe0\x16\x07\xb8\xc5\x01nq\x80[\x1c\xe0\x16\
\x07\xb8\xc5\xd1\x00\xe2Y\x0d\xae\x80D\xb8\x03\xdc\xe2\x00\
\xb78\xc0-\x0ep\x8b\x03\x18c\xcc\xb3\x0a\x5c\x01q\
\x85\x03\xdc\xe2\x00\xe6-z`\xd7\xba}G&\xa4\xd1\
\x8ek;>\xc8y>\x90\xe0\x8fa\x8c\xb1X\x7f\xe2\
\xc0\xacy+\xf6\xdet\x1e\xd2\xa8\xf4\xf5\xc2\x09I\xfa\
\xebI}\xa6o\xae\x8e\x14\xc8\xad\xe2\x91>fRZ\
f\x95\xd6E\x00\xa4\xaa\xa0\x03\xb3,]\x0b\xfeQ\x1c\
R\xf7n\x1b\xc6U\xba|\xae4\xe4d\x7f\xc6]F\
_V\x17R\xd8\x94\x85Q\x1e\xdag\xf2\x06\x0d6,\
\xeb\x1d\x86\x04Yx\xa5\xe9g\x86\x97\xba\x80*>\xed\
($4\x83\x85[\xbc\x85*B\x96\xb2\xf0K\xd4W\
\xeaAvE\xd9\x800\xff\x19\xd5 \xbf\xc72[\xa5\
w\x8db\x901\xfa\xfd\x8c{r\xfc\xdcE\xc1\xe0\xfc\
\xd73S[\xea\xd7\x08\xaa\x05\xd9\xaa\xf7Q\x1e\xbe\xe2\
T\xc3{\xe5\xe1`\xaaN\xad\x98\xf3*A\xeet\xd1\
t\xd07\xf3\x9c\xb6\xde\xe1\x17\xb4\x92L\x95 \x85\x9a\
\xee\x0d:\xad_\xf3\xbbn\x9a\x17\xee\xb4:\x90:\xcd\
\x98}f\xadQ\xdd\x1b#h\xddI\xea@6\xd2\xbe\
\xbde\xf66\xa4\x12\xdf\x05e CI\xd7\xc6\x87\xcc\
j\xdf|\x8cT_\xa2\x0a\xe4\x12\xb9\x17v\xfa\xcb\xbc\
\xfeq2\xefzD\x15H>\xf9\x1f.\xb5\x0a\xcc%\
\x81\xc3\x8a@z\xe1n=\x1e\xb2\x0a\x5c\x8d\xc1\x897\
\xd4\x80\x5c$\xff\xbf[\xad#\xb3p\xa2\x97\x1a\x90\xb5\
\xb8W\xedk\xac#\xc7\x88\xfd\xa2\x12\x90,\xdc\xa97\
y2\xe4\x8b\xabD\x09H<\xee\xd4\x01\x9eL\x1e\xce\
d\xdf\xff\xf7\xaf\x03wKo\xbc\x80\x14\xa0e\xa5\x00\
\xc8y2\xde\xe5Z\xb3:\x80C=\x0co\xad\x06e\
\x91\x00\xc8\x16\xdc\xc4X\xbeAM\x1c\x9e)\xdeV\x00\
\xb2\x107\xf1\x11_j\xa4\xfe\xfbQ&$\x1d7q\
\x90/\xb5\x18\xa7V*\x00\xc1S\x11\x1f\xe7Zh)\
\x19-\xcb\x87\xd4\xe0\x81Vw\xce\xd8\x19\xdc\xb1\x0c\xf9\
\x90s\xfa=\xb2*\xb5>\xdd{\xbbD\xc87V\xc3\
&\x83\x82\xe7b-\xe4C\x0am}i\x01\x0c\xc7\xb9\
\xab\xd2!\xe4\x1e\xbd\x997\x97\x8ds'\xa4C\xc8@\
\xf6G\xde\x5c.\xce\xed\x91\x0e\x99\x88[(\xe3\xcd-\
\xc7\xb9-\xf7\xa6\xc1ew\xcb~\xf4\xc7@\x19-7\
\x1e<d\x14\xee\xd0\x9f\xbc\xb9b\x9c+\x94>\xfa}\
\x0aw\xa8\xd6\xe6\x10-(\x1d\x92\x82\x17@\xc1\xe6\xd7\
\xf6\x02\xe9\x10\xbcr\xd8\x9e\xbb\x81\x83\xa6\xd31\x09\x90\
\xcex\xb3\x96\xbb\x012\xdb\xcd\x91\x0ey\x185\x99\xc8\
\xdd\xc0\xcf\x182C:$\xcej\xad\xcd\xa0\x9c\xc6\x90\
\xa9\xd2!\xad\xf4\xe7\xac\xe1\x0e6'J\x87\xc4\xda\x1a\
\xc5k\xc6\xf1\x93\xa4C\xda\xa2&\x13\xb8\x1b8\x81!\
\xd3\x15\xfb\xb0\xc7s7p\x08CfI\x87\xe0\x99n\
,w\x03{1$W\xb1\x1b\xa2\x8f\xbb\x81\xdd\x18\xb2\
X:\xa4\x07\xee\x10\xf7#e\xa5\xa6\x9b=\x12 \x03\
q\x87n\xf1\xe6\xc8\xcc\xe3=\xe9\x90\xd1\xb8C\xd7l\
\x0e\xe3WI\x87\x90)\xebY\xde\xdc\x87\xa6\x9b*\x12\
 \xb3q\x87\xbe\xe7\xcd-0\x9d\x22K\x80\xbc\x83;\
\xf4\x05on\x8a\xe9\x14Y\x02\xa4\x08w\xa8\xc0\xe6\x14\
\xf9\xb6t\xc8\xb7\xa6\xf3\x0a\xde\x99%\xbd\x91J\x80\x94\
aH:g\xac\xbe\x19\x8a%\xcb\x87\x90E\xec$\xce\
\xd8o\xd8?J>\x04\x12\xf0\xde\x13\xe7c\xfc;1\
d\x9e\x02\x10rG\xdc\xc7\x97z\x1b\xa76(\x00!\
{O\xef\xf3\xa52p\xea\xb8\x02\x90m\xe4Iq\xae\
P\x1d\x9e\x8fE\xff\xab\x00\x84|l[p\xed\xbd\x91\
U\xad\x9e`\x0a\x19\xe8\x08\x04\xc81\x8b]6>\x22\
\xaf\x98C\xfa;\x03!\xeb\xf1\xe3y2\x8f\xe2\xcc\x97\
\xf4\xefW\xd0\x9fS\x9d\x81\xac\xc7\x9dj\xfe\xb7u\x84\
<\xf8\xe0\xab\xa2\x15*\xcd\xdfyb \x15\x1e\xdc\xad\
|\xeb\x08y\x0eg\xb0\xa6B->\x00\xe4\x0c\x04\x9e\
\xc0\xddj[i\x158\x15eIoboI\xa3q\
\x102G\xa2\xeb\xb8\x9a\x12\x22\x1b\xa1\x1e\x9dG\x9d\xd1\
\x09'O\x8d3\x90kMH\xc7v\x9a\xd7'\xbbn\
l\x88N\x9d\x8e\xf66\xf4\x1a\x07\x81\xe7\xe8\x01\xbd\xfd\
f\xb57\xd1\x03\x1az\xe7\xc6\xba\xa3\x1a;\x1c\x82\xec\
\xa1\x9b\xae\xb1\xdb\x8c+\xaf\xa2\x87\x14\x93\xf5\x96\x90\xf0\
\xa3\xc4y\x0eA \x8dJ<\xaf\x19\xec\x8a^\xca\xd4\
l5\x17\xeb\xd5\x9b\x84\xaa\xf4s\x0a\xb2[\xe7\xacN\
~\x85\xb6^yn\x8c\xa6bj\xbd\xde\x15\xf3\xccG\
\xc7\xa2 0DgS?zT\xc1\xd1\x06\xef\x9a;\
?,\x1b\xe4\xd19e\xa2\xbf\xeeB\x0e\xa44[r\
\xd3\x19\xc8\x99\xe6\xfaO(4I\x1e\x915mN\xce\
\xb4\x17\x87v\x8d\xd6\xaf1[\xff\x82\xe74Wz6\
{N\xee\xb4\x9723\x86\x0d\x08$'\x16\x89\x82\xd8\
:\xf3\xc6\x18c}\xaa\xf9\x86\xa2\xa4,\x15\x06\xa9\x1d\
f\xcb\xd1\xae\xdc\xe8\x82\xe3Ls\xf3\x85A\xa0\xb2\xb7\
\x0dG\x0b\xe3\x85\xc9M\xa6\xc1W\xc5A\xa0\xbcS\xd8\
\x8eV{\x8d/W\xdd\xda,9A \x04.\xf7\x0b\
\xd3\xd1\xe1\x88\xd9\xe5r\xcd\xa2cDB\xa0*=,\
\xc73\x97L\xaf\xf6\x87\xd9\xc9\xf8\xc1B!\x10Z\xdd\
\x86\xdf\x91b\xf5 Q\x89I\xb8\xafX\x08\xc0\x95\xb1\
^n\xc9v\xab\x8b\x99\xbc\xb9\x12EC\x00~\x99\xca\
{:?\xc1r\x9bnY\xb4Q\xb6\xadx\x08@\xc5\
\xf24\xbe\x97e\xae\xe5\xa5\x8e\x0c6\x88\xfa\x9c\x80\x00\
@\xc5\x9a\xc9=\xad\x8f\x85G\x9f\xe0X\xa6\x98\x12\xaf\
\x19\x9du\xcb\x98W\x12r\x06\x02\x00P\xb5o]\xde\
\xcb\xe9\x81\x94\xc4x\xbf\xbfsR -;\x7f;=\
*>\xa0\x9eg\x03\xe2\xd8\x9a\x99\xa3\x03\x89q~\x7f\
\xe7\x94\xa1\xd9\x0b7\xfc\x14\xde\x0f\xc44\x1e\xc21\xc7\
e\x0e\xfc~\x97\x10H]_\x02i}92!p\
\x88~\x0b\x8c\x8bP\x08\xdd\xcbelg\x84B\xae\xb7\
#\x90\xa4\xea\xc8\x84\xc0\xc7\xf6\x9frW\x0bR?\x80\
NaOF&\x04\x8e\xd2\x1b\xe5\xa0\x90HG\x910\
\x08=\xa7a\xb0\xae\xf5\xa0\x1c^q\x90\xca\x8et\x00\
xUX[k\xbdL\x1c\x04>\xa5/\xc9d\x91\x0e\
\x81\x10\xcdODx\x8e\x0a{_\x89\x85\xfc\x8ag,\
i\xdb\x05\xbe\x1eB!0\xbf\xc1\x98<\xe3\x08\x08u\
\x08\x85Tw\xbd\xff\x9bg\xd9gA\xb0C(\xe4\xde\
\xb3\xb2\xads*@\xb8C,\x042\x18K\x08V\x8a\
\xbb~\xf1\xff\xc3l\xb1\x90\xf2\xfe\x9f\xd4\x80#\x0e\xc1\
\x10\xb1\xa5\xa1\x83\xb9\xc5\xc1\xdc\xe2\x88\x5c\xc8F2\xbc\
v\x8b\x83\xb9\xc5\xc1\xdc\xe2`nq0\xb78\x98[\
\x1c\xcc-\x8e\xc8\x83l\xd4\xdf\x15\xfa\x0f\x13\xf4\xa6<\
X\xcd\xe8\x1c\x00\x00\x00\x00IEND\xaeB`\x82\
\
\x00\x00\x03\xae\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00\xc0\x00\x00\x00\xb0\x04\x03\x00\x00\x00\xab\xd0|\xf5\
\x00\x00\x00-PLTE\x00\x00\x00A\xcdRA\xcd\
RA\xcdRD\xccQH\xcbPB\xccQI\xcbP\
D\xccQE\xccQE\xccQD\xccQD\xccQC\
\xccQA\xcdR\x9e%\xfb\x88\x00\x00\x00\x0etRN\
S\x00\x18Oc\x8f\xab\xae\xb8\xbd\xc9\xd6\xdf\xe9\xf0\xe7\
5\xddl\x00\x00\x03\x22IDATx\xda\xb5\xd8\xc1\
\x89\xdb@\x18\xc5q)\x15\x08\x1b\x0cY\xf6\x90l\x01\
\x82m\xc0\x90R\xdc\x80o!\xe7\x94\xa0\x06\x5cB \
%\xa4\x01\x1fR@\x0e\xbb\x10\xd8\xb0L\x0d9\xd8\xb2\
\xa5\x19[\xf3\xbd\xf7}o\xee\xf6\x8fw\xd0\x7f\x84\x9a\
\xc6y\xfa\xa1\xd1\x9e\xfe\xbd\x13\x03\xe9\x97\x1a\x10O\xe8\
S\xfa\xdd\x89\x81\xf4\xac\x06^;1 \x9d\xd0\xa7\x94\
\xd2\xdfN\x0c\xa4\x9d\x1a\x10N8\x01\xc2\x09g\xe0\xad\
\x13\x03iP\x03\xb2\x09# \x9bp\x01T\xcd\xbb\x00\
\xaal_\x01\xd1\x84+ \xca\xf6\x04\xd04o\x0aH\
\xb2=\x05$\x13f\x80\xa2y3@\xd1\xbc9 \x98\
0\x07\x04\x132 \xbey\x19\x10\xdf\xbc\x1c\x08\x9f\x90\
\x03\xe1\x13\x0a \xbay\x05\x10\x9d\xed\x12`&\xac\x9e\
\xee\x9em\x01\x10\xd9n\x8f\x09:p\xf3>`\xff\x8f\
g\x1b\x05\xe0\x090\x806\x0f\x06\xd0\xe6\xe1\x008\x01\
\x07\xc0\x09\x04\x805\x8f\x00\xb0\xe61\x004\x81\x01\xa0\
\x09\x14\x804\x8f\x02\x90ls\x000\x81\x03\x80l\x93\
\x80\xbdy,`\xce6\x0b\x98'\xd0\x80\xb5y4`\
m\x1e\x0f\x18'\xf0\x80q\x82\x03x\xab\xbd\xb0|q\
\x02\xa9\x02<\xbch\x81\xf6\xe8\x05^\x97\x81\xc7\xe4\x05\
\x96\x1f\xb5\xd5\xd1\x0b,\xc7\xa2\xdd&'\xf0\xfeyq\
\xc0:y\x81\xe5;\xa7\xddz\x81\xca\x95\xb3N^\xe0\
\xe7\xf2\x80\xbd\x17\xa8\xbc\xb9l\x92\x17\x18\x0c\x03<\x80\
i\x80\x07\xf8\xbe<\xe0\xe0\x05*W\xc1\xc7\xe4\x05v\
\xa6\x01<P\x19\xf0\x90\xbc\xc0s%\xd3^\xa0\xf2J\
\xf4\x98\xbc@5\xd3N\xa0\x9ei\x1f`\xc8\xb4\x0f0\
d\xda\x05X2\xed\x02,\x99\x9e\x01\xed\x0fA\xe5\xa6\
\x00\xf8Aj\x00\x06\x9c\x81\x85\xd3\xfb\x060\x80-\xd3\
<`\xcc4\x0f\xec\xa0\x018`\xcd4\x0dX3\xcd\
\x02\xe6L\xb3\x809\xd3$`\xcf\xf4\xe5\x17\xf7\x9f\xe1\
\xa7\xae\x00\x80L[\xce\xa7\x02\x002M\x01H\xa6)\
\x00\xc94\x03`\x95#\x80!x@\x0e\x84\x0f\xc8\x01\
,\xd38\x00f\x1a\x07v\xe1\x03\xe6\x00\x9ai\x18@\
3\x8d\x02p\xa6Q\x00\xce4\x08\xe0\x99\xc6\x80\xe0L\
\x97@p\xa6\x0b :\xd3\x05\x10\x9d\xe9\x1c\x88\xaf\x5c\
\x06\x0c\xa2\x01# \x1b0\x02\xcb\x99n\x0e^\xa0\xf6\
\xfd;y\x81\xda\xe7\xef\xa3\x13XU\x06\x90\x9d\xbb\x02\
\xd5\xd3\x1e\xc4\x00uYB\x00?\xc1\x08\xf0\x13\xac\x00\
\xfd,[\x01\xfaa6\x03\xec\x043\xc0N\xb0\x03\xe4\
\x9df\x07\xc8K\x0d\x00\xb8\x09\x00\xd0\xac\xfe\x88\x01\xaa\
y\x10\xc0\xbc\x9dB\x003\x01\x03\x88\xe6a\x00\xd1<\
\x10\xc0'\x80\x00>\x01\x05\xe0\xe6\xa1\x00\xdc<\x18\xb8\
5\xe1\xdf\xb7\xbb\xe7k\xd7\x04Lxi\x22\xcf\x8d\xe6\
\xc5\x027\xb2\x1d\x0c\x94\x13\x82\x812\xdb\xd1@\xd1\xbc\
p \xcfv8\x90O\x88\x07\xb2\xe6\xc5\x03Y\xf3\x04\
\xc0|\x82\x00\x98OP\x00\xb3\xe6)\x80Y\xf3$\xc0\
t\x82\x04\x98N\xd0\x00\x93\xe6i\x80I\xb6E\xc0u\
\x82\x08\xb8f[\x05\x5c\x9a'\x03\xc6l\xcb\x80q\x82\
\x0e87O\x07\x9c\x9b'\x04N\x13\x84\xc0i\x82\x12\
h\xf7b\xa0\xd9\xa8\x81v/\x06\x9a\x8d\x1ah\xb7b\
\xa0Y\xab\x81v\xeb\x06\xfe\x03$\xe7\x91\x89\x97#\xf5\
\xaf\x00\x00\x00\x00IEND\xaeB`\x82\
\x00\x00\x00\xbf\
/\
/ Copyright (C) \
2020 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick.Co\
ntrols.Material\x0a\
\x0aToolBar {\x0a    M\
aterial.foregrou\
nd: \x22white\x22\x0a}\x0a\
\x00\x00\x01\x0a\
[\
Icon Theme]\x0aName\
=Gallery\x0aComment\
=Qt Quick Contro\
ls 2 Gallery Exa\
mple Icon Theme\x0a\
\x0aDirectories=20x\
20,20x20@2,20x20\
@3,20x20@4\x0a\x0a[20x\
20]\x0aSize=20\x0aType\
=Fixed\x0a\x0a[20x20@2\
]\x0aSize=20\x0aScale=\
2\x0aType=Fixed\x0a\x0a[2\
0x20@3]\x0aSize=20\x0a\
Scale=3\x0aType=Fix\
ed\x0a\x0a[20x20@4]\x0aSi\
ze=20\x0aScale=4\x0aTy\
pe=Fixed\x0a\
\x00\x00\x00\xe3\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00<\x00\x00\x00<\x04\x03\x00\x00\x00\xc8\xd2\xc4A\
\x00\x00\x00$PLTE\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xa6X\x02G\
\x00\x00\x00\x0ctRNS\x00\x1c\x1d\x1e=l|}\
\xd8\xd9\xda\xfa\x85T\xfd\xb6\x00\x00\x00bIDAT\
8\xcbc`\xa070\xc5+\xcb4\xbb\x00\x9f\xb4\xd5\
\xeemx5\xef\xde]\x80O\xf3\xee\xdd[\xf0j\xde\
\x9d\x80W\xf36r5k\x8ej\x1e\x11\x9a\xadh\xa7\
\x99A\x02(\xbd\x1d\xb74c\xf7n\x82\x89\x7f\x1b\xf9\
\xda)\xb4}T\xfb\x10\xd5Na\xb2\xc1_\xe4\x82\xb5\
'\x90]\xdc\x03\xb5'PP\xd5P\x0d\x00\x00^\xe9\
\x99\xeb`\xde0?\x00\x00\x00\x00IEND\xaeB\
`\x82\
\x00\x00\x00\x82\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00<\x00\x00\x00<\x02\x03\x00\x00\x00G\x921\xe1\
\x00\x00\x00\x09PLTE\x00\x00\x00\xff\xff\xff\x00\x00\
\x00s\xc6\x83q\x00\x00\x00\x02tRNS\x00\x00v\
\x93\xcd8\x00\x00\x00&IDAT(\xcfc\x08E\
\x05\x0c\x03\xcd\x0f[\x85\x0c\xa6R\x9d\x1f\xc2\x80\x0cD\
\x87\x1c\x7f4|\x066|\x06Y~\x01\x00\x0e\xe9\x0f\
x\xb7\x12\x1d\x13\x00\x00\x00\x00IEND\xaeB`\
\x82\
\x00\x00\x00\xc1\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00<\x00\x00\x00<\x04\x03\x00\x00\x00\xc8\xd2\xc4A\
\x00\x00\x00\x15PLTE\x00\x00\x00\xff\xff\xff\x22\x1f\
\x1f\x22\x1f\x1f\x22\x1f\x1f\x22\x1f\x1f\x22\x1f\x1fI:\x15\
.\x00\x00\x00\x06tRNS\x00\x00 0\x9f\xef,\
\x93$\x03\x00\x00\x00UIDAT8\xcbc\x10\xc4\
\x0b\x18\xa8'\xcd\xa0\x12\x96\xea\xc4\x80S\x9a9,-\
-\xd5\x00\xa7\xb4j\x1a\x10\x04\xe1\x94v\x03I\xa7\xe0\
\x94\x0e\x03I\xa7\xe2\x94N\x03\x03r\xa5\x09\x18N\xc0\
i\x04<F X\x08\x04*\x03\x18\x8cJ\x8f&\xc5\
\xd1\xa48\x9a\x14G\x93\x22m\xaa9\x00L\x0e\xa9W\
\xb5\x91\xf7\x9e\x00\x00\x00\x00IEND\xaeB`\x82\
\
\x00\x00\x00\xf6\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00P\x00\x00\x00P\x04\x03\x00\x00\x00|?\xef\x9e\
\x00\x00\x00\x1bPLTE\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x94Q\xe2 \x00\x00\x00\x09tRNS\x00\
1267\xe8\xe9\xeb\xfa\x91\xc7O\xc2\x00\x00\x00\x81\
IDATH\xc7\xed\xd6\xb1\x0d\x800\x0cDQ\x93\
\xcc\x90\x15R#Q\xd0\xb2\x05l\x00=\x14\xd9 7\
6#\xf0\x0b\x22\x82d\xd7\xa7gW\xd6\x99\xfd{&\
\x98\x0b\xfb\xc2\x82Y\x17\xca\xc5\xa2:2P\x88\x0c\x87\
\xa4\xba0\x90\xec\x0e\x87$\x9d\xcf\xe0\xfc6\x98\x1dt\
\xd0\xc1\x9e\xc1X\xbe\x02\x87M\xf0#&\xc1\x8f\x88w\
s\xb2\xc1\x95N:\xe9d{\x92\xd6\x99fW\xa2^\
\x98\xe0\xc7\xe2E\xd3\x12\x03\xcd\x86\x95\x81\xbc^w3\
7\xa7\xd5\xc3\xb4x\xa1\x14\xf6\x00\x00\x00\x00IEN\
D\xaeB`\x82\
\x00\x00\x00\x83\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00P\x00\x00\x00P\x02\x03\x00\x00\x00\xf3\x7f\x1a>\
\x00\x00\x00\x09PLTE\x00\x00\x00\xff\xff\xff\x00\x00\
\x00s\xc6\x83q\x00\x00\x00\x02tRNS\x00\x00v\
\x93\xcd8\x00\x00\x00'IDAT8\xcbc\x08\xc5\
\x02\x18F\x8a\xe0*\x0c08\x05\x190\xc0\xa8 %\
\x82\xa3\xf1>\x1a\xef\x839\xdeGn\x99\x0c\x00\xaa\xfe\
\xe3\x80<\x1b\xa3\xfb\x00\x00\x00\x00IEND\xaeB\
`\x82\
\x00\x00\x00\xdf\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00P\x00\x00\x00P\x04\x03\x00\x00\x00|?\xef\x9e\
\x00\x00\x00\x18PLTE\x00\x00\x00\xff\xff\xff\x22\x1f\
\x1f\x22\x1f\x1f\x22\x1f\x1f\x22\x1f\x1f\x22\x1f\x1f\x22\x1f\x1f\
G\xb0\x1c\xf2\x00\x00\x00\x07tRNS\x00\x00 `\
\xbf\xcf\xef\x1dV\xbf\x1a\x00\x00\x00oIDATH\
\xc7c\x10$\x120\x0c\x22\x85\x0c\x0c&\xe5\xe5\xce\x0c\
\x0c\x84\x15\xaa\x96\x03A\x10a\x85L\xe9 \x85e\x0a\
\x04\x15\xb2\x96\x83A\x00A\x85\xe6\x10\x85\xc5\x04\x15\xba\
C\x14\x96\x10TX\x0e\x05\x83Y!\xd1\x9e!:x\
\x88\x0ep\xa2\xa3\x90\xe8DA|2\x83\x82Q\x85#\
A\xe1h\xd93Z\xf6\x8c\x96=\xa3\x0aG\xcb\x9e\xd1\
\xb2gd\x97=\x83\xb0\xaf\x00\x00G\x10>\xf2\xcfQ\
\xf1\x00\x00\x00\x00\x00IEND\xaeB`\x82\
\x00\x00\x00\x92\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00\x14\x00\x00\x00\x14\x08\x06\x00\x00\x00\x8d\x89\x1d\x0d\
\x00\x00\x00YIDAT8\x8d\xed\xd4\xb1\x09\x80@\
\x0c\x05\xd0\x87\x16\x8e\xe4\xa6\xda\x0a\xeeegw\x13X\
\xd9Z\x08\xe2%bs\xbf\xcfKB $\xa7\xaf\xac\
\x99P\xb0g\x0c\xb0\xe2\xc0\x86!\x0b+\x18\xa3\xd8\xd2\
\xb0\x7f\xb1\xeakv\x91\xceo\x92\xbarC\xbfE\xd3\
\x9e\xc3\x1d\x1a~_Wt\x964\xe1cN\x85\xa7,\
\x143\xcaZ\x14\x00\x00\x00\x00IEND\xaeB`\
\x82\
\x00\x00\x00{\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00\x14\x00\x00\x00\x14\x08\x06\x00\x00\x00\x8d\x89\x1d\x0d\
\x00\x00\x00BIDATx\xda\xec\x94\xc1\x0d\x000\
\x08\x02\x8f\xc6\xfdW\xc6\x1d\x1a|)\x03\x10\xf0\x0c\xb2\
MR\x8f\xb0\xe2\x86\x05 )\xd2\xdb\xb6f\x12\x02\xda\
\x03e\xec\x86\xa9\xef>\xcaG\xf9\xcbq\xdf\xc06\x00\
\x00\x00\xff\xff\x03\x00\xf7\x83\x0c-\xe7\xf9\xf9<\x00\x00\
\x00\x00IEND\xaeB`\x82\
\x00\x00\x00{\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00\x14\x00\x00\x00\x14\x02\x03\x00\x00\x00\xf0\xe7\xf5\x9e\
\x00\x00\x00\x0cPLTE\x00\x00\x00\xff\xff\xff\x22\x1f\
\x1f\x22\x1f\x1f\x13\xee\x05I\x00\x00\x00\x03tRNS\
\x00\x00\x80\x17\xceG\xfe\x00\x00\x00\x1bIDAT\x08\
[c\x08\x05\x01\x86\xd0\xd0}`\xf2?\x12\x09\x11\xc1\
D\x94\xab\x04\x02\x00\xd4P%\xa9\xa8Z\xf7\x1d\x00\x00\
\x00\x00IEND\xaeB`\x82\
\x00\x00\x00\xb8\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00(\x00\x00\x00(\x04\x03\x00\x00\x00~\xd0\xa5^\
\x00\x00\x00\x18PLTE\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\
V7\x1c*\x00\x00\x00\x08tRNS\x00\x0b\x0e\x90\
\xbc\xbd\xc2\xfa`jr\x94\x00\x00\x00GIDAT\
(\xcfc` \x0f(+`\x8a1\xa5'a\x0a\x8a\
\x96\x97a(e\x0c\xc7\x22\xa8V^\x9eD\x94B\xd1\
\xe1\xa4\x10\x9b\xaf\x99\xd2\xb1\x85\x99;\x16A\xac\xdaq\
(\x15\x19\x9eJI\x0a\x01,\x82X\x13-\xd6\xe4\x8d\
\x17\x00\x00\xbe(.\x06\xf5\xa2\x90\xab\x00\x00\x00\x00I\
END\xaeB`\x82\
\x00\x00\x00~\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00(\x00\x00\x00(\x02\x03\x00\x00\x00\xf1\x90P\xfe\
\x00\x00\x00\x09PLTE\x00\x00\x00\xff\xff\xff\x00\x00\
\x00s\xc6\x83q\x00\x00\x00\x02tRNS\x00\x00v\
\x93\xcd8\x00\x00\x00\x22IDAT\x18\x95c\x08\x85\
\x03\x062\x99Q\xab\xa0`)af\x00\x03\x14\xb0R\
\x979\xd4\xdc@q\xa8\x03\x00\xea\x17x\xdd\x86\xef\x87\
\xad\x00\x00\x00\x00IEND\xaeB`\x82\
\x00\x00\x00\x9e\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00(\x00\x00\x00(\x04\x03\x00\x00\x00~\xd0\xa5^\
\x00\x00\x00\x0fPLTE\x00\x00\x00\xff\xff\xff\x22\x1f\
\x1f\x22\x1f\x1f\x22\x1f\x1f\xc1T\xc5\x91\x00\x00\x00\x04t\
RNS\x00\x00P\xdf\x1fD\xf4\x8b\x00\x00\x00:I\
DAT(\xcfc\x10\xc4\x02\x18\xf0\x0a2\x998+\
`\x08\xaa\xb8\xb88a\x08\x9a\xb8\xb88c\x08\xba\x00\
\x01q\x82X\xb5c\xb5\x08\xab\x93\x18\x80`\xe0\x04G\
Ci8\x85\x12\xc1\xdc\x01\x00\xcb~A\x15\x05\x8a+\
\xbc\x00\x00\x00\x00IEND\xaeB`\x82\
"

qt_resource_name = b"\
\x00\x05\
\x00o\xa6S\
\x00i\
\x00c\x00o\x00n\x00s\
\x00\x0b\
\x0cCR|\
\x00g\
\x00a\x00l\x00l\x00e\x00r\x00y\x00.\x00q\x00m\x00l\
\x00\x15\
\x08\x1e\x16f\
\x00q\
\x00t\x00q\x00u\x00i\x00c\x00k\x00c\x00o\x00n\x00t\x00r\x00o\x00l\x00s\x002\x00.\
\x00c\x00o\x00n\x00f\
\x00\x09\
\x08\xac\xef\x1c\
\x00+\
\x00M\x00a\x00t\x00e\x00r\x00i\x00a\x00l\
\x00\x0b\
\x083\x9c<\
\x00T\
\x00o\x00o\x00l\x00B\x00a\x00r\x00.\x00q\x00m\x00l\
\x00\x06\
\x07\x03}\xc3\
\x00i\
\x00m\x00a\x00g\x00e\x00s\
\x00\x06\
\x07\x84+\x02\
\x00q\
\x00m\x00l\x00d\x00i\x00r\
\x00\x05\
\x00v}\xc3\
\x00p\
\x00a\x00g\x00e\x00s\
\x00\x0f\
\x07\x87\xe7<\
\x00S\
\x00p\x00i\x00n\x00B\x00o\x00x\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\x00\x15\
\x02E\x02\xfc\
\x00P\
\x00a\x00g\x00e\x00I\x00n\x00d\x00i\x00c\x00a\x00t\x00o\x00r\x00P\x00a\x00g\x00e\
\x00.\x00q\x00m\x00l\
\x00\x0e\
\x09\x09P|\
\x00T\
\x00a\x00b\x00B\x00a\x00r\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\x00\x15\
\x0fE\xff\x5c\
\x00B\
\x00u\x00s\x00y\x00I\x00n\x00d\x00i\x00c\x00a\x00t\x00o\x00r\x00P\x00a\x00g\x00e\
\x00.\x00q\x00m\x00l\
\x00\x12\
\x05P\x5c\x1c\
\x00S\
\x00c\x00r\x00o\x00l\x00l\x00a\x00b\x00l\x00e\x00P\x00a\x00g\x00e\x00.\x00q\x00m\
\x00l\
\x00\x13\
\x0dr\xfb|\
\x00P\
\x00r\x00o\x00g\x00r\x00e\x00s\x00s\x00B\x00a\x00r\x00P\x00a\x00g\x00e\x00.\x00q\
\x00m\x00l\
\x00\x13\
\x08\xce#\x1c\
\x00S\
\x00e\x00a\x00r\x00c\x00h\x00F\x00i\x00e\x00l\x00d\x00P\x00a\x00g\x00e\x00.\x00q\
\x00m\x00l\
\x00\x10\
\x06\xd3\x8b\x1c\
\x00T\
\x00e\x00x\x00t\x00A\x00r\x00e\x00a\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\x00\x0f\
\x0b\xe33|\
\x00T\
\x00o\x00o\x00l\x00T\x00i\x00p\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\x00\x0f\
\x0b\xc87|\
\x00T\
\x00o\x00o\x00l\x00B\x00a\x00r\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\x00\x0e\
\x02%\xd0|\
\x00S\
\x00l\x00i\x00d\x00e\x00r\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\x00\x0f\
\x00\xf4\xb9\xfc\
\x00T\
\x00u\x00m\x00b\x00l\x00e\x00r\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\x00\x10\
\x02a.\x1c\
\x00T\
\x00r\x00e\x00e\x00V\x00i\x00e\x00w\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\x00\x11\
\x0fX4\x5c\
\x00M\
\x00o\x00n\x00t\x00h\x00G\x00r\x00i\x00d\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\
\x00\x11\
\x05\x22\x04\xdc\
\x00S\
\x00p\x00l\x00i\x00t\x00V\x00i\x00e\x00w\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\
\x00\x11\
\x03\x22,\x5c\
\x00T\
\x00a\x00b\x00l\x00e\x00V\x00i\x00e\x00w\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\
\x00\x0e\
\x0e\xa2\x84\x9c\
\x00B\
\x00u\x00t\x00t\x00o\x00n\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\x00\x11\
\x04\xf7 \x1c\
\x00G\
\x00a\x00l\x00l\x00e\x00r\x00y\x00C\x00o\x00n\x00f\x00i\x00g\x00.\x00q\x00m\x00l\
\
\x00\x0e\
\x0b\xc5|\x5c\
\x00S\
\x00w\x00i\x00t\x00c\x00h\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\x00\x11\
\x0fw<\xdc\
\x00T\
\x00e\x00x\x00t\x00F\x00i\x00e\x00l\x00d\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\
\x00\x10\
\x05\xb8\x16\x1c\
\x00C\
\x00o\x00m\x00b\x00o\x00B\x00o\x00x\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\x00\x10\
\x00w\xb4\x5c\
\x00D\
\x00e\x00l\x00e\x00g\x00a\x00t\x00e\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\x00\x0c\
\x0e\x8bV\xfc\
\x00D\
\x00i\x00a\x00l\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\x00\x0d\
\x0c\xc8%\xdc\
\x00F\
\x00r\x00a\x00m\x00e\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\x00\x0f\
\x0c\xe8\x19\xfc\
\x00M\
\x00e\x00n\x00u\x00B\x00a\x00r\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\x00\x11\
\x03$Q\x5c\
\x00S\
\x00w\x00i\x00p\x00e\x00V\x00i\x00e\x00w\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\
\x00\x11\
\x00\xa3\xff|\
\x00S\
\x00c\x00r\x00o\x00l\x00l\x00B\x00a\x00r\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\
\x00\x13\
\x002\xb3\xbc\
\x00R\
\x00a\x00n\x00g\x00e\x00S\x00l\x00i\x00d\x00e\x00r\x00P\x00a\x00g\x00e\x00.\x00q\
\x00m\x00l\
\x00\x10\
\x0fY+\x5c\
\x00G\
\x00r\x00o\x00u\x00p\x00B\x00o\x00x\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\x00\x13\
\x0fN\x9e\x5c\
\x00R\
\x00a\x00d\x00i\x00o\x00B\x00u\x00t\x00t\x00o\x00n\x00P\x00a\x00g\x00e\x00.\x00q\
\x00m\x00l\
\x00\x17\
\x0a>\xfa\x1c\
\x00S\
\x00c\x00r\x00o\x00l\x00l\x00I\x00n\x00d\x00i\x00c\x00a\x00t\x00o\x00r\x00P\x00a\
\x00g\x00e\x00.\x00q\x00m\x00l\
\x00\x0e\
\x08\xd2\xfe\x5c\
\x00D\
\x00i\x00a\x00l\x00o\x00g\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\x00\x10\
\x048\xf8\x1c\
\x00C\
\x00h\x00e\x00c\x00k\x00B\x00o\x00x\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\x00\x13\
\x05\xac\xa6\xdc\
\x00D\
\x00e\x00l\x00a\x00y\x00B\x00u\x00t\x00t\x00o\x00n\x00P\x00a\x00g\x00e\x00.\x00q\
\x00m\x00l\
\x00\x11\
\x07%R|\
\x00S\
\x00t\x00a\x00c\x00k\x00V\x00i\x00e\x00w\x00P\x00a\x00g\x00e\x00.\x00q\x00m\x00l\
\
\x00\x09\
\x09j\x86g\
\x00a\
\x00r\x00r\x00o\x00w\x00.\x00p\x00n\x00g\
\x00\x0a\
\x06\xebDg\
\x00a\
\x00r\x00r\x00o\x00w\x00s\x00.\x00p\x00n\x00g\
\x00\x0e\
\x0d=\xfd'\
\x00q\
\x00t\x00-\x00l\x00o\x00g\x00o\x00@\x004\x00x\x00.\x00p\x00n\x00g\
\x00\x0d\
\x02\xa9\x8a\x07\
\x00a\
\x00r\x00r\x00o\x00w\x00s\x00@\x002\x00x\x00.\x00p\x00n\x00g\
\x00\x0c\
\x0e\xa8z'\
\x00a\
\x00r\x00r\x00o\x00w\x00@\x004\x00x\x00.\x00p\x00n\x00g\
\x00\x0c\
\x0e\xb8z'\
\x00a\
\x00r\x00r\x00o\x00w\x00@\x003\x00x\x00.\x00p\x00n\x00g\
\x00\x0e\
\x0d-\xfd'\
\x00q\
\x00t\x00-\x00l\x00o\x00g\x00o\x00@\x003\x00x\x00.\x00p\x00n\x00g\
\x00\x0c\
\x0e\x88z'\
\x00a\
\x00r\x00r\x00o\x00w\x00@\x002\x00x\x00.\x00p\x00n\x00g\
\x00\x0e\
\x0d\x1d\xfd'\
\x00q\
\x00t\x00-\x00l\x00o\x00g\x00o\x00@\x002\x00x\x00.\x00p\x00n\x00g\
\x00\x0d\
\x02\x89\x8a\x07\
\x00a\
\x00r\x00r\x00o\x00w\x00s\x00@\x004\x00x\x00.\x00p\x00n\x00g\
\x00\x0b\
\x05R\xbf'\
\x00q\
\x00t\x00-\x00l\x00o\x00g\x00o\x00.\x00p\x00n\x00g\
\x00\x0d\
\x02\x99\x8a\x07\
\x00a\
\x00r\x00r\x00o\x00w\x00s\x00@\x003\x00x\x00.\x00p\x00n\x00g\
\x00\x07\
\x0d\x83,Y\
\x00g\
\x00a\x00l\x00l\x00e\x00r\x00y\
\x00\x07\
\x05{TR\
\x002\
\x000\x00x\x002\x000\x00@\x002\
\x00\x05\
\x005{P\
\x002\
\x000\x00x\x002\x000\
\x00\x07\
\x05{TT\
\x002\
\x000\x00x\x002\x000\x00@\x004\
\x00\x0b\
\x0b\xba\x81\xb5\
\x00i\
\x00n\x00d\x00e\x00x\x00.\x00t\x00h\x00e\x00m\x00e\
\x00\x07\
\x05{TS\
\x002\
\x000\x00x\x002\x000\x00@\x003\
\x00\x08\
\x07\x9eZG\
\x00b\
\x00a\x00c\x00k\x00.\x00p\x00n\x00g\
\x00\x0a\
\x0d\xc8&G\
\x00d\
\x00r\x00a\x00w\x00e\x00r\x00.\x00p\x00n\x00g\
\x00\x08\
\x0cXY'\
\x00m\
\x00e\x00n\x00u\x00.\x00p\x00n\x00g\
"

qt_resource_struct = b"\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x08\x00\x00\x00\x01\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x009\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\xb4\x00\x02\x00\x00\x00#\x00\x00\x00\x16\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x90\x00\x02\x00\x00\x00\x0c\x00\x00\x00\x0a\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\xa2\x00\x00\x00\x00\x00\x01\x00\x00\x0c\xe1\
\x00\x00\x01\x9b\xc6<\xa6)\
\x00\x00\x00,\x00\x00\x00\x00\x00\x01\x00\x00\x0b\xe5\
\x00\x00\x01\x99-^\x9a:\
\x00\x00\x00t\x00\x00\x00\x00\x00\x01\x00\x00\x0cI\
\x00\x00\x01\x9b\xc6<(c\
\x00\x00\x00\x5c\x00\x02\x00\x00\x00\x01\x00\x00\x00\x09\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x10\x00\x01\x00\x00\x00\x01\x00\x00\x00\x00\
\x00\x00\x01\x9b\xc6D\x84W\
\x00\x00\x00t\x00\x00\x00\x00\x00\x01\x00\x01\x1f\xbb\
\x00\x00\x01\x975l\xc7\xef\
\x00\x00\x070\x00\x00\x00\x00\x00\x01\x00\x01\x0c%\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x07l\x00\x00\x00\x00\x00\x01\x00\x01\x1c\x09\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x06r\x00\x00\x00\x00\x00\x01\x00\x00\xd2\x7f\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x07P\x00\x00\x00\x00\x00\x01\x00\x01\x10\xe4\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x066\x00\x00\x00\x00\x00\x01\x00\x00\xab\x8c\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x06\x1e\x00\x00\x00\x00\x00\x01\x00\x00\xaaI\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x07\x0e\x00\x00\x00\x00\x00\x01\x00\x00\xf8\xd9\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x06\xce\x00\x00\x00\x00\x00\x01\x00\x00\xda\x9f\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x06P\x00\x00\x00\x00\x00\x01\x00\x00\xadZ\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x06\xf0\x00\x00\x00\x00\x00\x01\x00\x00\xf6\xf9\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x06\x92\x00\x00\x00\x00\x00\x01\x00\x00\xd5>\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x06\xb0\x00\x00\x00\x00\x00\x01\x00\x00\xd88\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x04\xd0\x00\x00\x00\x00\x00\x01\x00\x00\x83g\
\x00\x00\x01\x9b\xc6CC\xeb\
\x00\x00\x03\xf8\x00\x01\x00\x00\x00\x01\x00\x00f\x89\
\x00\x00\x01\x99.H\x8e\xfe\
\x00\x00\x04\xa8\x00\x00\x00\x00\x00\x01\x00\x00~\x10\
\x00\x00\x01\x99.H\x8e\xff\
\x00\x00\x02|\x00\x00\x00\x00\x00\x01\x00\x003\xf0\
\x00\x00\x01\x99.H\x8f\x00\
\x00\x00\x02Z\x00\x00\x00\x00\x00\x01\x00\x0009\
\x00\x00\x01\x9b\xc6CC\xeb\
\x00\x00\x00\xe8\x00\x00\x00\x00\x00\x01\x00\x00\x10\x84\
\x00\x00\x01\x99.H\x8e\xfe\
\x00\x00\x02\xa0\x00\x00\x00\x00\x00\x01\x00\x006l\
\x00\x00\x01\x99.H\x8f\x00\
\x00\x00\x03\x16\x00\x00\x00\x00\x00\x01\x00\x00M\x16\
\x00\x00\x01\x99.H\x8e\xff\
\x00\x00\x04\x80\x00\x00\x00\x00\x00\x01\x00\x00x\x9b\
\x00\x00\x01\x99.H\x8e\xff\
\x00\x00\x05\xa4\x00\x00\x00\x00\x00\x01\x00\x00\x9b\x9a\
\x00\x00\x01\x99.H\x8e\xfd\
\x00\x00\x03`\x00\x00\x00\x00\x00\x01\x00\x00X\xf5\
\x00\x00\x01\x99.H\x8e\xfe\
\x00\x00\x02\xee\x00\x00\x00\x00\x00\x01\x00\x00EN\
\x00\x00\x01\x99.H\x8e\xff\
\x00\x00\x01j\x00\x00\x00\x00\x00\x01\x00\x00\x1b\x9e\
\x00\x00\x01\x99-e\xb2 \
\x00\x00\x05\xca\x00\x00\x00\x00\x00\x01\x00\x00\x9fr\
\x00\x00\x01\x99.H\x8e\xfd\
\x00\x00\x03\xd2\x00\x00\x00\x00\x00\x01\x00\x00`2\
\x00\x00\x01\x99.H\x8e\xfd\
\x00\x00\x01\xec\x00\x00\x00\x00\x00\x01\x00\x00'\xfc\
\x00\x00\x01\x9b\xc6CC\xeb\
\x00\x00\x05\xf6\x00\x00\x00\x00\x00\x01\x00\x00\xa2d\
\x00\x00\x01\x99.H\x8e\xff\
\x00\x00\x00\xc4\x00\x00\x00\x00\x00\x01\x00\x00\x0d$\
\x00\x00\x01\x9b\xc6F\x1b\xb0\
\x00\x00\x01\xc0\x00\x00\x00\x00\x00\x01\x00\x00!q\
\x00\x00\x01\x99.H\x8e\xff\
\x00\x00\x05\x82\x00\x01\x00\x00\x00\x01\x00\x00\x95\xc9\
\x00\x00\x01\x99.H\x8e\xfe\
\x00\x00\x01\x18\x00\x00\x00\x00\x00\x01\x00\x00\x13\x12\
\x00\x00\x01\x99.H\x8e\xff\
\x00\x00\x05N\x00\x00\x00\x00\x00\x01\x00\x00\x91F\
\x00\x00\x01\x99.H\x8e\xff\
\x00\x00\x03\x88\x00\x00\x00\x00\x00\x01\x00\x00Y\xb9\
\x00\x00\x01\x99.H\x8e\xff\
\x00\x00\x026\x00\x01\x00\x00\x00\x01\x00\x00-\xf8\
\x00\x00\x01\x99.H\x8f\x00\
\x00\x00\x02\x12\x00\x00\x00\x00\x00\x01\x00\x00+\x08\
\x00\x00\x01\x99.H\x8f\x00\
\x00\x00\x04<\x00\x00\x00\x00\x00\x01\x00\x00n\xa5\
\x00\x00\x01\x99.H\x8e\xfe\
\x00\x00\x04\x5c\x00\x00\x00\x00\x00\x01\x00\x00s\xd6\
\x00\x00\x01\x99.H\x8e\xfe\
\x00\x00\x01\x94\x00\x00\x00\x00\x00\x01\x00\x00\x1d\xa3\
\x00\x00\x01\x9b\xc6CC\xeb\
\x00\x00\x04\x1e\x00\x00\x00\x00\x00\x01\x00\x00kw\
\x00\x00\x01\x9b\xc6CC\xeb\
\x00\x00\x03>\x00\x01\x00\x00\x00\x01\x00\x00V\xad\
\x00\x00\x01\x99.H\x8e\xfd\
\x00\x00\x01:\x00\x00\x00\x00\x00\x01\x00\x00\x18\xce\
\x00\x00\x01\x99.H\x8e\xfd\
\x00\x00\x05\x22\x00\x00\x00\x00\x00\x01\x00\x00\x8d\x01\
\x00\x00\x01\x99.H\x8e\xff\
\x00\x00\x02\xc6\x00\x01\x00\x00\x00\x01\x00\x00B$\
\x00\x00\x01\x99.H\x8e\xfe\
\x00\x00\x04\xfc\x00\x00\x00\x00\x00\x01\x00\x00\x87\x9b\
\x00\x00\x01\x99.H\x8e\xfe\
\x00\x00\x03\xaa\x00\x00\x00\x00\x00\x01\x00\x00]v\
\x00\x00\x01\x9b\xc6CC\xeb\
\x00\x00\x07\x8c\x00\x02\x00\x00\x00\x05\x00\x00\x00:\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x07\xb4\x00\x02\x00\x00\x00\x03\x00\x00\x00H\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x07\xa0\x00\x02\x00\x00\x00\x03\x00\x00\x00E\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x07\xf4\x00\x02\x00\x00\x00\x03\x00\x00\x00B\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x07\xc4\x00\x02\x00\x00\x00\x03\x00\x00\x00?\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x07\xd8\x00\x00\x00\x00\x00\x01\x00\x01 ~\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x08\x08\x00\x00\x00\x00\x00\x01\x00\x01#\xbe\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x088\x00\x00\x00\x00\x00\x01\x00\x01%?\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x08\x1e\x00\x00\x00\x00\x00\x01\x00\x01$\xb8\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x08\x08\x00\x00\x00\x00\x00\x01\x00\x01!\x8c\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x088\x00\x00\x00\x00\x00\x01\x00\x01\x22\xf9\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x08\x1e\x00\x00\x00\x00\x00\x01\x00\x01\x22s\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x08\x08\x00\x00\x00\x00\x00\x01\x00\x01'\xb6\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x088\x00\x00\x00\x00\x00\x01\x00\x01(\xf4\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x08\x1e\x00\x00\x00\x00\x00\x01\x00\x01(r\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x08\x08\x00\x00\x00\x00\x00\x01\x00\x01&\x22\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x088\x00\x00\x00\x00\x00\x01\x00\x01'7\
\x00\x00\x01\x975l\xc7\xf0\
\x00\x00\x08\x1e\x00\x00\x00\x00\x00\x01\x00\x01&\xb8\
\x00\x00\x01\x975l\xc7\xf0\
"

def qInitResources():
    QtCore.qRegisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

def qCleanupResources():
    QtCore.qUnregisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

qInitResources()
